#include "info.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "screen.h"
#include "spells.h"


/* extern functions */
char *title_male(int chclass, int level);
char *title_female(int chclass, int level);
int level_exp(int chclass, int level);
int compute_armor_class(struct char_data *ch);
struct time_info_data *real_time_passed(time_t t2, time_t t1);
bitvector_t find_class_bitvector(const char *arg);
bitvector_t find_race_bitvector(const char *arg);

/* extern variables */
extern struct spell_info_type spell_info[];
extern char *class_abbrevs[];
extern char *race_abbrevs[];
extern const char *pc_race_types[];


/* global */
int boot_high = 0;


void diag_char_to_char(struct char_data *i, struct char_data *ch)
{
	struct {
		byte percent;
		const char *text;
	} diagnosis[] = {
		{ 100, "is in excellent condition."			},
		{  90, "has a few scratches."			},
		{  75, "has some small wounds and bruises."		},
		{  50, "has quite a few wounds."			},
		{  30, "has some big nasty wounds and scratches."	},
		{  15, "looks pretty hurt."				},
		{   0, "is in awful condition."			},
		{  -1, "is bleeding awfully from big wounds."	},
	};
	int percent, ar_index;
	const char *pers = PERS(i, ch);

	if (GET_MAX_HIT(i) > 0)
		percent = (100 * GET_HIT(i)) / GET_MAX_HIT(i);
	else
		percent = -1;		/* How could MAX_HIT be < 1?? */

	for (ar_index = 0; diagnosis[ar_index].percent >= 0; ar_index++)
		if (percent >= diagnosis[ar_index].percent)
			break;

	send_to_char(ch, "%c%s %s\r\n",
					 UPPER(*pers), pers + 1, diagnosis[ar_index].text);
}


void look_at_char(struct char_data *i, struct char_data *ch)
{
	int j, found;
	char buf2[MAX_STRING_LENGTH];

	if (!ch->desc)
		return;

   if (i->player.description)
		send_to_char(ch, "%s", i->player.description);
	else
		act("You see nothing special about $m.", FALSE, i, 0, ch, TO_VICT);

	diag_char_to_char(i, ch);

   if (RIDING(i) && IN_ROOM(RIDING(i)) == IN_ROOM(i)) {
		if (RIDING(i) == ch)
			act("$e is mounted on you.", FALSE, i, 0, ch, TO_VICT);
		else {
			snprintf(buf2, sizeof(buf2), "$e is mounted upon %s.", 
						PERS(RIDING(i), ch));
			act(buf2, FALSE, i, 0, ch, TO_VICT);
		}
   } else if (RIDDEN_BY(i) && IN_ROOM(RIDDEN_BY(i)) == IN_ROOM(i)) {
		if (RIDDEN_BY(i) == ch)
			act("You are mounted upon $m.", FALSE, i, 0, ch, TO_VICT);
		else {
			snprintf(buf2, sizeof(buf2), "$e is mounted by %s.", 
						PERS(RIDDEN_BY(i), ch));
			act(buf2, FALSE, i, 0, ch, TO_VICT);
		}
   }
  
	found = FALSE;
	for (j = 0; !found && j < NUM_WEARS; j++)
		if (GET_EQ(i, j) && CAN_SEE_OBJ(ch, GET_EQ(i, j)))
			found = TRUE;

	if (found) {
		send_to_char(ch, "\r\n");	/* act() does capitalization. */
		act("$n is using:", FALSE, i, 0, ch, TO_VICT);
		for (j = 0; j < NUM_WEARS; j++)
			if (GET_EQ(i, j) && CAN_SEE_OBJ(ch, GET_EQ(i, j))) {
				send_to_char(ch, "%s", wear_where[j]);
				show_obj_to_char(GET_EQ(i, j), ch, SHOW_OBJ_SHORT);
			}
	}
	if (ch != i && (IS_THIEF(ch) || GET_LEVEL(ch) >= LVL_IMMORT)) {
		found = FALSE;
		act("\r\nYou attempt to peek at $s inventory:", 
			 FALSE, i, 0, ch, TO_VICT);
		list_obj_to_char(i->carrying, ch, 1, FALSE);
	}
}


const char *positions[] = {
	" is lying here, dead.",
	" is lying here, mortally wounded.",
	" is lying here, incapacitated.",
	" is lying here, stunned.",
	" is sleeping here.",
	" is resting here.",
	" is sitting here.",
	"!FIGHTING!",
	" is standing here.",
	" is flying here.",
};

/* This is basically a copy of list_one_char() (see below) with some
 * minor modifiactions.  -ejg */
void list_all_char(struct char_data * i, struct char_data * ch, int num)
{
	char buf[MAX_STRING_LENGTH] = ""; 
	char buf2[MAX_STRING_LENGTH] = "";
	
	if (IS_NPC(i) && i->player.long_descr && GET_POS(i) == GET_DEFAULT_POS(i)) {
		if (AFF_FLAGGED(i, AFF_INVISIBLE))
			strcat(buf, "*");

		if (AFF_FLAGGED(ch, AFF_DETECT_ALIGN)) {
			if (IS_EVIL(i))
				strcat(buf, "(Red Aura) ");
			else if (IS_GOOD(i))
				strcat(buf, "(Blue Aura) ");
		}
		strcat(buf, i->player.long_descr);

		if (num > 1) {
			while ((buf[strlen(buf)-1] == '\r') ||
					 (buf[strlen(buf)-1] == '\n') ||
					 (buf[strlen(buf)-1] == ' ')) {
				buf[strlen(buf)-1] = '\0';
			}
			snprintf(buf2, sizeof(buf2), " [%d]\n\r", num);
			strcat(buf, buf2);
		}
		send_to_char(ch, buf);

		if (AFF_FLAGGED(i, AFF_SANCTUARY))
			act("...$e glows with a bright light!", FALSE, i, 0, ch, TO_VICT);
		if (AFF_FLAGGED(i, AFF_BLIND))
			act("...$e is groping around blindly!", FALSE, i, 0, ch, TO_VICT);
		if (AFF_FLAGGED(i, AFF_MIRROR_IMAGE))
			act("...$e is surrounded by mirror images.", FALSE, i, 0, ch, TO_VICT);

		return;
	}

	if (IS_NPC(i))
		send_to_char(ch, "%c%s", 
						 UPPER(*i->player.short_descr), i->player.short_descr);
	else
		send_to_char(ch, "%s %s", 
						 i->player.name, GET_TITLE(i));

	if (AFF_FLAGGED(i, AFF_INVISIBLE))
		send_to_char(ch, " (invisible)");
	if (AFF_FLAGGED(i, AFF_HIDE))
		send_to_char(ch, " (hidden)");
	if (!IS_NPC(i)) {
		if (!i->desc)
			send_to_char(ch, " (linkless)");
		else if (EDITING(i->desc))
			send_to_char(ch, " (%s)", edit_modes[EDITING(i->desc)->mode]);
		else if (PRF_FLAGGED(i, PRF_AFK))
			send_to_char(ch, " (%s)", afk_aliases[(GET_CLASS(i))]);
	}

	if (GET_POS(i) != POS_FIGHTING) {
		if (!SITTING(i)) 
			send_to_char(ch, "%s", positions[(int) GET_POS(i)]);
		else {
			struct obj_data *chair = SITTING(i);
			send_to_char(ch, " is %s upon %s.", 
							 ((GET_POS(i) == POS_SITTING) ? "sitting" : "resting"),
							 (CAN_SEE_OBJ(ch, chair) ? 
							  chair->short_description : "something"));
		}
	} else {
		if (FIGHTING(i)) {
			send_to_char(ch, " is here, fighting ");
			if (FIGHTING(i) == ch)
				send_to_char(ch, "YOU!");
			else {
				if (IN_ROOM(i) == IN_ROOM(FIGHTING(i)))
					send_to_char(ch, "%s!", PERS(FIGHTING(i), ch));
				else
					send_to_char(ch,  "someone who has already left!");
			}
		} else			/* NIL fighting pointer */
			send_to_char(ch, " is here struggling with thin air.");
	}

	if (AFF_FLAGGED(ch, AFF_DETECT_ALIGN)) {
		if (IS_EVIL(i))
			send_to_char(ch, " (Red Aura)");
		else if (IS_GOOD(i))
			send_to_char(ch, " (Blue Aura)");
	}
	send_to_char(ch, "\r\n");

	if (AFF_FLAGGED(i, AFF_BLIND))
		act("...$e is groping around blindly!", FALSE, i, 0, ch, TO_VICT);
	if (AFF_FLAGGED(i, AFF_SANCTUARY))
		act("...$e glows with a bright light!", FALSE, i, 0, ch, TO_VICT);
	if (AFF_FLAGGED(i, AFF_MIRROR_IMAGE))
		act("...$e is surrounded by mirror images.", FALSE, i, 0, ch, TO_VICT);
}


void list_one_char(struct char_data *i, struct char_data *ch)
{
	if (IS_NPC(i) && i->player.long_descr && GET_POS(i) == GET_DEFAULT_POS(i)) {
		if (AFF_FLAGGED(i, AFF_INVISIBLE))
			send_to_char(ch, "*");

		if (AFF_FLAGGED(ch, AFF_DETECT_ALIGN)) {
			if (IS_EVIL(i))
				send_to_char(ch, "(Red Aura) ");
			else if (IS_GOOD(i))
				send_to_char(ch, "(Blue Aura) ");
		}
		send_to_char(ch, "%s", i->player.long_descr);

		if (AFF_FLAGGED(i, AFF_SANCTUARY))
			act("...$e glows with a bright light!", FALSE, i, 0, ch, TO_VICT);
		if (AFF_FLAGGED(i, AFF_BLIND))
			act("...$e is groping around blindly!", FALSE, i, 0, ch, TO_VICT);
		if (AFF_FLAGGED(i, AFF_MIRROR_IMAGE))
			act("...$e is surrounded by mirror images.", FALSE, i, 0, ch, TO_VICT);

		return;
	}

	if (IS_NPC(i))
		send_to_char(ch, "%c%s", 
						 UPPER(*i->player.short_descr), i->player.short_descr);
	else
		send_to_char(ch, "%s %s", 
						 i->player.name, GET_TITLE(i));

	if (AFF_FLAGGED(i, AFF_INVISIBLE))
		send_to_char(ch, " (invisible)");
	if (AFF_FLAGGED(i, AFF_HIDE))
		send_to_char(ch, " (hidden)");
	if (!IS_NPC(i)) {
		if (!i->desc)
			send_to_char(ch, " (linkless)");
		else if (EDITING(i->desc))
			send_to_char(ch, " (%s)", edit_modes[EDITING(i->desc)->mode]);
		else if (PRF_FLAGGED(i, PRF_AFK))
			send_to_char(ch, " (%s)", afk_aliases[(GET_CLASS(i))]);
	}

   if (RIDING(i) && IN_ROOM(RIDING(i)) == IN_ROOM(i)) {
		send_to_char(ch, " is here, mounted upon ");
		if (RIDING(i) == ch)
			send_to_char(ch, "you");
		else
			send_to_char(ch, PERS(RIDING(i), ch));
		send_to_char(ch, ". ");
   } else if (GET_POS(i) != POS_FIGHTING)
		send_to_char(ch, "%s", positions[(int) GET_POS(i)]);
	else {
		if (FIGHTING(i)) {
			send_to_char(ch, " is here, fighting ");
			if (FIGHTING(i) == ch)
				send_to_char(ch, "YOU!");
			else {
				if (IN_ROOM(i) == IN_ROOM(FIGHTING(i)))
					send_to_char(ch, "%s!", PERS(FIGHTING(i), ch));
				else
					send_to_char(ch,  "someone who has already left!");
			}
		} else			/* NIL fighting pointer */
			send_to_char(ch, " is here struggling with thin air.");
	}

	if (AFF_FLAGGED(ch, AFF_DETECT_ALIGN)) {
		if (IS_EVIL(i))
			send_to_char(ch, " (Red Aura)");
		else if (IS_GOOD(i))
			send_to_char(ch, " (Blue Aura)");
	}
	send_to_char(ch, "\r\n");

	if (AFF_FLAGGED(i, AFF_BLIND))
		act("...$e is groping around blindly!", FALSE, i, 0, ch, TO_VICT);
	if (AFF_FLAGGED(i, AFF_SANCTUARY))
		act("...$e glows with a bright light!", FALSE, i, 0, ch, TO_VICT);
	if (AFF_FLAGGED(i, AFF_MIRROR_IMAGE))
		act("...$e is surrounded by mirror images.", FALSE, i, 0, ch, TO_VICT);
}


/* New list_char_to_char() function so we can have mob-stacking, just
   like obj-stacking, so instead of:

a cityguard stands here.
a cityguard stands here.
a cityguard stands here.
a cityguard stands here.
a cityguard stands here.

you get:
a cityguard stands here.[4]
 */
void list_char_to_char(struct char_data *list, struct char_data *ch)
{
	struct char_data *i, *plr_list[100];
	int counter, locate_list[100], found = FALSE;
	int num = 0, bits_equal = TRUE, aindex;

	for (i = list; i; i = i->next_in_room) {
      if (i != ch)  {
			if (CAN_SEE(ch, i)) {
            if (num < 100) {
					found = FALSE;
					for (counter = 0; (counter < num&& !found); counter++) {
						bits_equal = TRUE;

						for (aindex = 0; 
							  aindex < sizeof(struct affections); 
							  aindex++) {
							if (((char*)&AFF_FLAGS(i))[aindex] != ((char*)&AFF_FLAGS(plr_list[counter]))[aindex])
								bits_equal = FALSE;
						}
						
						if (i->nr == plr_list[counter]->nr &&

							 /* for odd reasons, you may want to seperate
							  * same nr mobs by factors other than position,
							  * the aff flags, and the fighters. (perhaps you
							  * want to list switched chars differently.. or
							  * polymorphed chars?) */

							 (GET_POS(i)   == GET_POS(plr_list[counter]))   &&
							 bits_equal &&
							 (FIGHTING(i)  == FIGHTING(plr_list[counter]))  &&
							 !strcmp(GET_NAME(i), GET_NAME(plr_list[counter]))) {
							locate_list[counter] += 1;
							found = TRUE;
						}
					}

					if (!found) {
						plr_list[num] = i;
						locate_list[num] = 1;
						num++;
					}
            } else 
					list_all_char(i, ch, 0);
			}
      }
	}

	if (num) {
		for (counter=0; counter<num; counter++) {
			if (locate_list[counter] > 1) 
				list_all_char(plr_list[counter], ch, locate_list[counter]);
			else 
				list_all_char(plr_list[counter], ch, 0);
		}
	}
}

/* Original list_char_to_char() function 
void list_char_to_char(struct char_data *list, struct char_data *ch)
{
	struct char_data *i;

	for (i = list; i; i = i->next_in_room)
		if (ch != i) {
			if (CAN_SEE(ch, i))
				list_one_char(i, ch);
			else if (IS_DARK(IN_ROOM(ch)) && !CAN_SEE_IN_DARK(ch) &&
						AFF_FLAGGED(i, AFF_INFRAVISION))
				send_to_char(ch, "You see a pair of glowing red eyes looking "
								 "your way.\r\n");
		}
}
*/

ACMD(do_score)
{
	struct time_info_data playing_time;

	if (IS_NPC(ch))
		return;

	send_to_char(ch, "You are a %d year old %s.\r\n", GET_AGE(ch),
					 pc_race_types[(int)GET_RACE(ch)]);

	if (age(ch)->month == 0 && age(ch)->day == 0)
		send_to_char(ch, "  It's your birthday today.\r\n");
	else
		send_to_char(ch, "\r\n");

	send_to_char(ch, 
					 "You have %d(%d) hit, %d(%d) mana and %d(%d) "
					 "movement points.\r\n", 
					 GET_HIT(ch), GET_MAX_HIT(ch), GET_MANA(ch), 
					 GET_MAX_MANA(ch), GET_MOVE(ch), GET_MAX_MOVE(ch));

	send_to_char(ch, "Your armor class is %d/10, and your alignment is %d.\r\n",
					 compute_armor_class(ch), GET_ALIGNMENT(ch));

	send_to_char(ch, "You have scored %d exp.\r\n", GET_EXP(ch));

	/* How much money we toting around? */
	send_to_char(ch, "You are carrying");
	if (GET_PLATINUM(ch) <= 0)
		send_to_char(ch, " no platinum coins");
	else
		send_to_char(ch, " %d platinum coins", GET_PLATINUM(ch));

	if (GET_GOLD(ch) <= 0)
		send_to_char(ch, ", no gold coins");
	else
		send_to_char(ch, ", %d gold coins", GET_GOLD(ch));

	if (GET_SILVER(ch) <= 0)
		send_to_char(ch, ", no silver coins");
	else
		send_to_char(ch, ", %d silver coins", GET_SILVER(ch));

	if (GET_COPPER(ch) <= 0)
		send_to_char(ch, " and no copper coins.\r\n");
	else
		send_to_char(ch, " and %d copper coins.\r\n", GET_COPPER(ch));

	if (GET_SHARES(ch) > 0)
		send_to_char(ch, "You have %d shares worth a total of %d gold\r\n",
						 GET_SHARES(ch), GET_SHARES(ch) * share_value);

	if (GET_LEVEL(ch) < LVL_IMMORT)
		send_to_char(ch, "You need %d exp to reach your next level.\r\n",
						 level_exp(GET_CLASS(ch), GET_LEVEL(ch) + 1) - GET_EXP(ch));

	playing_time = *real_time_passed((time(0) - ch->player.time.logon) +
												ch->player.time.played, 0);
	send_to_char(ch, "You have been playing for %d day%s and %d hour%s.\r\n",
					 playing_time.day, playing_time.day == 1 ? "" : "s",
					 playing_time.hours, playing_time.hours == 1 ? "" : "s");

	send_to_char(ch, "This ranks you as %s %s (level %d).\r\n",
					 GET_NAME(ch), GET_TITLE(ch), GET_LEVEL(ch));

	if (RIDING(ch))
		send_to_char(ch, "You are riding %s; ", GET_NAME(RIDING(ch)));
	else if (RIDDEN_BY(ch))
		send_to_char(ch, "You are being ridden by %s; ", 
						 GET_NAME(RIDDEN_BY(ch)));
	
	switch (GET_POS(ch)) {
		case POS_DEAD:
			send_to_char(ch, "You are DEAD!\r\n");
			break;
		case POS_MORTALLYW:
			send_to_char(ch, "You are mortally wounded!  You should seek help!\r\n");
			break;
		case POS_INCAP:
			send_to_char(ch, "You are incapacitated, slowly fading away...\r\n");
			break;
		case POS_STUNNED:
			send_to_char(ch, "You are stunned!  You can't move!\r\n");
			break;
		case POS_SLEEPING:
			send_to_char(ch, "You are sleeping.\r\n");
			break;
		case POS_RESTING:
			send_to_char(ch, "You are resting.\r\n");
			break;
		case POS_SITTING:
			if (!SITTING(ch))
				send_to_char(ch, "You are sitting.\r\n");
			else { 
				struct obj_data *chair = SITTING(ch);
				send_to_char(ch, "You are sitting upon %s.\r\n",
								 chair->short_description);
			}
			break;
		case POS_FIGHTING:
			send_to_char(ch, "You are fighting %s.\r\n", FIGHTING(ch) ? PERS(FIGHTING(ch), ch) : "thin air");
			break;
		case POS_STANDING:
			send_to_char(ch, "You are standing.\r\n");
			break;
		case POS_FLYING:
			send_to_char(ch, "You are flying.\r\n");
			break;
		default:
			send_to_char(ch, "You are floating.\r\n");
			break;
	}

	if (GET_COND(ch, DRUNK) > 10)
		send_to_char(ch, "You are intoxicated.\r\n");

	if (GET_COND(ch, FULL) == 0)
		send_to_char(ch, "You are hungry.\r\n");

	if (GET_COND(ch, THIRST) == 0)
		send_to_char(ch, "You are thirsty.\r\n");

	if (AFF_FLAGGED(ch, AFF_BLIND))
		send_to_char(ch, "You have been blinded!\r\n");

	if (AFF_FLAGGED(ch, AFF_INVISIBLE))
		send_to_char(ch, "You are invisible.\r\n");

	if (AFF_FLAGGED(ch, AFF_DETECT_INVIS))
		send_to_char(ch, "You are sensitive to the presence of invisible things.\r\n");

	if (AFF_FLAGGED(ch, AFF_SENSE_LIFE))
		send_to_char(ch, "You are sensitive to the presence of hidden things.\r\n");

	if (AFF_FLAGGED(ch, AFF_SANCTUARY))
		send_to_char(ch, "You are protected by Sanctuary.\r\n");

	if (AFF_FLAGGED(ch, AFF_AWARE))
		send_to_char(ch, "You have heightened awareness.\r\n");

	if (AFF_FLAGGED(ch, AFF_POISON))
		send_to_char(ch, "You are poisoned!\r\n");

	if (AFF_FLAGGED(ch, AFF_CHARM))
		send_to_char(ch, "You have been charmed!\r\n");

	if (affected_by_spell(ch, SPELL_ARMOR))
		send_to_char(ch, "You feel protected.\r\n");

	if (AFF_FLAGGED(ch, AFF_INFRAVISION))
		send_to_char(ch, "Your eyes are glowing red.\r\n");

	if (PRF_FLAGGED(ch, PRF_SUMMONABLE))
		send_to_char(ch, "You are summonable by other players.\r\n");

	if (AFF_FLAGGED(ch, AFF_MIRROR_IMAGE))
		send_to_char(ch, "You are surrounded by mirror images.\r\n");
}


void perform_mortal_where(struct char_data *ch, char *arg)
{
	struct char_data *i;
	struct descriptor_data *d;

	if (!*arg) {
		send_to_char(ch, "Players in your Zone\r\n--------------------\r\n");
		for (d = descriptor_list; d; d = d->next) {
			if (STATE(d) != CON_PLAYING || d->character == ch)
				continue;
			if ((i = (d->original ? d->original : d->character)) == NULL)
				continue;
			if (IN_ROOM(i) == NOWHERE || !CAN_SEE(ch, i))
				continue;
			if (world[IN_ROOM(ch)].zone != world[IN_ROOM(i)].zone)
				continue;
			send_to_char(ch, "%-20s - %s\r\n", 
							 GET_NAME(i), world[IN_ROOM(i)].name);
		}
	} else {			/* print only FIRST char, not all. */
		for (i = character_list; i; i = i->next) {
			if (IN_ROOM(i) == NOWHERE || i == ch)
				continue;
			if (!CAN_SEE(ch, i) || 
				 world[IN_ROOM(i)].zone != world[IN_ROOM(ch)].zone)
				continue;
			if (!isname(arg, i->player.name))
				continue;
			send_to_char(ch, "%-25s - %s\r\n",
							 GET_NAME(i), world[IN_ROOM(i)].name);
			return;
		}
		send_to_char(ch, "Nobody around by that name.\r\n");
	}
}


void perform_immort_where(struct char_data *ch, char *arg)
{
	struct char_data *i;
	struct obj_data *k;
	struct descriptor_data *d;
	int num = 0, found = 0;

	if (!*arg) {
		send_to_char(ch, "Players\r\n-------\r\n");
		for (d = descriptor_list; d; d = d->next)
			if (STATE(d) == CON_PLAYING) {
				i = (d->original ? d->original : d->character);
				if (i && CAN_SEE(ch, i) && (IN_ROOM(i) != NOWHERE)) {
					if (d->original)
						send_to_char(ch, "%-20s - [%5d] %s (in %s)\r\n",
									GET_NAME(i), GET_ROOM_VNUM(IN_ROOM(d->character)),
									world[IN_ROOM(d->character)].name, 
									GET_NAME(d->character));
					else
						send_to_char(ch, "%-20s - [%5d] %s\r\n", 
									GET_NAME(i), GET_ROOM_VNUM(IN_ROOM(i)), 
									world[IN_ROOM(i)].name);
				}
			}
	} else {
		for (i = character_list; i; i = i->next)
			if (CAN_SEE(ch, i) && 
				 IN_ROOM(i) != NOWHERE &&
				 isname(arg, i->player.name)) {
				found = 1;
				send_to_char(ch, "M%3d. %-25s - [%5d] %s\r\n", ++num, GET_NAME(i),
								 GET_ROOM_VNUM(IN_ROOM(i)), world[IN_ROOM(i)].name);
			}
		for (num = 0, k = object_list; k; k = k->next)
			if (CAN_SEE_OBJ(ch, k) && isname(arg, k->name)) {
				found = 1;
				print_object_location(++num, k, ch, TRUE);
			}
		if (!found)
			send_to_char(ch, "Couldn't find any such thing.\r\n");
	}
}



ACMD(do_where)
{
	char arg[MAX_INPUT_LENGTH];

	one_argument(argument, arg);

	if (GET_LEVEL(ch) >= LVL_IMMORT)
		perform_immort_where(ch, arg);
	else
		perform_mortal_where(ch, arg);
}



ACMD(do_levels)
{
	char buf[MAX_STRING_LENGTH];
	size_t i, len = 0, nlen;

	if (IS_NPC(ch)) {
		send_to_char(ch, "You ain't nothin' but a hound-dog.\r\n");
		return;
	}

	for (i = 1; i < LVL_IMMORT; i++) {
		nlen = snprintf(buf + len, sizeof(buf) - len, 
							 "[%2d] %8d-%-8d : ", i,
							 level_exp(GET_CLASS(ch), i), 
							 level_exp(GET_CLASS(ch), i + 1) - 1);
		if (len + nlen >= sizeof(buf) || nlen < 0)
			break;
		len += nlen;

		switch (GET_SEX(ch)) {
			case SEX_MALE:
			case SEX_NEUTRAL:
				nlen = snprintf(buf + len, sizeof(buf) - len, 
									 "%s\r\n", 
									 title_male(GET_CLASS(ch), i));
				break;
			case SEX_FEMALE:
				nlen = snprintf(buf + len, sizeof(buf) - len, 
									 "%s\r\n", 
									 title_female(GET_CLASS(ch), i));
				break;
			default:
				nlen = snprintf(buf + len, sizeof(buf) - len,
									 "Oh dear.  You seem to be sexless.\r\n");
				break;
		}
		if (len + nlen >= sizeof(buf) || nlen < 0)
			break;
		len += nlen;
	}

	if (len < sizeof(buf))
		snprintf(buf + len, sizeof(buf) - len,
					"[%2d] %8d          : Immortality\r\n",
					LVL_IMMORT, level_exp(GET_CLASS(ch), LVL_IMMORT));
	page_string(ch->desc, buf, TRUE);
}



ACMD(do_consider)
{
	char buf[MAX_INPUT_LENGTH];
	struct char_data *victim;
	int diff;

	one_argument(argument, buf);

	if (!(victim = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM))) {
		send_to_char(ch, "Consider killing who?\r\n");
		return;
	}
	if (victim == ch) {
		send_to_char(ch, "Easy!  Very easy indeed!\r\n");
		return;
	}
	if (!IS_NPC(victim)) {
		send_to_char(ch, "Would you like to borrow a cross and a shovel?\r\n");
		return;
	}
	diff = (GET_LEVEL(victim) - GET_LEVEL(ch));

	if (diff <= -10)
		send_to_char(ch, "Now where did that chicken go?\r\n");
	else if (diff <= -5)
		send_to_char(ch, "You could do it with a needle!\r\n");
	else if (diff <= -2)
		send_to_char(ch, "Easy.\r\n");
	else if (diff <= -1)
		send_to_char(ch, "Fairly easy.\r\n");
	else if (diff == 0)
		send_to_char(ch, "The perfect match!\r\n");
	else if (diff <= 1)
		send_to_char(ch, "You would need some luck!\r\n");
	else if (diff <= 2)
		send_to_char(ch, "You would need a lot of luck!\r\n");
	else if (diff <= 3)
		send_to_char(ch, "You would need a lot of luck and great equipment!\r\n");
	else if (diff <= 5)
		send_to_char(ch, "Do you feel lucky, punk?\r\n");
	else if (diff <= 10)
		send_to_char(ch, "Are you mad!?\r\n");
	else if (diff <= 100)
		send_to_char(ch, "You ARE mad!\r\n");

	if (AFF_FLAGGED(victim, AFF_AWARE))
		send_to_char(victim, "%s is considering to KILL you!\r\n", GET_NAME(ch));
}



ACMD(do_diagnose)
{
	char buf[MAX_INPUT_LENGTH];
	struct char_data *vict;

	one_argument(argument, buf);

	if (*buf) {
		if (!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM)))
			send_to_char(ch, "%s", NOPERSON);
		else
			diag_char_to_char(vict, ch);
	} else {
		if (FIGHTING(ch))
			diag_char_to_char(FIGHTING(ch), ch);
		else
			send_to_char(ch, "Diagnose who?\r\n");
	}
}


ACMD(do_whois)
{
   const char *immlevels[LVL_IMPL - (LVL_IMMORT-1)] = {
		"[Immortal]",	   /* highest mortal level +1 */
		"[God]",		      /* highest mortal level +2 */
		"[Greater God]",	/* highest mortal level +3 */
		"[Implementor]",	/* highest mortal level +4 */
	};

	char buf[MAX_STRING_LENGTH];
   struct char_data *victim = NULL;
   skip_spaces(&argument);

   if (!*argument) {
		send_to_char(ch, "Who?\r\n");
	} else {
		CREATE(victim, struct char_data, 1);
		clear_char(victim);
		CREATE(victim->player_specials, struct player_special_data, 1);
		if (load_char(argument, victim) >= 0) {
			*buf = '\0';

			if (GET_LEVEL(victim) >= LVL_IMMORT)
				sprintf(buf + strlen(buf), "%s %s %s\r\n",
						  immlevels[GET_LEVEL(victim)-LVL_IMMORT], 
						  GET_NAME(victim), GET_TITLE(victim));

			else if (GET_LEVEL(victim) < LVL_IMMORT)
				sprintf(buf + strlen(buf), "Level %d %s %s - %s %s\r\n",
						  GET_LEVEL(victim), RACE_ABBR(victim), CLASS_ABBR(victim), 
						  GET_NAME(victim), GET_TITLE(victim));

			send_to_char(ch, buf);
		} else
			send_to_char(ch, "There is no such player.\r\n");

		free_char(victim); 
	}
}


/*********************************************************************
 * New 'do_who' by Daniel Koepke [aka., "Argyle Macleod"] of The Keep *
 ******************************************************************* */
 
char *WHO_USAGE =
"Usage: who [minlev[-maxlev]] [-n name] [-c classes] [-d races] [-rzqimo]\r\n"
"\r\n"
"Classes: (M)age, (C)leric, (T)hief, (W)arrior\r\n"
"\r\n"
"Races: (H)uman, (D)warf, (E)lf, (G)nome\r\n"
"\r\n"
" Switches: \r\n"
"_.,-'^'-,._\r\n"
"\r\n"
"  -r = who is in the current room\r\n"
"  -z = who is in the current zone\r\n"
"\r\n"
"  -q = only show questers\r\n"
"  -i = only show immortals\r\n"
"  -m = only show mortals\r\n"
"  -o = only show outlaws\r\n"
"\r\n";
  

/* FIXME: This whole thing just needs rewritten. */
ACMD(do_who)
{
	struct descriptor_data *d;
   struct char_data *wch;
   char Imm_buf[MAX_STRING_LENGTH];
   char Mort_buf[MAX_STRING_LENGTH];
   char name_search[MAX_NAME_LENGTH+1];
	char buf[MAX_STRING_LENGTH];
	char mode;
   
   int low = 0, high = LVL_IMPL, showclass = 0, showrace = 0;
   int who_room = FALSE, who_zone = FALSE, who_quest = 0;
   int outlaws = FALSE, noimm = FALSE, nomort = FALSE;
   
   int Wizards = 0, Mortals = 0;

   const char *WizLevels[LVL_IMPL - (LVL_IMMORT-1)] = {
		"   Immortal    ",
		"      God      ",
		"  Greater God  ",
		"* Implementor *"
   };
   
	skip_spaces(&argument);
	strcpy(buf, argument);	/* strcpy: OK (sizeof: argument == buf) */
	name_search[0] = '\0';

   /* the below is from stock CircleMUD -- found no reason to rewrite it */
	while (*buf) {
		char arg[MAX_INPUT_LENGTH], buf1[MAX_INPUT_LENGTH];

		half_chop(buf, arg, buf1);
		if (isdigit(*arg)) {
			sscanf(arg, "%d-%d", &low, &high);
			strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
		} else if (*arg == '-') {
			mode = *(arg + 1);   /* just in case; we destroy arg in the switch */
			switch (mode) {
				case 'o':
					outlaws = TRUE;
					strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
					break;
				case 'z':
					who_zone = TRUE;
					strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
					break;
				case 'q':
					who_quest = TRUE;
					strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
					break;
				case 'l':
					half_chop(buf1, arg, buf);
					sscanf(arg, "%d-%d", &low, &high);
					break;
				case 'n':
					half_chop(buf1, name_search, buf);
					break;
				case 'r':
					who_room = TRUE;
					strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
					break;
				case 'c':
					half_chop(buf1, arg, buf);
					showclass = find_class_bitvector(arg);
					break;
				case 'd':
					half_chop(buf1, arg, buf);
					showrace = find_race_bitvector(arg);
					break;
				case 'i':
					nomort = TRUE;
					strcpy(buf, buf1);
					break;
				case 'm':
					noimm = TRUE;
					strcpy(buf, buf1);
					break;
				default:
					send_to_char(ch, "%s", WHO_USAGE);
					return;
			}				/* end of switch */

		} else {			/* endif */
			send_to_char(ch, "%s", WHO_USAGE);
			return;
		}
	}				/* end while (parser) */

   strcpy(Imm_buf, 
			 "Immortals currently online\r\n--------------------------\r\n");
   strcpy(Mort_buf, 
			 "Mortals currently online\r\n------------------------\r\n");

	for (d = descriptor_list; d; d = d->next) {
		if (STATE(d) != CON_PLAYING)
			continue;

		if (d->original)
			wch = d->original;
		else if (!(wch = d->character))
			continue;

		if (!CAN_SEE(ch, wch))
			continue;
		if (GET_LEVEL(wch) < low || GET_LEVEL(wch) > high)
			continue;
		if ((noimm && GET_LEVEL(wch) >= LVL_IMMORT) || 
			 (nomort && GET_LEVEL(wch) < LVL_IMMORT))
			continue;
		if (*name_search && str_cmp(GET_NAME(wch), name_search) && 
			 !strstr(GET_TITLE(wch), name_search))
			continue;
		if (outlaws && !PLR_FLAGGED(wch, PLR_KILLER) && 
			 !PLR_FLAGGED(wch, PLR_THIEF))
			continue;
		if (who_quest && !PRF_FLAGGED(wch, PRF_QUEST))
			continue;
		if (who_zone && IN_ZONE(ch) != IN_ZONE(wch))
			continue;
		if (who_room && (IN_ROOM(wch) != IN_ROOM(ch)))
			continue;
		if (showclass && !(showclass & (1 << GET_CLASS(wch))))
			continue;
		if (showrace && !(showrace & (1 << GET_RACE(wch))))
			continue;

		if (GET_LEVEL(wch) >= LVL_IMMORT) {
			sprintf(Imm_buf + strlen(Imm_buf), "%s[%s] %s %s", QYEL, 
						WizLevels[GET_LEVEL(wch)-LVL_IMMORT], GET_NAME(wch), 
						GET_TITLE(wch));
			Wizards++;
		} else {
			sprintf(Mort_buf + strlen(Mort_buf), "[%2d %s %s] %s %s", 
					  GET_LEVEL(wch), RACE_ABBR(wch), CLASS_ABBR(wch), 
					  GET_NAME(wch), GET_TITLE(wch));
			Mortals++;
		}
 
		*buf = '\0'; /* **BUG FIX: Revision 2** */
		if (GET_INVIS_LEV(wch))
			sprintf(buf + strlen(buf), " (i%d)", GET_INVIS_LEV(wch));
		else if (AFF_FLAGGED(wch, AFF_INVISIBLE))
			strcat(buf, " (invis)");
 
		if (EDITING(d))
			send_to_char(ch, " (%s)", edit_modes[EDITING(d)->mode]);
 
		if (PRF_FLAGGED(wch, PRF_DEAF))
			strcat(buf, " (deaf)");
		if (PRF_FLAGGED(wch, PRF_NOTELL))
			strcat(buf, " (notell)");
		if (PRF_FLAGGED(wch, PRF_QUEST))
			strcat(buf, " (quest)");
		if (PLR_FLAGGED(wch, PLR_THIEF))
			strcat(buf, " (THIEF)");
		if (PLR_FLAGGED(wch, PLR_KILLER))
			strcat(buf, " (KILLER)");
		if (GET_LEVEL(wch) >= LVL_IMMORT)
			strcat(buf, QNRM);
		if (PRF_FLAGGED(wch, PRF_AFK))
			send_to_char(ch, " (%s)", afk_aliases[(GET_CLASS(wch))]);
		strcat(buf, "\r\n");
     
		if (GET_LEVEL(wch) >= LVL_IMMORT)
			strcat(Imm_buf, buf);
		else
			strcat(Mort_buf, buf);
	}				/* end of for */
 
	if (Wizards) {
		page_string(ch->desc, Imm_buf, 0);
		send_to_char(ch, "\r\n");
	}
   
	if (Mortals) {
		page_string(ch->desc, Mort_buf, 0);
		send_to_char(ch, "\r\n");
	}
   
	if ((Wizards + Mortals) == 0)
		strcpy(buf, "No wizards or mortals are currently visible to you.\r\n");

	if (Wizards)
		sprintf(buf, "There %s %d visible immortal%s%s", 
				  (Wizards == 1 ? "is" : "are"), Wizards, 
				  (Wizards == 1 ? "" : "s"), 
				  (Mortals ? " and there" : "."));

	if (Mortals)
		sprintf(buf + strlen(buf), "%s %s %d visible mortal%s.", 
				  (Wizards ? "" : "There"), 
				  (Mortals == 1 ? "is" : "are"), Mortals, 
				  (Mortals == 1 ? "" : "s"));
	strcat(buf, "\r\n");
   
	if ((Wizards + Mortals) > boot_high)
		boot_high = Wizards+Mortals;
	sprintf(buf + strlen(buf), "There is a boot time high of %d "
			  "player%s.\r\n", boot_high, (boot_high == 1 ? "" : "s"));

	send_to_char(ch, buf);
}


#define USERS_FORMAT \
"format: users [-l minlevel[-maxlevel]] [-n name] [-h host] [-c classlist] [-r racelist] [-o] [-p]\r\n"

/* BIG OL' FIXME: Rewrite it all. Similar to do_who(). */
ACMD(do_users)
{
	char line[200], line2[220], idletime[10], classname[20];
	char state[30], *timeptr, mode;
	char name_search[MAX_INPUT_LENGTH], host_search[MAX_INPUT_LENGTH];
	struct char_data *tch;
	struct descriptor_data *d;
	int low = 0, high = LVL_IMPL, num_can_see = 0, len = 0;
	int showrace = 0, showclass = 0, outlaws = 0, playing = 0, deadweight = 0;
	char buf[MAX_INPUT_LENGTH], arg[MAX_INPUT_LENGTH];

	host_search[0] = name_search[0] = '\0';

	strcpy(buf, argument);	/* strcpy: OK (sizeof: argument == buf) */
	while (*buf) {
		char buf1[MAX_INPUT_LENGTH];

		half_chop(buf, arg, buf1);
		if (*arg == '-') {
			mode = *(arg + 1);  /* just in case; we destroy arg in the switch */
			switch (mode) {
				case 'o':
				case 'k':
					outlaws = 1;
					playing = 1;
					strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
					break;
				case 'p':
					playing = 1;
					strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
					break;
				case 'd':
					deadweight = 1;
					strcpy(buf, buf1);	/* strcpy: OK (sizeof: buf1 == buf) */
					break;
				case 'l':
					playing = 1;
					half_chop(buf1, arg, buf);
					sscanf(arg, "%d-%d", &low, &high);
					break;
				case 'n':
					playing = 1;
					half_chop(buf1, name_search, buf);
					break;
				case 'h':
					playing = 1;
					half_chop(buf1, host_search, buf);
					break;
				case 'c':
					playing = 1;
					half_chop(buf1, arg, buf);
					showclass = find_class_bitvector(arg);
					break;
				case 'r':
					playing = 1;
					half_chop(buf1, arg, buf);
					showrace = find_race_bitvector(arg);
					break;
				default:
					send_to_char(ch, "%s", USERS_FORMAT);
					return;
			}				/* end of switch */

		} else {			/* endif */
			send_to_char(ch, "%s", USERS_FORMAT);
			return;
		}
	}				/* end while (parser) */
	send_to_char(ch,
					 "Num Class       Name         State          Idl Login@   Site\r\n"
					 "--- ----------- ------------ -------------- --- -------- -----------------------\r\n");

	one_argument(argument, arg);

	for (d = descriptor_list; d; d = d->next) {
		if (STATE(d) != CON_PLAYING && playing)
			continue;
		if (STATE(d) == CON_PLAYING && deadweight)
			continue;
		if (STATE(d) == CON_PLAYING) {
			if (d->original)
				tch = d->original;
			else if (!(tch = d->character))
				continue;

			if (*host_search && !strstr(d->host, host_search))
				continue;
			if (*name_search && str_cmp(GET_NAME(tch), name_search))
				continue;
			if (!CAN_SEE(ch, tch) || GET_LEVEL(tch) < low || GET_LEVEL(tch) > high)
				continue;
			if (outlaws && !PLR_FLAGGED(tch, PLR_KILLER) &&
				 !PLR_FLAGGED(tch, PLR_THIEF))
				continue;
			if (showclass && !(showclass & (1 << GET_CLASS(tch))))
				continue;
			if (showrace && !(showrace & (1 << GET_RACE(tch))))
				continue;
			if (GET_INVIS_LEV(ch) > GET_LEVEL(ch))
				continue;

			if (d->original)
				snprintf(classname, sizeof(classname), "[%2d %s %s]", 
							GET_LEVEL(d->original), RACE_ABBR(d->original),
							CLASS_ABBR(d->original));
			else
				snprintf(classname, sizeof(classname), "[%2d %s %s]",
							GET_LEVEL(d->character), RACE_ABBR(d->character), 
							CLASS_ABBR(d->character));
		} else
			strcpy(classname, "   -   ");

		timeptr = asctime(localtime(&d->login_time));
		timeptr += 11;
		*(timeptr + 8) = '\0';

		if (STATE(d) == CON_PLAYING && d->original)
			strcpy(state, "Switched");
		else
			strcpy(state, connected_types[STATE(d)]);

		if (d->character && 
			 STATE(d) == CON_PLAYING && 
			 GET_LEVEL(d->character) < LVL_GOD)
			snprintf(idletime, sizeof(idletime), "%3d",
						d->character->char_specials.timer *
						SECS_PER_MUD_HOUR / SECS_PER_REAL_MIN);
		else
			strcpy(idletime, "");

		sprintf(line, "%3d %-7s %-12s %-14s %-3s %-8s ", 
					d->desc_num, classname,
							d->original && d->original->player.name ?
					d->original->player.name :
							d->character && d->character->player.name ?
					d->character->player.name :
					"UNDEFINED", state, idletime, timeptr);

		if (d->host && *d->host)
			sprintf(line + strlen(line), "[%s]\r\n", d->host);
		else
			sprintf(line, "%s", "[Hostname unknown]\r\n");

		if (STATE(d) != CON_PLAYING) {
			sprintf(line2, "%s%s%s", QGRN, line, QNRM);
			strcpy(line, line2);
		}

		if (STATE(d) != CON_PLAYING ||
			 (STATE(d) == CON_PLAYING && CAN_SEE(ch, d->character))) {
			send_to_char(ch, "%s", line);
			num_can_see++;
		}
	} /* for (d = descriptor_list; d; d = d->next) { */

	send_to_char(ch, "\r\n%d visible sockets connected.\r\n", num_can_see);
}


ACMD(do_attributes)
{
	send_to_char(ch, "                 Character attributes for %s\r\n", 
					 GET_NAME(ch));
	send_to_char(ch, "\r\nLevel: %d   Race: %s   Class: %s\r\n", 
					 GET_LEVEL(ch), RACE_ABBR(ch), CLASS_ABBR(ch));
	send_to_char(ch, "Age: %d yrs / %d mths  Height: %d cm  "
					 "Weight: %d lbs\r\n", age(ch)->year, age(ch)->month, 
					 GET_HEIGHT(ch), GET_WEIGHT(ch));

	if (IS_WARRIOR(ch) && GET_STR(ch) == 18)
		send_to_char(ch, "Str: %d/%3d   Int: %d       Wis: %d\r\nDex: %d       Con: %d       Cha: %d\r\n",
						 GET_STR(ch), GET_ADD(ch), GET_INT(ch), GET_WIS(ch),
						 GET_DEX(ch), GET_CON(ch), GET_CHA(ch));
	else
		send_to_char(ch, "Str: %d       Int: %d       Wis: %d\r\nDex: %d       Con: %d       Cha: %d\r\n",
						 GET_STR(ch), GET_INT(ch), GET_WIS(ch), GET_DEX(ch),
						 GET_CON(ch), GET_CHA(ch));
	
	send_to_char(ch, "Armor Class: %d\r\n", GET_AC(ch));
	send_to_char(ch, "Hitroll: %d   Damroll: %d\r\n",
					 (ch)->points.hitroll, (ch)->points.damroll);
	send_to_char(ch, "Alignment: %d\r\n", GET_ALIGNMENT(ch));
	send_to_char(ch, "Saving Throws: PAR[%d]  ROD[%d]  PET[%d]  BRE[%d] SPE[%d]\r\n",
					 GET_SAVE(ch, 0), GET_SAVE(ch, 1), GET_SAVE(ch, 2), 
					 GET_SAVE(ch, 3), GET_SAVE(ch, 4));
}


/* Affects screen for spell affects {Jon Barrett, 
 *	mizzrym@hotmail.com, 26APR98} */
ACMD(do_affects)
{
	struct affected_type* af;
	int type = 1, different = FALSE, index;
	char sname[256];
	char added_aff[MAX_STRING_LENGTH];

	send_to_char(ch, "You carry these affections: \r\n");

	for (af = ch->affected; af; af = af->next) {
		strcpy(sname, spell_info[af->type].name);
		strcat(sname, ":");

		if (GET_LEVEL(ch) >= 7) {  /* first state is here */
			send_to_char(ch, "   %s%-22s%s    affects %s%s%s by %s%d%s for %s%d%s hours\r\n", 
					  CCCYN(ch, C_NRM),	(type ? sname : ""), CCNRM(ch, C_NRM),
					  CCCYN(ch, C_NRM),
							 ((GET_LEVEL(ch) >=14 && af->location) ? apply_types[af->location] : "Something"),  /* second state is here */
					  CCNRM(ch, C_NRM),
					  CCCYN(ch, C_NRM), af->modifier, CCNRM(ch, C_NRM),
					  CCCYN(ch, C_NRM), af->duration, CCNRM(ch, C_NRM));			

			different = FALSE;
			for (index = 0; index < sizeof(af->bitvector); index++) {
				if (((char*)&af->next->bitvector)[index] != (((char*)&af->bitvector)[index]))
					different = TRUE;
			}
			
			if (GET_LEVEL(ch) >= 21 &&
				 (!ISEMPTY(af->bitvector) && (!af->next || different))) {

				/* third state is here */
				sprintfield((char*)&af->bitvector, affected_bits, added_aff, sizeof(added_aff));
				send_to_char(ch, "%35sadds %s%s\r\n", CCCYN(ch, C_NRM), added_aff, CCNRM(ch, C_NRM));
			}
		} else if (type) {
			send_to_char(ch, "   %s%-25s%s\r\n", CCCYN(ch, C_NRM), sname, CCNRM(ch, C_NRM));
		}
		type = af->next ? (af->next->type != af->type) : 1;
	}
}


char *howgood(int percent)
{
	if (percent == 0)
		return (" (not learned)");
	else if (percent <= 10)
		return(" (awful)");
	else if (percent <= 20)
		return(" (bad)");
	else if (percent <= 40)
		return(" (poor)");
	else if (percent <= 55)
		return(" (average)");
	else if (percent <= 70)
		return(" (fair)");
	else if (percent <= 80)
		return(" (good)");
	else if (percent <= 85)
		return(" (very good)");
	else if (percent <= 99)
		return(" (superb)");
	else
		return(" (SUPERB)");
}

ACMD(do_spells)
{
	char buf[MAX_STRING_LENGTH] = "";
	char buf2[MAX_STRING_LENGTH] = "";
	int i, len = 0;

	strcpy(buf2, "You know of the following Spells:\r\n");

	for (i = 1; i <= MAX_SPELLS; i++) {
		if (GET_LEVEL(ch) >= spell_info[i].min_level[(int) GET_CLASS(ch)]) {
			if (GET_SKILL(ch, i) > 0) {
				len += snprintf(buf, sizeof(buf), "%-20s %s\r\n", 
						  spell_info[i].name, howgood(GET_SKILL(ch, i)));

				if (len >= sizeof(buf2)) /* Avoid buffer overflows.  -ejg */
					break;
				
				strcat(buf2, buf);
			}
		}
	}
	page_string(ch->desc, buf2, 1);
}


ACMD(do_skills)
{
	char buf[MAX_STRING_LENGTH] = "";
	char buf2[MAX_STRING_LENGTH] = "";
	int i, len = 0;

	strcpy(buf2, "You know of the following Skills:\r\n");

	for (i = MAX_SPELLS+1; i <= MAX_SKILLS; i++) {
		if (GET_LEVEL(ch) >= spell_info[i].min_level[(int) GET_CLASS(ch)]) {
			if (GET_SKILL(ch, i) > 0) {
				len += snprintf(buf, sizeof(buf), "%-20s %s\r\n", 
									 spell_info[i].name, howgood(GET_SKILL(ch, i)));

				if (len >= sizeof(buf2)) /* Avoid buffer overflows.  -ejg */
					break;
				
				strcat(buf2, buf);
			}
		}
	}
	page_string(ch->desc, buf2, 1);
}

