#include "info.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"


void show_obj_to_char(struct obj_data *obj, struct char_data *ch, int mode)
{
	int found;
	struct char_data *temp;

	if (!obj || !ch) {
		log("SYSERR: NULL pointer in show_obj_to_char(): obj=%p ch=%p", obj, ch);
		return;
	}

	switch (mode) {
		case SHOW_OBJ_LONG:
			if (GET_OBJ_VAL(obj, 1) == 0 || !OBJ_SAT_IN_BY(obj))
				send_to_char(ch, "%s", obj->description);
			else {
				temp = OBJ_SAT_IN_BY(obj);
				for (temp = OBJ_SAT_IN_BY(obj); temp; temp = NEXT_SITTING(temp)) {
					if (temp == ch)
						found = 1;
				}
				if (found == 1)
					send_to_char(ch, "You are %s upon %s.", 
									 GET_POS(ch) == POS_SITTING ? "sitting" : "resting",
									 obj->short_description);
				else
					send_to_char(ch, "%s", obj->description);      
			}
			break;

		case SHOW_OBJ_SHORT:
			send_to_char(ch, "%s", obj->short_description);
			break;

		case SHOW_OBJ_ACTION:
			switch (GET_OBJ_TYPE(obj)) {
				case ITEM_NOTE:
					if (obj->action_description) {
						char notebuf[MAX_NOTE_LENGTH + 64];

						snprintf(notebuf, sizeof(notebuf), "There is something written on it:\r\n\r\n%s", obj->action_description);
						page_string(ch->desc, notebuf, TRUE);
					} else
						send_to_char(ch, "It's blank.\r\n");
					return;

				case ITEM_DRINKCON:
					send_to_char(ch, "It looks like a drink container.");
					break;

				default:
					send_to_char(ch, "You see nothing special..");
					break;
			}
			break;

		default:
			log("SYSERR: Bad display mode (%d) in show_obj_to_char().", mode);
			return;
	}

	show_obj_modifiers(obj, ch);
	send_to_char(ch, "\r\n");
}


void show_obj_modifiers(struct obj_data *obj, struct char_data *ch)
{
	if (OBJ_FLAGGED(obj, ITEM_INVISIBLE))
		send_to_char(ch, " (invisible)");

	if (OBJ_FLAGGED(obj, ITEM_BURIED))
		send_to_char(ch, " (buried)");

	if (OBJ_FLAGGED(obj, ITEM_BLESS) && AFF_FLAGGED(ch, AFF_DETECT_ALIGN))
		send_to_char(ch, " ..It glows blue!");

	if (OBJ_FLAGGED(obj, ITEM_MAGIC) && AFF_FLAGGED(ch, AFF_DETECT_MAGIC))
		send_to_char(ch, " ..It glows yellow!");

	if (OBJ_FLAGGED(obj, ITEM_GLOW))
		send_to_char(ch, " ..It has a soft glowing aura!");

	if (OBJ_FLAGGED(obj, ITEM_HUM))
		send_to_char(ch, " ..It emits a faint humming sound!");
}


void list_obj_to_char(struct obj_data *list, struct char_data *ch, int mode, int show)
{
	struct obj_data *i, *j;
	int num, found = FALSE;

	for (i = list; i; i = i->next_content) {
      num = 0;
      for (j = list; j != i; j = j->next_content) {
			if (j->item_number == NOTHING) {
				if(strcmp(j->short_description, i->short_description) == 0)
					break;
			} else if (j->item_number == i->item_number)
				break;
		}
		
      if (j != i)
			continue;

      for (j = i; j; j = j->next_content)
		{
			if (j->item_number == NOTHING) {
				if(strcmp(j->short_description, i->short_description) == 0)
					num++;
			} else if (j->item_number == i->item_number)
				num++;
		}
		
      if (CAN_SEE_OBJ(ch, i) || (GET_OBJ_TYPE(i) == ITEM_LIGHT)) {
			if (num != 1)
				send_to_char(ch, "(%2i) ", num);

			show_obj_to_char(i, ch, mode);
			found = TRUE;
      }
	}
	if (!found && show)
		send_to_char(ch, " Nothing.\r\n");
}

#if 0
/* Original list_obj_to_char().  It has been replaced by the above one
 * that stacks objects that are the same. */
void list_obj_to_char(struct obj_data *list, struct char_data *ch, int mode, int show)
{
	struct obj_data *i;
	bool found = FALSE;

	for (i = list; i; i = i->next_content) {
		if (CAN_SEE_OBJ(ch, i)) {
			show_obj_to_char(i, ch, mode);
			found = TRUE;
		}
	}
	if (!found && show)
		send_to_char(ch, " Nothing.\r\n");
}
#endif


void look_in_obj(struct char_data *ch, char *arg)
{
	struct obj_data *obj = NULL;
	struct char_data *dummy = NULL;
	int amt, bits;

	if (!*arg)
		send_to_char(ch, "Look in what?\r\n");
	else if (!(bits = generic_find(arg, FIND_OBJ_INV | FIND_OBJ_ROOM |
											 FIND_OBJ_EQUIP, ch, &dummy, &obj))) {
		send_to_char(ch, "There doesn't seem to be %s %s here.\r\n", AN(arg), arg);
	} else if ((GET_OBJ_TYPE(obj) != ITEM_DRINKCON) &&
				  (GET_OBJ_TYPE(obj) != ITEM_FOUNTAIN) &&
				  (GET_OBJ_TYPE(obj) != ITEM_CONTAINER))
		send_to_char(ch, "There's nothing inside that!\r\n");
	else {
		if (GET_OBJ_TYPE(obj) == ITEM_CONTAINER) {
			if (OBJVAL_FLAGGED(obj, CONT_CLOSED))
				send_to_char(ch, "It is closed.\r\n");
			else {
				send_to_char(ch, "%s", fname(obj->name));
				switch (bits) {
					case FIND_OBJ_INV:
						send_to_char(ch, " (carried): \r\n");
						break;
					case FIND_OBJ_ROOM:
						send_to_char(ch, " (here): \r\n");
						break;
					case FIND_OBJ_EQUIP:
						send_to_char(ch, " (used): \r\n");
						break;
				}

				list_obj_to_char(obj->contains, ch, SHOW_OBJ_SHORT, TRUE);
			}
		} else {		/* item must be a fountain or drink container */
			if (GET_OBJ_VAL(obj, 1) <= 0)
				send_to_char(ch, "It is empty.\r\n");
			else {
				if (GET_OBJ_VAL(obj,0) <= 0 || GET_OBJ_VAL(obj,1)>GET_OBJ_VAL(obj,0)) {
					send_to_char(ch, "Its contents seem somewhat murky.\r\n"); /* BUG */
				} else {
					char buf2[MAX_STRING_LENGTH];
					amt = (GET_OBJ_VAL(obj, 1) * 3) / GET_OBJ_VAL(obj, 0);
					sprinttype(GET_OBJ_VAL(obj, 2), color_liquid, buf2, sizeof(buf2));
					send_to_char(ch, "It's %sfull of a %s liquid.\r\n", fullness[amt], buf2);
				}
			}
		}
	}
}



char *find_exdesc(char *word, struct extra_descr_data *list)
{
	struct extra_descr_data *i;

	for (i = list; i; i = i->next)
		if (isname(word, i->keyword))
			return (i->description);

	return (NULL);
}


ACMD(do_examine)
{
	struct char_data *tmp_char;
	struct obj_data *tmp_object;
	char tempsave[MAX_INPUT_LENGTH], arg[MAX_INPUT_LENGTH];

	one_argument(argument, arg);

	if (!*arg) {
		send_to_char(ch, "Examine what?\r\n");
		return;
	}

	/* look_at_target() eats the number. */
	look_at_target(ch, strcpy(tempsave, arg));	/* strcpy: OK */

	generic_find(arg, FIND_OBJ_INV | FIND_OBJ_ROOM | FIND_CHAR_ROOM |
					 FIND_OBJ_EQUIP, ch, &tmp_char, &tmp_object);

	if (tmp_object) {
		if ((GET_OBJ_TYPE(tmp_object) == ITEM_DRINKCON) ||
			 (GET_OBJ_TYPE(tmp_object) == ITEM_FOUNTAIN) ||
			 (GET_OBJ_TYPE(tmp_object) == ITEM_CONTAINER)) {
			send_to_char(ch, "When you look inside, you see:\r\n");
			look_in_obj(ch, arg);
		}
	}
}



ACMD(do_gold)
{
	int i, amount;

	amount = (GET_PLATINUM(ch) + GET_GOLD(ch) + GET_SILVER(ch) + GET_COPPER(ch));
	
	if (amount == 0)
		send_to_char(ch, "You're broke!\r\n");
	else if (amount == 1)
		send_to_char(ch, "You have one miserable little coin.\r\n");
	else {
		send_to_char(ch, "You have");
		for (i = 4; i > 0 ; i--)
			send_to_char(ch, "%s%d %s",
							 (((i-1) == PLAT_COINS) ? "" : 
							  (((i-1) == COPPER_COINS) ? ", " : " and ")),
							 GET_MONEY(ch, i-1), coin_types[i-1]);
		send_to_char(ch, " coins.\r\n");
	}
}


ACMD(do_inventory)
{
	send_to_char(ch, "You are carrying:\r\n");
	list_obj_to_char(ch->carrying, ch, SHOW_OBJ_SHORT, TRUE);
}


ACMD(do_equipment)
{
	int i, found = 0;

	send_to_char(ch, "You are using:\r\n");
	for (i = 0; i < NUM_WEARS; i++) {
		if (GET_EQ(ch, i)) {
			if (CAN_SEE_OBJ(ch, GET_EQ(ch, i))) {
				send_to_char(ch, "%s", wear_where[i]);
				show_obj_to_char(GET_EQ(ch, i), ch, SHOW_OBJ_SHORT);
				found = TRUE;
			} else {
				send_to_char(ch, "%s", wear_where[i]);
				send_to_char(ch, "Something.\r\n");
				found = TRUE;
			}
		}
	}
	if (!found)
		send_to_char(ch, " Nothing.\r\n");
}


void print_object_location(int num, struct obj_data *obj, struct char_data *ch,
									int recur)
{
	if (num > 0)
		send_to_char(ch, "O%3d. %-25s - ", num, obj->short_description);
	else
		send_to_char(ch, "%33s", " - ");

	if (IN_ROOM(obj) != NOWHERE)
		send_to_char(ch, "[%5d] %s\r\n", GET_ROOM_VNUM(IN_ROOM(obj)), world[IN_ROOM(obj)].name);
	else if (CARRIED_BY(obj))
		send_to_char(ch, "carried by %s\r\n", PERS(CARRIED_BY(obj), ch));
	else if (WORN_BY(obj))
		send_to_char(ch, "worn by %s\r\n", PERS(WORN_BY(obj), ch));
	else if (IN_OBJ(obj)) {
		send_to_char(ch, "inside %s%s\r\n", IN_OBJ(obj)->short_description, (recur ? ", which is" : " "));
		if (recur)
			print_object_location(0, IN_OBJ(obj), ch, recur);
	} else
		send_to_char(ch, "in an unknown location\r\n");
}


