/* The do_set function and related stuff */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"


/* extern variables */
extern int top_of_p_table;
extern struct time_info_data time_info;

/* extern functions */
int parse_class(struct char_data *ch, char arg);
int parse_race(char arg);
int set_contact(struct char_data *ch, char *name);
void sun_text(void);


/* local functions */
int perform_set(struct char_data *ch, struct char_data *vict, int mode, 
			char *val_arg);

ACMD(do_set);
ACMD(do_set_time);


/***************** The do_set function ***********************************/

#define PC   1
#define NPC  2
#define BOTH 3

#define MISC	0
#define BINARY	1
#define NUMBER	2

#define SET_OR_REMOVE(flagset, flags) { \
	if (on) \
          flagset.flags = 1; \
	else if (off) \
          flagset.flags = 0; \
        }

#define RANGE(low, high) (value = MAX((low), MIN((high), (value))))


/* The set options available */
struct set_struct {
	const char *cmd;
	const char level;
	const char pcnpc;
	const char type;
} set_fields[] = {
   { "brief",		LVL_GOD, 	PC, 	BINARY },  /* 0 */
   { "invstart", 	LVL_GOD, 	PC, 	BINARY },  /* 1 */
   { "title",		LVL_GOD, 	PC, 	MISC },
   { "nosummon", 	LVL_GRGOD, 	PC, 	BINARY },
   { "maxhit",		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "maxmana", 	LVL_GRGOD, 	BOTH, 	NUMBER },  /* 5 */
   { "maxmove", 	LVL_GRGOD, 	BOTH, 	NUMBER },
   { "hit", 		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "mana",		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "move",		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "align",		LVL_GOD, 	BOTH, 	NUMBER },  /* 10 */
   { "str",		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "stradd",		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "int", 		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "wis", 		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "dex", 		LVL_GRGOD, 	BOTH, 	NUMBER },  /* 15 */
   { "con", 		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "cha",		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "ac", 		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "gold",		LVL_GOD, 	BOTH, 	NUMBER },
   { "bank",		LVL_GOD, 	PC, 	NUMBER },  /* 20 */
   { "exp", 		LVL_GRGOD, 	BOTH, 	NUMBER },
   { "hitroll", 	LVL_GRGOD, 	BOTH, 	NUMBER },
   { "damroll", 	LVL_GRGOD, 	BOTH, 	NUMBER },
   { "invis",		LVL_IMPL, 	PC, 	NUMBER },
   { "nohassle", 	LVL_GRGOD, 	PC, 	BINARY },  /* 25 */
   { "frozen",		LVL_FREEZE, 	PC, 	BINARY },
   { "practices", 	LVL_GRGOD, 	PC, 	NUMBER },
   { "lessons", 	LVL_GRGOD, 	PC, 	NUMBER },
   { "drunk",		LVL_GRGOD, 	BOTH, 	MISC },
   { "hunger",		LVL_GRGOD, 	BOTH, 	MISC },    /* 30 */
   { "thirst",		LVL_GRGOD, 	BOTH, 	MISC },
   { "killer",		LVL_GOD, 	PC, 	BINARY },
   { "thief",		LVL_GOD, 	PC, 	BINARY },
   { "level",		LVL_IMPL, 	BOTH, 	NUMBER },
   { "room",		LVL_IMPL, 	BOTH, 	NUMBER },  /* 35 */
   { "roomflag", 	LVL_GRGOD, 	PC, 	BINARY },
   { "siteok",		LVL_GRGOD, 	PC, 	BINARY },
   { "deleted", 	LVL_IMPL, 	PC, 	BINARY },
   { "class",		LVL_GRGOD, 	BOTH, 	MISC },
   { "nowizlist", 	LVL_GOD, 	PC, 	BINARY },  /* 40 */
   { "quest",		LVL_GOD, 	PC, 	BINARY },
   { "loadroom", 	LVL_GRGOD, 	PC, 	MISC },
   { "color",		LVL_GOD, 	PC, 	BINARY },
   { "idnum",		LVL_IMPL, 	PC, 	NUMBER },
   { "passwd",		LVL_IMPL, 	PC, 	MISC },    /* 45 */
   { "nodelete", 	LVL_GOD, 	PC, 	BINARY },
   { "sex", 		LVL_GRGOD, 	BOTH, 	MISC },
   { "age",		LVL_GRGOD,	BOTH,	NUMBER },
   { "height",		LVL_GOD,	BOTH,	NUMBER },
   { "weight",		LVL_GOD,	BOTH,	NUMBER },  /* 50 */
	{ "contact",   LVL_GRGOD, PC,   MISC },
	{ "race",      LVL_GRGOD, BOTH, MISC },
	{ "private",   LVL_GRGOD, PC, NUMBER },
	{ "shares",    LVL_GRGOD, PC, NUMBER },
	{ "attacks",   LVL_GRGOD, NPC, NUMBER }, /* 55 */
   { "platinum",	LVL_GOD, 	BOTH, 	NUMBER },
   { "silver",		LVL_GOD, 	BOTH, 	NUMBER },
   { "copper",		LVL_GOD, 	BOTH, 	NUMBER },
   { "\n", 0, BOTH, MISC }
};


int perform_set(struct char_data *ch, struct char_data *vict, int mode,
					 char *val_arg)
{
	int i, on = 0, off = 0, value = 0;
	room_rnum rnum;
	room_vnum rvnum;

	/* Check to make sure all the levels are correct */
	if (GET_LEVEL(ch) != LVL_IMPL) {
		if (!IS_NPC(vict) && GET_LEVEL(ch) <= GET_LEVEL(vict) && vict != ch) {
			send_to_char(ch, "Maybe that's not such a great idea...\r\n");
			return (0);
		}
	}
	if (GET_LEVEL(ch) < set_fields[mode].level) {
		send_to_char(ch, "You are not godly enough for that!\r\n");
		return (0);
	}

	/* Make sure the PC/NPC is correct */
	if (IS_NPC(vict) && !(set_fields[mode].pcnpc & NPC)) {
		send_to_char(ch, "You can't do that to a beast!\r\n");
		return (0);
	} else if (!IS_NPC(vict) && !(set_fields[mode].pcnpc & PC)) {
		send_to_char(ch, "That can only be done to a beast!\r\n");
		return (0);
	}

	/* Find the value of the argument */
	if (set_fields[mode].type == BINARY) {
		if (!strcmp(val_arg, "on") || !strcmp(val_arg, "yes"))
			on = 1;
		else if (!strcmp(val_arg, "off") || !strcmp(val_arg, "no"))
			off = 1;
		if (!(on || off)) {
			send_to_char(ch, "Value must be 'on' or 'off'.\r\n");
			return (0);
		}
		send_to_char(ch, "%s %s for %s.\r\n", set_fields[mode].cmd, 
						 ONOFF(on), GET_NAME(vict));
	} else if (set_fields[mode].type == NUMBER) {
		value = atoi(val_arg);
		send_to_char(ch, "%s's %s set to %d.\r\n", GET_NAME(vict), 
						 set_fields[mode].cmd, value);
	} else
		send_to_char(ch, "%s", OK);

	switch (mode) {
		case 0:
			SET_OR_REMOVE(PRF_FLAGS(vict), PRF_BRIEF);
			break;
		case 1:
			SET_OR_REMOVE(PLR_FLAGS(vict), PLR_INVSTART);
			break;
		case 2:
			set_title(vict, val_arg);
			send_to_char(ch, "%s's title is now: %s\r\n", 
							 GET_NAME(vict), GET_TITLE(vict));
			break;
		case 3:
			SET_OR_REMOVE(PRF_FLAGS(vict), PRF_SUMMONABLE);
			send_to_char(ch, "Nosummon %s for %s.\r\n", 
							 ONOFF(!on), GET_NAME(vict));
			break;
		case 4:
			vict->points.max_hit = RANGE(1, 5000);
			affect_total(vict);
			break;
		case 5:
			vict->points.max_mana = RANGE(1, 5000);
			affect_total(vict);
			break;
		case 6:
			vict->points.max_move = RANGE(1, 5000);
			affect_total(vict);
			break;
		case 7:
			vict->points.hit = RANGE(-9, vict->points.max_hit);
			affect_total(vict);
			break;
		case 8:
			vict->points.mana = RANGE(0, vict->points.max_mana);
			affect_total(vict);
			break;
		case 9:
			vict->points.move = RANGE(0, vict->points.max_move);
			affect_total(vict);
			break;
		case 10:
			GET_ALIGNMENT(vict) = RANGE(-1000, 1000);
			affect_total(vict);
			break;
		case 11:
			if (IS_NPC(vict) || GET_LEVEL(vict) >= LVL_GRGOD)
				RANGE(3, 25);
			else
				RANGE(3, 18);
			vict->real_abils.str = value;
			vict->real_abils.str_add = 0;
			affect_total(vict);
			break;
		case 12:
			vict->real_abils.str_add = RANGE(0, 100);
			if (value > 0)
				vict->real_abils.str = 18;
			affect_total(vict);
			break;
		case 13:
			if (IS_NPC(vict) || GET_LEVEL(vict) >= LVL_GRGOD)
				RANGE(3, 25);
			else
				RANGE(3, 18);
			vict->real_abils.intel = value;
			affect_total(vict);
			break;
		case 14:
			if (IS_NPC(vict) || GET_LEVEL(vict) >= LVL_GRGOD)
				RANGE(3, 25);
			else
				RANGE(3, 18);
			vict->real_abils.wis = value;
			affect_total(vict);
			break;
		case 15:
			if (IS_NPC(vict) || GET_LEVEL(vict) >= LVL_GRGOD)
				RANGE(3, 25);
			else
				RANGE(3, 18);
			vict->real_abils.dex = value;
			affect_total(vict);
			break;
		case 16:
			if (IS_NPC(vict) || GET_LEVEL(vict) >= LVL_GRGOD)
				RANGE(3, 25);
			else
				RANGE(3, 18);
			vict->real_abils.con = value;
			affect_total(vict);
			break;
		case 17:
			if (IS_NPC(vict) || GET_LEVEL(vict) >= LVL_GRGOD)
				RANGE(3, 25);
			else
				RANGE(3, 18);
			vict->real_abils.cha = value;
			affect_total(vict);
			break;
		case 18:
			vict->points.armor = RANGE(-100, 100);
			affect_total(vict);
			break;
		case 19:
			remove_money_from_char(vict, GET_MONEY(vict, GOLD_COINS), GOLD_COINS);
			add_money_to_char(vict, RANGE(0, 100000000), GOLD_COINS);
			break;
		case 20:
			GET_BANK_GOLD(vict) = RANGE(0, 100000000);
			break;
		case 21:
			vict->points.exp = RANGE(0, 50000000);
			break;
		case 22:
			vict->points.hitroll = RANGE(-20, 20);
			affect_total(vict);
			break;
		case 23:
			vict->points.damroll = RANGE(-20, 20);
			affect_total(vict);
			break;
		case 24:
			if (GET_LEVEL(ch) < LVL_IMPL && ch != vict) {
				send_to_char(ch, "You aren't godly enough for that!\r\n");
				return (0);
			}
			GET_INVIS_LEV(vict) = RANGE(0, GET_LEVEL(vict));
			break;
		case 25:
			if (GET_LEVEL(ch) < LVL_IMPL && ch != vict) {
				send_to_char(ch, "You aren't godly enough for that!\r\n");
				return (0);
			}
			SET_OR_REMOVE(PRF_FLAGS(vict), PRF_NOHASSLE);
			break;
		case 26:
			if (ch == vict && on) {
				send_to_char(ch, "Better not -- could be a long winter!\r\n");
				return (0);
			}
			SET_OR_REMOVE(PLR_FLAGS(vict), PLR_FROZEN);
			break;
		case 27:
		case 28:
			GET_PRACTICES(vict) = RANGE(0, 100);
			break;
		case 29:
		case 30:
		case 31:
			if (!str_cmp(val_arg, "off")) {
				GET_COND(vict, (mode - 29)) = -1; /* warning: magic number here */
				send_to_char(ch, "%s's %s now off.\r\n", GET_NAME(vict), set_fields[mode].cmd);
			} else if (is_number(val_arg)) {
				value = atoi(val_arg);
				RANGE(0, 24);
				GET_COND(vict, (mode - 29)) = value; /* and here too */
				send_to_char(ch, "%s's %s set to %d.\r\n", GET_NAME(vict), set_fields[mode].cmd, value);
			} else {
				send_to_char(ch, "Must be 'off' or a value from 0 to 24.\r\n");
				return (0);
			}
			break;
		case 32:
			SET_OR_REMOVE(PLR_FLAGS(vict), PLR_KILLER);
			break;
		case 33:
			SET_OR_REMOVE(PLR_FLAGS(vict), PLR_THIEF);
			break;
		case 34:
			if (value > GET_LEVEL(ch) || value > LVL_IMPL) {
				send_to_char(ch, "You can't do that.\r\n");
				return (0);
			}
			RANGE(0, LVL_IMPL);
			vict->player.level = value;
			break;
		case 35:
			if ((rnum = real_room(value)) == NOWHERE) {
				send_to_char(ch, "No room exists with that number.\r\n");
				return (0);
			}
			if (IN_ROOM(vict) != NOWHERE)	/* Another Eric Green special. */
				char_from_room(vict);
			char_to_room(vict, rnum);
			break;
		case 36:
			SET_OR_REMOVE(PRF_FLAGS(vict), PRF_ROOMFLAGS);
			break;
		case 37:
			SET_OR_REMOVE(PLR_FLAGS(vict), PLR_SITEOK);
			break;
		case 38:
			SET_OR_REMOVE(PLR_FLAGS(vict), PLR_DELETED);
			break;
		case 39:
			if ((i = parse_class(ch, *val_arg)) == CLASS_UNDEFINED) {
				send_to_char(ch, "That is not a class.\r\n");
				return (0);
			}
			GET_CLASS(vict) = i;
			break;
		case 40:
			SET_OR_REMOVE(PLR_FLAGS(vict), PLR_NOWIZLIST);
			break;
		case 41:
			SET_OR_REMOVE(PRF_FLAGS(vict), PRF_QUEST);
			break;
		case 42:
			if (!str_cmp(val_arg, "off")) 
				PLR_FLAGS(vict).PLR_LOADROOM = 0;
			else if (is_number(val_arg)) {
				rvnum = atoi(val_arg);
				if (real_room(rvnum) != NOWHERE) {
					PLR_FLAGS(vict).PLR_LOADROOM = 1;
					GET_LOADROOM(vict) = rvnum;
					send_to_char(ch, "%s will enter at room #%d.", 
									 GET_NAME(vict), GET_LOADROOM(vict));
				} else {
					send_to_char(ch, "That room does not exist!\r\n");
					return (0);
				}
			} else {
				send_to_char(ch, "Must be 'off' or a room's virtual number.\r\n");
				return (0);
			}
			break;
		case 43:
			SET_OR_REMOVE(PRF_FLAGS(vict), PRF_COLOR_1);
			SET_OR_REMOVE(PRF_FLAGS(vict), PRF_COLOR_2);
			break;
		case 44:
			if (GET_IDNUM(ch) != 1 || !IS_NPC(vict))
				return (0);
			GET_IDNUM(vict) = value;
			break;
		case 45:
			if (GET_IDNUM(ch) > 1) {
				send_to_char(ch, "Please don't use this command, yet.\r\n");
				return (0);
			}
			if (GET_LEVEL(vict) >= LVL_GRGOD) {
				send_to_char(ch, "You cannot change that.\r\n");
				return (0);
			}
			strncpy(GET_PASSWD(vict), CRYPT(val_arg, GET_NAME(vict)), MAX_PWD_LENGTH);	/* strncpy: OK (G_P:MAX_PWD_LENGTH) */
			*(GET_PASSWD(vict) + MAX_PWD_LENGTH) = '\0';
			send_to_char(ch, "Password changed to '%s'.\r\n", val_arg);
			break;
		case 46:
			SET_OR_REMOVE(PLR_FLAGS(vict), PLR_NODELETE);
			break;
		case 47:
			if ((i = search_block(val_arg, genders, FALSE)) < 0) {
				send_to_char(ch, "Must be 'male', 'female', or 'neutral'.\r\n");
				return (0);
			}
			GET_SEX(vict) = i;
			break;
		case 48:	/* set age */
			if (value < 2 || value > 200) {	/* Arbitrary limits. */
				send_to_char(ch, "Ages 2 to 200 accepted.\r\n");
				return (0);
			}
			/*
			 * NOTE: May not display the exact age specified due to the integer
			 * division used elsewhere in the code.  Seems to only happen for
			 * some values below the starting age (17) anyway. -gg 5/27/98
			 */
			vict->player.time.birth = time(0) - ((value - 17) * SECS_PER_MUD_YEAR);
			break;
		case 49:	/* Blame/Thank Rick Glover. :) */
			GET_HEIGHT(vict) = value;
			affect_total(vict);
			break;
		case 50:
			GET_WEIGHT(vict) = value;
			affect_total(vict);
			break;
		case 51: 
			if (GET_LEVEL(vict) >= LVL_IMMORT) {
				send_to_char(ch, "Can't set that on Immortals!\r\n");
				return (0);
			}

			if (set_contact(vict, val_arg)) {
				send_to_char(ch, "%s's contact is now: %s\r\n", GET_NAME(vict),
								 (GET_CONTACT(vict) ? GET_CONTACT(vict) : "off"));
			} else {
				send_to_char(ch, "Invalid contact name.\r\n");
				return (0);
			}
			break;
		case 52:
			if ((i = parse_race(*val_arg)) == RACE_UNDEFINED) {
				send_to_char(ch, "That is not a valid race.\r\n");
				return (0);
			}
			GET_RACE(vict) = i;
			break;
		case 53:
			GET_PRIVATE(vict) = RANGE(0, top_of_p_table);
			break;
		case 54:
			GET_SHARES(vict) = RANGE(0, 1000000);
			break;
		case 55:
			vict->mob_specials.num_attacks = RANGE(0, 10);
		case 56:
			remove_money_from_char(vict, GET_MONEY(vict, PLAT_COINS), PLAT_COINS);
			add_money_to_char(vict, RANGE(0, 100000000), PLAT_COINS);
			break;
		case 57:
			remove_money_from_char(vict, GET_MONEY(vict, SILVER_COINS), SILVER_COINS);
			add_money_to_char(vict, RANGE(0, 100000000), SILVER_COINS);
			break;
		case 58:
			remove_money_from_char(vict, GET_MONEY(vict, COPPER_COINS), COPPER_COINS);
			add_money_to_char(vict, RANGE(0, 100000000), COPPER_COINS);
			break;

		default:
			send_to_char(ch, "Can't set that!\r\n");
			return (0);
	}

	return (1);
}


ACMD(do_set)
{
	struct char_data *vict = NULL, *cbuf = NULL;
	char field[MAX_INPUT_LENGTH], name[MAX_INPUT_LENGTH], buf[MAX_INPUT_LENGTH];
	int mode, len, player_i = 0, retval;
	char is_file = 0, is_player = 0;

	half_chop(argument, name, buf);

	if (!strcmp(name, "file")) {
		is_file = 1;
		half_chop(buf, name, buf);
	} else if (!str_cmp(name, "player")) {
		is_player = 1;
		half_chop(buf, name, buf);
	} else if (!str_cmp(name, "mob"))
		half_chop(buf, name, buf);

	half_chop(buf, field, buf);

	if (!*name || !*field) {
		send_to_char(ch, "Usage: set <victim> <field> <value>\r\n");
		return;
	}

	/* find the target */
	if (!is_file) {
		if (is_player) {
			if (!(vict = get_player_vis(ch, name, NULL, FIND_CHAR_WORLD))) {
				send_to_char(ch, "There is no such player.\r\n");
				return;
			}
		} else { /* is_mob */
			if (!(vict = get_char_vis(ch, name, NULL, FIND_CHAR_WORLD))) {
				send_to_char(ch, "There is no such creature.\r\n");
				return;
			}
		}
	} else if (is_file) {
		/* try to load the player off disk */
		CREATE(cbuf, struct char_data, 1);
		clear_char(cbuf);

		CREATE(cbuf->player_specials, struct player_special_data, 1);
		if ((player_i = load_char(name, cbuf)) > -1) {
			if (GET_LEVEL(cbuf) >= GET_LEVEL(ch)) {
				free_char(cbuf);
				send_to_char(ch, "Sorry, you can't do that.\r\n");
				return;
			}
			vict = cbuf;
		} else {
			free_char(cbuf);
			send_to_char(ch, "There is no such player.\r\n");
			return;
		}
	}

	/* find the command in the list */
	len = strlen(field);
	for (mode = 0; *(set_fields[mode].cmd) != '\n'; mode++)
		if (!strncmp(field, set_fields[mode].cmd, len))
			break;

	/* perform the set */
	retval = perform_set(ch, vict, mode, buf);

	/* save the character if a change was made */
	if (retval) {
		if (!is_file && !IS_NPC(vict))
			save_char(vict);
		if (is_file) {
			GET_PFILEPOS(cbuf) = player_i;
			save_char(cbuf);
			send_to_char(ch, "Saved in file.\r\n");
		}
	}

	/* free the memory if we allocated it earlier */
	if (is_file)
		free_char(cbuf);
}


ACMD(do_set_time)
{
	char arg[MAX_INPUT_LENGTH];
	int new_time = 0;

	one_argument(argument, arg);
	
	new_time = atoi(arg);
	if (new_time < 0 || new_time > 23) {
		send_to_char(ch, "Invalid time!  Must be between 0 and 23.\r\n");
		return;
	}
	
	time_info.hours = new_time;

	send_to_char(ch, "Time set to %d.\r\n", new_time);
	sun_text();
}
