/* the do_show command */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"


/* extern vars */
extern char *class_abbrevs[];
extern char *race_abbrevs[];
extern int top_of_p_table;
extern int buf_switches, buf_largecount, buf_overflows;


/* extern functions */
void hcontrol_list_houses(struct char_data *ch);
size_t print_zone_to_buf(char *bufptr, size_t left, zone_rnum zone);
void show_shops(struct char_data *ch, char *value);
void show_gm(struct char_data * ch, char *arg);
void	Crash_listrent(struct char_data *ch, char *name);


/* local functions */
ACMD(do_file);
ACMD(do_liblist);
ACMD(do_omni);
ACMD(do_show);


ACMD(do_show)
{
	FILE *fl;
	int i, j, k, l, con;		/* i, j, k to specifics? */
	int done = FALSE;
	size_t len, nlen;
	zone_rnum zrn;
	zone_vnum zvn;
	byte self = FALSE;
	struct char_data *vict = NULL;
	struct obj_data *obj;
	struct descriptor_data *d;
	char field[MAX_INPUT_LENGTH], value[MAX_INPUT_LENGTH];
	char arg[MAX_INPUT_LENGTH], buf[MAX_STRING_LENGTH];

	struct show_struct {
		const char *cmd;
		const char level;
	} fields[] = {
		{ "nothing",	0  },				/* 0 */
		{ "zones",		LVL_IMMORT },			/* 1 */
		{ "player",		LVL_GOD },
		{ "rent",		LVL_GOD },
		{ "stats",		LVL_IMMORT },
		{ "errors",		LVL_IMPL },			/* 5 */
		{ "death",		LVL_GOD },
		{ "godrooms",	LVL_GOD },
		{ "shops",		LVL_IMMORT },
		{ "houses",		LVL_GOD },
		{ "snoop",		LVL_GRGOD },			/* 10 */
		{ "immortals", LVL_IMPL},
		{ "guilds",    LVL_IMMORT },
		{ "\n", 0 }
	};

	skip_spaces(&argument);

	if (!*argument) {
		send_to_char(ch, "Show options:\r\n");
		for (j = 0, i = 1; fields[i].level; i++)
			if (fields[i].level <= GET_LEVEL(ch))
				send_to_char(ch, "%-15s%s", fields[i].cmd, (!(++j % 5) ? "\r\n" : ""));
		send_to_char(ch, "\r\n");
		return;
	}

	strcpy(arg, two_arguments(argument, field, value));	/* strcpy: OK (argument <= MAX_INPUT_LENGTH == arg) */

	for (l = 0; *(fields[l].cmd) != '\n'; l++)
		if (!strncmp(field, fields[l].cmd, strlen(field)))
			break;

	if (GET_LEVEL(ch) < fields[l].level) {
		send_to_char(ch, "You are not godly enough for that!\r\n");
		return;
	}
	if (!strcmp(value, "."))
		self = TRUE;
	buf[0] = '\0';

	switch (l) {
		/* show zone */
		case 1:
			/* tightened up by JE 4/6/93 */
			if (self)
				print_zone_to_buf(buf, sizeof(buf), world[IN_ROOM(ch)].zone);
			else if (*value && is_number(value)) {
				for (zvn = atoi(value), zrn = 0; 
					  zone_table[zrn].number != zvn && zrn <= top_of_zone_table; 
					  zrn++)
					; /* Just cycling to the desired zone. */
				if (zrn <= top_of_zone_table)
					print_zone_to_buf(buf, sizeof(buf), zrn);
				else {
					send_to_char(ch, "That is not a valid zone.\r\n");
					return;
				}
			} else
				for (len = zrn = 0; zrn <= top_of_zone_table; zrn++) {
					nlen = print_zone_to_buf(buf + len, sizeof(buf) - len, zrn);
					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			page_string(ch->desc, buf, TRUE);
			break;

			/* show player */
		case 2:
			if (!*value) {
				send_to_char(ch, "A name would help.\r\n");
				return;
			}

			CREATE(vict, struct char_data, 1);
			clear_char(vict);
			CREATE(vict->player_specials, struct player_special_data, 1);
			if (load_char(value, vict) < 0) {
				send_to_char(ch, "There is no such player.\r\n");
				free_char(vict);
				return;
			}

			sprintf(buf, "Player: %-12s (%s) [%2d %s %s], Align: %-5d\r\n", 
					  GET_NAME(vict), genders[(int) GET_SEX(vict)], 
					  GET_LEVEL(vict), race_abbrevs[(int)GET_RACE(vict)],
					  class_abbrevs[(int)GET_CLASS(vict)], GET_ALIGNMENT(vict));
			send_to_char(ch, "Copper: %-8d  Bal: %-8d  Shr: %-8d  Exp: %-8d  "
							 "Lessons: %-3d\r\n", convert_all_to_copper(vict), 
							 GET_BANK_GOLD(vict), GET_SHARES(vict), GET_EXP(vict),
							 GET_PRACTICES(vict));
			/* ctime() uses static buffer: do not combine. */
			send_to_char(ch, "Started: %-20.16s  ",
							 ctime(&vict->player.time.birth));
			send_to_char(ch, "Last: %-20.16s  Played: %3dh %2dm\r\n", 
							 ctime(&vict->player.time.logon),
							 (int) (vict->player.time.played / 3600),
							 (int) (vict->player.time.played / 60 % 60));
			free_char(vict);
			break;

			/* show rent */
		case 3:
			if (!*value) {
				send_to_char(ch, "A name would help.\r\n");
				return;
			}
			Crash_listrent(ch, value);
			break;

			/* show stats */
		case 4:
			i = 0;
			j = 0;
			k = 0;
			con = 0;
			for (vict = character_list; vict; vict = vict->next) {
				if (IS_NPC(vict))
					j++;
				else if (CAN_SEE(ch, vict)) {
					i++;
					if (vict->desc)
						con++;
				}
			}
			for (obj = object_list; obj; obj = obj->next)
				k++;
			send_to_char(ch,
							 "Current stats:\r\n"
							 "  %5d players in game  %5d connected\r\n"
							 "  %5d registered\r\n"
							 "  %5d mobiles          %5d prototypes\r\n"
							 "  %5d objects          %5d prototypes\r\n"
							 "  %5d rooms            %5d zones\r\n"
							 "  %5d large bufs\r\n"
							 "  %5d buf switches     %5d overflows\r\n",
							 i, con,
							 top_of_p_table + 1,
							 j, top_of_mobt + 1,
							 k, top_of_objt + 1,
							 top_of_world + 1, top_of_zone_table + 1,
							 buf_largecount,
							 buf_switches, buf_overflows
							 );
			break;

			/* show errors */
		case 5:
			len = strlcpy(buf, "Errant Rooms\r\n------------\r\n", sizeof(buf));
			for (i = 0, k = 0; i <= top_of_world; i++)
				for (j = 0; j < NUM_OF_DIRS; j++)
					if (world[i].dir_option[j] && world[i].dir_option[j]->to_room == 0) {
						nlen = snprintf(buf + len, sizeof(buf) - len, "%2d: [%5d] %s\r\n", ++k, GET_ROOM_VNUM(i), world[i].name);
						if (len + nlen >= sizeof(buf) || nlen < 0)
							break;
						len += nlen;
					}
			page_string(ch->desc, buf, TRUE);
			break;

			/* show death */
		case 6:
			len = strlcpy(buf, "Death Traps\r\n-----------\r\n", sizeof(buf));
			for (i = 0, j = 0; i <= top_of_world; i++)
				if (ROOM_FLAGGED(i, ROOM_DEATH)) {
					nlen = snprintf(buf + len, sizeof(buf) - len, "%2d: [%5d] %s\r\n", ++j, GET_ROOM_VNUM(i), world[i].name);
					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			page_string(ch->desc, buf, TRUE);
			break;

			/* show godrooms */
		case 7:
			len = strlcpy(buf, "Godrooms\r\n--------------------------\r\n", sizeof(buf));
			for (i = 0, j = 0; i <= top_of_world; i++)
				if (ROOM_FLAGGED(i, ROOM_GODROOM)) {
					nlen = snprintf(buf + len, sizeof(buf) - len, "%2d: [%5d] %s\r\n", ++j, GET_ROOM_VNUM(i), world[i].name);
					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			page_string(ch->desc, buf, TRUE);
			break;

			/* show shops */
		case 8:
			show_shops(ch, value);
			break;

			/* show houses */
		case 9:
			hcontrol_list_houses(ch);
			break;

			/* show snoop */
		case 10:
			i = 0;
			send_to_char(ch, "People currently snooping:\r\n--------------------------\r\n");
			for (d = descriptor_list; d; d = d->next) {
				if (d->snooping == NULL || d->character == NULL)
					continue;
				if (STATE(d) != CON_PLAYING || GET_LEVEL(ch) < GET_LEVEL(d->character))
					continue;
				if (!CAN_SEE(ch, d->character) || IN_ROOM(d->character) == NOWHERE)
					continue;
				i++;
				send_to_char(ch, "%-10s - snooped by %s.\r\n", GET_NAME(d->snooping->character), GET_NAME(d->character));
			}
			if (i == 0)
				send_to_char(ch, "No one is currently snooping.\r\n");
			break;

		case 11:
			if (!(fl = fopen(PLAYER_FILE, "r+"))) {
				send_to_char(ch, "Can't open player file.");
				return;
			}
			snprintf(buf, sizeof(buf), "Player Name          Player Level\r\n");
			while (!done) {
				/* THIS IS BROKEN!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
				struct char_file_u vbuf;
			
				fread(&vbuf, sizeof(struct char_file_u), 1, fl);
				if (feof(fl)) {
					fclose(fl);
					done = TRUE;
				}
				if (!done)
				{
					if (vbuf.level >= LVL_IMMORT)
						nlen = snprintf(buf + len, sizeof(buf) - len, 
											 "%-20s %-2i\r\n", vbuf.name, vbuf.level);

					if (len + nlen >= sizeof(buf) || nlen < 0)
						break; /* while (!done) { */
					len += nlen;
				}
			}
			page_string(ch->desc, buf, TRUE);
			break;

		case 12:
			show_gm(ch, value);
			break;

			/* show what? */
		default:
			send_to_char(ch, "Sorry, I don't understand that.\r\n");
			break;
	}
}


ACMD(do_liblist)
{
	int first, last, nr, found = 0;
	char buf[MAX_STRING_LENGTH], buf2[MAX_INPUT_LENGTH];
	size_t len = 0, nlen = 0;

	two_arguments(argument, buf, buf2);

	if (!*buf || !*buf2) {
		switch (subcmd) {
			case SCMD_RLIST:
				send_to_char(ch, "Usage: rlist <beginning number> <ending number>\r\n");
				break;
			case SCMD_OLIST:
				send_to_char(ch, "Usage: olist <beginning number> <ending number>\r\n");
				break;
			case SCMD_MLIST:
				send_to_char(ch, "Usage: mlist <beginning number> <ending number>\r\n");
				break;
			case SCMD_ZLIST:
				send_to_char(ch, "Usage: zlist <beginning number> <ending number>\r\n");
				break;
			default:
				mudlog(BRF, LVL_GOD, TRUE, "SYSERR:: invalid SCMD passed to "
						 "ACMD(do_liblist)!");
				break;
		}
		return;
	}

	first = atoi(buf);
	last = atoi(buf2);

	if (first < 0 || first > 99999 || last < 0 || last > 99999) {
		send_to_char(ch, "Values must be between 0 and 99999.\r\n");
		return;
	}

	if (first >= last) {
		send_to_char(ch, "Second value must be greater than first.\r\n");
		return;
	}

	switch (subcmd) {
		case SCMD_RLIST:
			nlen = snprintf(buf + len, sizeof(buf) - len, 
								 "Room List From Vnum %d to %d\r\n\r\n", first, last);
			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;

			nlen = snprintf(buf + len, sizeof(buf) - len, 
								 "%6s [%5s] (%3s) %s\r\n", "", "vnum", "Zon", "Name");

			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;

			for (nr = 0; nr <= top_of_world && (world[nr].number <= last); nr++) {
				if (world[nr].number >= first) {
					nlen = snprintf(buf + len, sizeof(buf) - len, 
										 "%5d. [%5d] (%3d) %s\r\n", 
										 ++found, world[nr].number,
										 zone_table[world[nr].zone].number, 
										 world[nr].name);

					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			}
			break;
		case SCMD_OLIST:
			nlen = snprintf(buf + len, sizeof(buf) - len, 
								 "Object List From Vnum %d to %d\r\n", first, last);
			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;

			for (nr = 0; nr <= top_of_objt && (obj_index[nr].vnum <= last); nr++)
			{
				if (obj_index[nr].vnum >= first) {
					nlen = snprintf(buf + len, sizeof(buf) - len, 
										 "%5d. [%5d] %s\r\n", ++found, obj_index[nr].vnum,
										 obj_proto[nr].short_description);

					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			}
			break;
		case SCMD_MLIST:
			nlen = snprintf(buf + len, sizeof(buf) - len, 
								"Mob List From Vnum %d to %d\r\n", first, last);
			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;

			for (nr = 0; nr <= top_of_mobt && (mob_index[nr].vnum <= last); nr++)
			{
				if (mob_index[nr].vnum >= first) {
					nlen = snprintf(buf + len, sizeof(buf) - len,
										 "%5d. [%5d] %s\r\n", ++found, mob_index[nr].vnum,
										 mob_proto[nr].player.short_descr);

					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			}
			break;
		case SCMD_ZLIST:
			nlen = snprintf(buf + len, sizeof(buf) - len, 
								 "Zone List From Vnum %d to %d\r\n\r\n", first, last);
			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;

			nlen = snprintf(buf + len, sizeof(buf) - len, "%6s [%5s] %5s %s\r\n", 
								 "", "vnum", "Life", "Description");
			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;

			for (nr = 0; 
				  nr <= top_of_zone_table && (zone_table[nr].number <= last);
				  nr++) {
				if (zone_table[nr].number >= first) {
					nlen = snprintf(buf + len, sizeof(buf) - len,
										 "%5d. [%5d] (%3d) %s\r\n", ++found,
										 zone_table[nr].number, zone_table[nr].lifespan,
										 zone_table[nr].name);
					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			}
			break;
		default:
			mudlog(BRF, LVL_GOD, TRUE, "SYSERR:: invalid SCMD passed to "
					 "ACMD(do_liblist)!");
			return;
	}

	if (!found) {
		switch (subcmd) {
			case SCMD_RLIST:
				send_to_char(ch, "No rooms found within those parameters.\r\n");
				break;
			case SCMD_OLIST:
				send_to_char(ch, "No objects found within those parameters.\r\n");
				break;
			case SCMD_MLIST:
				send_to_char(ch, "No mobiles found within those parameters.\r\n");
				break;
			case SCMD_ZLIST:
				send_to_char(ch, "No zones found within those parameters.\r\n");
				break;
			default:
				mudlog(BRF, LVL_GOD, TRUE, "SYSERR:: invalid SCMD passed to "
						 "ACMD(do_liblist)!");
				break;
		}
		return;
	}

	if (ch->desc)
		page_string(ch->desc, buf, TRUE);
}


ACMD(do_file)
{
	FILE *req_file;
	int cur_line = 0, num_lines = 0, req_lines = 0, i, j, l, nlen = 0, len = 0;
	char field[MAX_INPUT_LENGTH], value[MAX_INPUT_LENGTH];
	char buf[MAX_STRING_LENGTH], arg[MAX_STRING_LENGTH];
	char buf2[MAX_STRING_LENGTH];

	struct file_struct {
		char *cmd;
		char level;
		char *file; 
	} fields[] = { 
		{ "none",     LVL_IMPL,    "Does Nothing" },
		{ "bug",      LVL_IMPL,    "../lib/misc/bugs" },
		{ "typo",     LVL_GOD,     "../lib/misc/typos" },
		{ "ideas",    LVL_GRGOD,   "../lib/misc/ideas" },
		{ "xnames",   LVL_IMPL,    "../lib/misc/xnames" },
		{ "levels",   LVL_IMMORT,  "../log/levels" },
		{ "rip",      LVL_IMMORT,  "../log/rip" },
		{ "players",  LVL_IMMORT,  "../log/newplayers" },
		{ "rentgone", LVL_GRGOD,   "../log/rentgone" },
		{ "godcmds",  LVL_IMPL,    "../log/godcmds" },
		{ "syslog",   LVL_GRGOD,   "../syslog" },
		{ "crash",    LVL_GRGOD,   "../syslog.CRASH" },
		{ "\n", 0, "\n" }
	};

	skip_spaces(&argument);

	if (!*argument) {
		strlcpy(buf, "USAGE: file <option> <num lines>\r\n\r\nFile "
				  "options:\r\n", sizeof(buf));
		for (j = 0, i = 1; fields[i].level; i++)
			if (fields[i].level <= GET_LEVEL(ch))
				send_to_char(ch, "%-15s%s\r\n", fields[i].cmd, fields[i].file);

		return;
	}

	strcpy(arg, two_arguments(argument, field, value));

	for (l = 0; *(fields[l].cmd) != '\n'; l++)
		if (!strncmp(field, fields[l].cmd, strlen(field)))
			break;

	if (*(fields[l].cmd) == '\n') {
		send_to_char(ch, "That is not a valid option!\r\n");
		return;
	}

	if (GET_LEVEL(ch) < fields[l].level) {
		send_to_char(ch, "You are not godly enough to view that file!\r\n");
		return;
	}

	if (!*value)
		req_lines = 15; /* default is the last 15 lines */
	else
		req_lines = atoi(value);

	/* open the requested file */
	if (!(req_file = fopen(fields[l].file, "r"))) {
		mudlog(BRF, LVL_IMPL, TRUE, "SYSERR: Error opening file %s using "
				 "'file' command.", fields[l].file);
		return;
	}

	/* count lines in requested file */
	get_line(req_file,buf);
	while (!feof(req_file)) {
		num_lines++;
		get_line(req_file,buf);
	}
	fclose(req_file);


	/* Limit # of lines printed to # requested or # of lines in file or
		150 lines */
	if (req_lines > num_lines) 
		req_lines = num_lines;

	if (req_lines > 20) 
		req_lines = 20;


	/* close and re-open */
	if (!(req_file = fopen(fields[l].file, "r"))) {
		mudlog(BRF, LVL_IMPL, TRUE, "SYSERR: Error opening file %s using "
				 "'file' command.", fields[l].file);
		return;
	}

	buf2[0] = '\0';

	/* and print the requested lines */
	get_line(req_file,buf);
	while (!feof(req_file)) {
		cur_line++;
		if (cur_line > (num_lines - req_lines))
			nlen = snprintf(buf2 + len, sizeof(buf2) - len, "%s\r\n", buf);
		
      if (len + nlen >= sizeof(buf2) || nlen < 0)
        break;
      len += nlen;

		get_line(req_file, buf);
   }
   page_string(ch->desc, buf2, TRUE);
   fclose(req_file);
}


ACMD(do_omni) {
	struct char_data *tch;
	int count = 0, all = TRUE;
	char buf[MAX_INPUT_LENGTH];
	struct char_data *victim;
	
	one_argument(argument, buf);

	if (*buf)
		all = FALSE;

	send_to_char(ch, "\r\nLvl  Class Race  Room   Name                Health  Position\r\n");
	send_to_char(ch, "------------------------------------------------------------\r\n");

	if (all)
	{
		for (tch = character_list; tch; tch = tch->next) {
			if (!IS_NPC(tch) && GET_LEVEL(tch) <= GET_LEVEL(ch)) {
				send_to_char(ch, "%-2d   %-5s %-4s  %-5d  %-20s %-3d%%   %-s\r\n",
								 GET_LEVEL(tch), CLASS_ABBR(tch), RACE_ABBR(tch),
								 world[IN_ROOM(tch)].number,
								 GET_NAME(tch), (int)(GET_HIT(tch)*100)/GET_MAX_HIT(tch),
								 position_types[(int)GET_POS(tch)]);
				count++;
			}
		}
		if (!count)
			send_to_char(ch, "  No mortals are on!\r\n");

	} else {
		if ((victim = get_char_vis(ch, buf, NULL, FIND_CHAR_WORLD)) != NULL) {
			if (IS_NPC(victim)) {
				send_to_char(ch, "Not on NPCs!\r\n");
				return;
			}
			
			send_to_char(ch, "%-2d   %-5s %-4s  %-5d  %-20s %-3d%%   %-s\r\n",
							 GET_LEVEL(victim), CLASS_ABBR(victim), 
							 RACE_ABBR(victim), world[IN_ROOM(victim)].number,
							 GET_NAME(victim), 
							 (int)(GET_HIT(victim)*100)/GET_MAX_HIT(victim),
							 position_types[(int)GET_POS(victim)]);
		}
	}
	
	send_to_char(ch, "\r\n");
}
