/********************************************************************\
*   History: Developed by: mlkesl@stthomas.edu			     *
*                     and: mlk                                       *
*   MapArea: when given a room, ch, x, and y                         *
*            this function will fill in values of map as it should   *
*   ShowMap: will simply spit out the contents of map array          *
*	    Would look much nicer if you built your own areas        *
*	    without all of the overlapping stock Rom has             *
*   do_map: core function, takes map size as argument                *
*   update: map is now a 2 dimensional array of integers             *
*	    uses SECT_MAX for null                                   *
*	    uses SECT_MAX+1 for mazes or one ways to SECT_ENTER	     *
*	    use the SECTOR numbers to represent sector types :)	     *
*                                                                    *
\********************************************************************/

 
/**********************************************************\
* WELCOME TO THE WORLD OF CIRCLEMUD ASCII MAP!!!           *
*  This was originall for Rom(who would use that?) and is  *
*  now available for CircleMUD. This was done on bpl17 but *
*  the only major conversions were room structures so it   *
*  should work on any Circle with fairly up to date rooms. *
*                                                          *
*  If you use this drop me a line maybe, and if your       *
*  generous put me in a line in your credits. Doesn't      *
*  matter though, ENJOY! Feel free to fix any bugs here    *
*  make sure you mail the CircleMUD discussion list to let *
*  everyone know!                                          *
*                                                          *
*  Edward Felch, efelch@hotmail.com  4/25/2001             *
\**********************************************************/


/**********************************************************\
 * Now it's my turn!  :-D                                 *
 * Actually, I didn't do much aside from updating it to   *
 * circle30bpl21, although I did separate out the hideous *
 * duplication of code to draw the map symbols and rolled *
 * it into one single function.  As well, I set it up to  *
 * use the stock color thingy in screen.h - if you have   *
 * some other color system feel free to change the color  *
 * codes accordingly.  Now aren't you glad you only have  *
 * to do that in one place instead of four!               *
 *                                                        *
 * Oh, I did also move the SECT_MAX define into structs.h *
 * so that when you add new sectors, you'll remember to   *
 * update it...                                           *
 *                                                        *
 *  -Edward Glamkowski  -5/10/02                          *
\**********************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "interpreter.h"
#include "screen.h"


#define URANGE(a, b, c)   ((b) < (a) ? (a) : ((b) > (c) ? (c) : (b)))

#define MAX_MAP     72
#define MAX_MAP_DIR  4


int map[MAX_MAP][MAX_MAP];
int offsets[4][2] = {
	{ -1, 0 }, { 0, 1 }, { 1, 0 }, { 0, -1 } 
};


/* Heavily modified - Edward */
void MapArea(room_rnum room, struct char_data *ch, int x, int y, int min, int max)
{
	struct room_direction_data *pexit;
	room_rnum prospect_room;
	int door;

	/* marks the room as visited */
	map[x][y] = world[room].sector_type;

	/* Otherwise we get a nasty crash */
	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_INDOORS))
		return;

	for (door = 0; door < MAX_MAP_DIR; door++) {
      if ((pexit = world[room].dir_option[door]) > 0  &&   
			 (pexit->to_room > 0 ) &&
			 (!EXIT_FLAGGED(pexit, EX_CLOSED))) {

			if (x < min || y < min || x > max || y >max)
				return;

			prospect_room = pexit->to_room;
         
			/* One way into area OR maze */	
			/* If not two way */
			if (world[prospect_room].dir_option[rev_dir[door]] &&
				 world[prospect_room].dir_option[rev_dir[door]]->to_room != room) {
				map[x][y] = SECT_MAX + 1;
				return;
			}
			/* End two way */

			/* Players cant see past these - be sure to update this if
			 * you add new sectors!   -ejg */
			if (world[prospect_room].sector_type == SECT_HILLS ||
				 world[prospect_room].sector_type == SECT_MOUNTAIN ||
				 world[prospect_room].sector_type == SECT_CITY ||
				 world[prospect_room].sector_type == SECT_INSIDE) { 
				map[x+offsets[door][0]][y+offsets[door][1]] = world[prospect_room].sector_type;
            /* ^--two way into area */		
			}

         if (map[x+offsets[door][0]][y+offsets[door][1]] == SECT_MAX) {
				MapArea(pexit->to_room, ch,
						  x + offsets[door][0], y + offsets[door][1], min, max);
         }
      } /* End if exit there */
	}
}


void draw_symbol(int sector, int color, struct char_data *ch)
{
	switch (sector) {
		case SECT_INSIDE:
			if (color)
				send_to_char(ch, QWHT);
			send_to_char(ch, "%");
			break;
		case SECT_CITY:
			if (color)
				send_to_char(ch, QWHT);
			send_to_char(ch, "#");
			break;
		case SECT_FIELD:
			if (color)
				send_to_char(ch, QGRN);
			send_to_char(ch, "\"");
			break;
		case SECT_FOREST:
			if (color)
				send_to_char(ch, QGRN);
			send_to_char(ch, "@");
			break;
		case SECT_HILLS:
			if (color)
				send_to_char(ch, QGRN);
			send_to_char(ch, "^");
			break;
		case SECT_MOUNTAIN:
			if (color)
				send_to_char(ch, QYEL);
			send_to_char(ch, "^");
			break;
		case SECT_WATER_SWIM:
			if (color)
				send_to_char(ch, QCYN);
			send_to_char(ch, ",");
			break;
		case SECT_WATER_NOSWIM:
			if (color)
				send_to_char(ch, QBLU);
			send_to_char(ch, ";");
			break;
		case SECT_FLYING:
			if (color)
				send_to_char(ch, QCYN);
			send_to_char(ch, "%");
			break;
		case SECT_UNDERWATER:
			if (color)
				send_to_char(ch, QBLU);
			send_to_char(ch, ":");
			break;
		case SECT_SWAMP:
			if (color)
				send_to_char(ch, QGRN);
			send_to_char(ch, "%");
			break;
		case SECT_JUNGLE:
			if (color)
				send_to_char(ch, QGRN);
			send_to_char(ch, "&");
			break;
		case SECT_LAVA:
			if (color)
				send_to_char(ch, QRED);
			send_to_char(ch, "\"");
			break;
		case SECT_TUNDRA:
			if (color)
				send_to_char(ch, QWHT);
			send_to_char(ch, "\"");
			break;
		case SECT_DESERT:
			if (color)
				send_to_char(ch, QYEL);
			send_to_char(ch, "\"");
			break;
		case SECT_MAX:
			if (color)
				send_to_char(ch, QNRM);
			send_to_char(ch, " ");
			break;
		case (SECT_MAX+1):
			if (color)
				send_to_char(ch, QNRM);
			send_to_char(ch, "?");
			break;
		default:
			if (color)
				send_to_char(ch, QMAG);
			send_to_char(ch, "*");
			break;
	}
}


/* mlk :: shows a map, specified by size */
void ShowMap(struct char_data *ch, int min, int max)
{
	int x, y;

	/* every row */
	for (x = min; x < max; ++x) { 
		/* every column */
		for (y = min; y < max; ++y) { 
			if (y == min || map[x][y-1] != map[x][y])
				draw_symbol(map[x][y], TRUE, ch);
			else
				draw_symbol(map[x][y], FALSE, ch);
		}
		send_to_char(ch, QNRM);
		send_to_char(ch, "\r\n"); 
	}
}


/* This is for simplicities sake, if the room's name if like "An
** unfinished room", it will replace it with a fake room name here -
** Edward
*/
void get_room_name(struct char_data *ch, int argument)
{
   switch(argument) {
		case SECT_INSIDE:
			send_to_char(ch, "   The Wilderness");
			break;
		case SECT_CITY:
			send_to_char(ch, "   Within a City");
			break;
		case SECT_FIELD:
			send_to_char(ch, "   On an Open Field");
			break;
		case SECT_FOREST:
			send_to_char(ch, "   Within a Forest");
			break;
		case SECT_HILLS:
			send_to_char(ch, "   On a Hill");
			break;
		case SECT_MOUNTAIN:
			send_to_char(ch, "   Climbing a Mountain");
			break;
		case SECT_WATER_SWIM:
			send_to_char(ch, "   In Shallow Water");
			break;
		case SECT_WATER_NOSWIM:
			send_to_char(ch, "   Adrift on Deep Blue Sea");
			break;
		case SECT_FLYING:
			send_to_char(ch, "   Floating in the Air");
			break;
		case SECT_UNDERWATER:
			send_to_char(ch, "   Underneath the Deep Blue Sea");
			break;
		case SECT_SWAMP:
			send_to_char(ch, "   Within a Murky Swamp");
			break;
		case SECT_JUNGLE:
			send_to_char(ch, "   Deep within the Jungle");
			break;
		case SECT_LAVA:
			send_to_char(ch, "   In a Lava Flow");
			break;
		case SECT_TUNDRA:
			send_to_char(ch, "   In a Frozen Wastes");
			break;
		case SECT_MAX:
			send_to_char(ch, "   The Wilderness");
			break;
		case (SECT_MAX+1):
			send_to_char(ch, "   The Wilderness");
			break;
		default:
			send_to_char(ch, "   The Wilderness");
			break;
	}
}


/* Will put a small map with current room desc and title. */
/* This is the main function to show the map, its do_map with " " */

/* Heavily modified - Edward */
void ShowRoom(struct char_data *ch, int min, int max)
{
	int x, y, str_pos = 0, desc_pos = 0;
	char desc[500];

	strlcpy(desc, world[IN_ROOM(ch)].description, sizeof(desc));

	/* mlk :: rounds edges */
	map[min][min]     = SECT_MAX;
	map[max-1][max-1] = SECT_MAX;
	map[min][max-1]   = SECT_MAX;
	map[max-1][min]   = SECT_MAX;

	/* every row */
	for (x = min; x < max; ++x) { 
		/* every column */
		for (y = min; y < max; ++y) { 
			if (y == min || (map[x][y-1] != map[x][y]))
				draw_symbol(map[x][y], TRUE, ch);
			else 
				draw_symbol(map[x][y], FALSE, ch);
		}

		if (x == min) {
			if (!strcmp(world[IN_ROOM(ch)].name, "The Wilderness") ||
				 !strcmp(world[IN_ROOM(ch)].name, "An unfinished room"))
				get_room_name(ch, world[IN_ROOM(ch)].sector_type);
			else
				send_to_char(ch, "   %s", world[IN_ROOM(ch)].name);
      
			if (GET_LEVEL(ch) >= LVL_IMMORT)
				send_to_char(ch, " [Room %ld]", world[IN_ROOM(ch)].number);
		}
		send_to_char(ch, QNRM);
		send_to_char(ch, "\r\n"); 
	}
	send_to_char(ch, "\r\n");  /* puts a line between contents/people */
}


/* This is the main map function, do_map(ch, " ") is good to use */
/* do_map(ch, "number") is for immortals to see the world map     */

/* Edward: If you play with some of the values here you can make the normal
** map people see larger or smaller. size = URANGE(9, size, MAX_MAP), the 9
** is the map size shown by default. Also look for: ShowMap (ch, min, max+1);
** and change the size of min and max and see what you like.
*/
ACMD(do_map)
{
	int size, center, x, y, min, max;
	char arg1[10];

	one_argument(argument, arg1);
	size = atoi(arg1);
	size = URANGE(9, size, MAX_MAP);
  
	center = MAX_MAP/2;

	min = MAX_MAP/2 - size/2;
	max = MAX_MAP/2 + size/2;

	for (x = 0; x < MAX_MAP; ++x)
      for (y = 0; y < MAX_MAP; ++y)
			map[x][y] = SECT_MAX;

	/* starts the mapping with the center room */
	MapArea(IN_ROOM(ch), ch, center, center, min-1, max-1); 

	/* marks the center, where ch is */
	map[center][center] = SECT_MAX+2;  /* can be any number above SECT_MAX+1 */
	/* switch default will print out the *	*/

	if (GET_LEVEL(ch) < LVL_IMMORT || IS_NPC(ch)) {
		if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_INDOORS)) {
			send_to_char(ch, "You can not do that here.\r\n");
			return;
		}

		if (IS_DARK(IN_ROOM(ch))) {
			send_to_char(ch, "The wilderness is pitch black at night... \r\n");
			return;
		} else {
         ShowRoom(ch, MAX_MAP/2-3, MAX_MAP/2+3);
         return;
		}
	}

	/* Mortals not in city or inside will always get a ShowRoom */
	if (GET_LEVEL(ch) >= LVL_IMMORT) {
		if (arg1[0] == '\0') 
			ShowRoom(ch, min, max+1); 
		else 
			ShowMap(ch, min, max+1); 

		return;
   }

   send_to_char(ch, "Huh?\r\n");
}
