#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "db.h"
#include "diskio.h"
#include "handler.h"
#include "interpreter.h"
#include "pfdefaults.h"


/* extern functions */
void get_one_line(FILE *fl, char *buf);
void free_alias(struct alias_data *a);
int find_name(char *name);


/* local functions */
void free_player_index(void);
void build_player_index(void);
int count_alias_records(FILE *fl);
struct char_data *create_char(void);
long get_ptable_by_name(const char *name);
long get_id_by_name(const char *name);
char *get_name_by_id(long id);
int load_char(char *name, struct char_data *ch);
void save_char(struct char_data *ch);
void save_etext(struct char_data *ch);
int create_entry(char *name);
void free_char(struct char_data *ch);
void reset_char(struct char_data *ch);
void clear_char(struct char_data *ch);
void init_char(struct char_data *ch);
void sprintbits(long vektor, char *outstring);
void tag_argument(char *argument, char *tag);

/* external ascii pfile vars */
extern struct pclean_criteria_data pclean_criteria[];
extern int selfdelete_fastwipe;

/* ascii pfiles - set this TRUE if you want poofin/poofout strings
 * saved in the pfiles */
#define ASCII_SAVE_POOFS	TRUE


/* local vars */
int top_of_p_table = 0;		/* ref to top of table		 */
int top_of_p_file = 0;	
long top_idnum = 0;		/* highest idnum in use		 */
struct player_index_element *player_table = NULL;	/* index to plr file	 */


void free_player_index(void)
{
	int tp;

	if (!player_table)
		return;

	for (tp = 0; tp <= top_of_p_table; tp++)
		if (player_table[tp].name)
			free(player_table[tp].name);

	free(player_table);
	player_table = NULL;
	top_of_p_table = 0;
}


/* new version to build player index for ascii pfiles */
/* generate index table for the player file */
void build_player_index(void)
{
	int rec_count = 0, i;
	FBFILE *plr_index;
	char index_name[40], line[256], bits[64];
	char arg2[80];

	sprintf(index_name, "%s", PLR_INDEX_FILE);
	if (!(plr_index = fbopen(index_name, FB_READ))) {
		top_of_p_table = -1;
		log("No player index file!  First new char will be IMP!");
		return;
	}
	
	while (fbgetline(plr_index, line)) {
		if (*line != '~')
			rec_count++;
	}
	fbrewind(plr_index);

	if (rec_count == 0) {
		player_table = NULL;
		top_of_p_table = -1;
		return;
	}

	CREATE(player_table, struct player_index_element, rec_count);
	for (i = 0; i < rec_count; i++) {
		fbgetline(plr_index, line);
		sscanf(line, "%ld %s %d %s %d %d", &player_table[i].id, arg2,
				 &player_table[i].level, bits, (int *)&player_table[i].last,
				 &player_table[i].autowiz);
		CREATE(player_table[i].name, char, strlen(arg2) + 1);
		strcpy(player_table[i].name, arg2);
		player_table[i].flags = asciiflag_conv(bits);
		top_idnum = MAX(top_idnum, player_table[i].id);
	}

	fbclose(plr_index);
	top_of_p_file = top_of_p_table = i - 1;
}

/*
 * Thanks to Andrey (andrey@alex-ua.com) for this bit of code, although I
 * did add the 'goto' and changed some "while()" into "do { } while()".
 *	-gg 6/24/98 (technically 6/25/98, but I care not.)
 */
int count_alias_records(FILE *fl)
{
	char key[READ_SIZE], next_key[READ_SIZE];
	char line[READ_SIZE], *scan;
	int total_keywords = 0;

	/* get the first keyword line */
	get_one_line(fl, key);

	while (*key != '$') {
		/* skip the text */
		do {
			get_one_line(fl, line);
			if (feof(fl))
				goto ackeof;
		} while (*line != '#');

		/* now count keywords */
		scan = key;
		do {
			scan = one_word(scan, next_key);
			if (*next_key)
				++total_keywords;
		} while (*next_key);

		/* get next keyword line (or $) */
		get_one_line(fl, key);

		if (feof(fl))
			goto ackeof;
	}

	return (total_keywords);

	/* No, they are not evil. -gg 6/24/98 */
  ackeof:	
	log("SYSERR: Unexpected end of help file.");
	exit(1);	/* Some day we hope to handle these things better... */
}


/*************************************************************************
 *  procedures for resetting, both play-time and boot-time	 	 *
 *************************************************************************/

/* create a character, and add it to the char list */
struct char_data *create_char(void)
{
	struct char_data *ch;

	CREATE(ch, struct char_data, 1);
	clear_char(ch);
	ch->next = character_list;
	character_list = ch;

	return (ch);
}


/*************************************************************************
 *  stuff related to the save/load player system				 *
 *************************************************************************/

long get_ptable_by_name(const char *name)
{
	int i;

	for (i = 0; i <= top_of_p_table; i++)
		if (!str_cmp(player_table[i].name, name))
			return (i);

	return (-1);
}


long get_id_by_name(const char *name)
{
	int i;

	for (i = 0; i <= top_of_p_table; i++)
		if (!str_cmp(player_table[i].name, name))
			return (player_table[i].id);

	return (-1);
}


char *get_name_by_id(long id)
{
	int i;

	for (i = 0; i <= top_of_p_table; i++)
		if (player_table[i].id == id)
			return (player_table[i].name);

	return (NULL);
}


#define NUM_OF_SAVE_THROWS	5

/* new load_char reads ascii pfiles */
/* Load a char, TRUE if loaded, FALSE if not */
int load_char(char *name, struct char_data *ch)
{
	int id, num = 0, num2 = 0, num3 = 0, num4 = 0, num5 = 0, i;
	FBFILE *fl;
	char filename[40];
	char buf[128], line[MAX_INPUT_LENGTH + 1], tag[6];
	char str1[READ_SIZE];
	struct affected_type af;


	if ((id = find_name(name)) < 0)
		return (-1);
	else {
		sprintf(filename, "%s%s%c%s%s%s", 
				  PLR_PREFIX, SLASH, *player_table[id].name, SLASH, 
				  player_table[id].name, PLR_SUFFIX);
		if(!(fl = fbopen(filename, FB_READ))) {
			mudlog(NRM, LVL_GOD, TRUE, "SYSERR: Couldn't open player file %s", 
					 filename);
			return (-1);
		}

		/* character initializations */
		/* initializations necessary to keep some things straight */
		ch->affected = NULL;
		for (i = 1; i <= MAX_SKILLS; i++)
			GET_SKILL(ch, i) = 0;
		GET_SEX(ch) = PFDEF_SEX;
		GET_CLASS(ch) = PFDEF_CLASS;
		GET_RACE(ch) = PFDEF_RACE;
		GET_LEVEL(ch) = PFDEF_LEVEL;
		GET_HOME(ch) = PFDEF_HOMETOWN;
		GET_HEIGHT(ch) = PFDEF_HEIGHT;
		GET_WEIGHT(ch) = PFDEF_WEIGHT;
		GET_ALIGNMENT(ch) = PFDEF_ALIGNMENT;
		EMPTY(PLR_FLAGS(ch));
		EMPTY(AFF_FLAGS(ch));
		for (i = 0; i < NUM_OF_SAVE_THROWS; i++)
			GET_SAVE(ch, i) = PFDEF_SAVETHROW;
		GET_LOADROOM(ch) = PFDEF_LOADROOM;
		GET_INVIS_LEV(ch) = PFDEF_INVISLEV;
		GET_FREEZE_LEV(ch) = PFDEF_FREEZELEV;
		GET_WIMP_LEV(ch) = PFDEF_WIMPLEV;
		GET_COND(ch, FULL) = PFDEF_HUNGER;
		GET_COND(ch, THIRST) = PFDEF_THIRST;
		GET_COND(ch, DRUNK) = PFDEF_DRUNK;
		GET_BAD_PWS(ch) = PFDEF_BADPWS;
		EMPTY(PRF_FLAGS(ch));
		GET_PRACTICES(ch) = PFDEF_PRACTICES;
		GET_MONEY(ch, GOLD_COINS) = PFDEF_GOLD;
		GET_MONEY(ch, COPPER_COINS) = PFDEF_COPPER;
		GET_MONEY(ch, SILVER_COINS) = PFDEF_SILVER;
		GET_MONEY(ch, PLAT_COINS) = PFDEF_PLAT;
		GET_BANK_GOLD(ch) = PFDEF_BANK;
		GET_EXP(ch) = PFDEF_EXP;
		GET_HITROLL(ch) = PFDEF_HITROLL;
		GET_DAMROLL(ch) = PFDEF_DAMROLL;
		GET_AC(ch) = PFDEF_AC;
		ch->real_abils.str = PFDEF_STR;
		ch->real_abils.str_add = PFDEF_STRADD;
		ch->real_abils.dex = PFDEF_DEX;
		ch->real_abils.intel = PFDEF_INT;
		ch->real_abils.wis = PFDEF_WIS;
		ch->real_abils.con = PFDEF_CON;
		ch->real_abils.cha = PFDEF_CHA;
		GET_HIT(ch) = PFDEF_HIT;
		GET_MAX_HIT(ch) = PFDEF_MAXHIT;
		GET_MANA(ch) = PFDEF_MANA;
		GET_MAX_MANA(ch) = PFDEF_MAXMANA;
		GET_MOVE(ch) = PFDEF_MOVE;
		GET_MAX_MOVE(ch) = PFDEF_MAXMOVE;

		while (fbgetline(fl, line)) {
			tag_argument(line, tag);
			num = atoi(line);

			switch (*tag) {
				case 'A':
					if (!strcmp(tag, "Ac  "))
						GET_AC(ch) = num;
					else if (!strcmp(tag, "Act "))
						ASCIICONV(PLR_FLAGS(ch), line);
					else if (!strcmp(tag, "Aff "))
						ASCIICONV(AFF_FLAGS(ch), line);
					else if (!strcmp(tag, "Affs")) {
						i = 0;
						do {
							fbgetline(fl, line);
							sscanf(line, "%d %d %d %d %s", &num, &num2, &num3, &num4, &str1);
							if (num > 0) {
								af.type = num;
								af.duration = num2;
								af.modifier = num3;
								af.location = num4;
								ASCIICONV(af.bitvector, str1);
								affect_to_char(ch, &af);
								i++;
							}
						} while (num != 0);
					} else if(!strcmp(tag, "Alin"))
						GET_ALIGNMENT(ch) = num;
					break;
				case 'B':
					if (!strcmp(tag, "Badp"))
						GET_BAD_PWS(ch) = num;
					else if (!strcmp(tag, "Bank"))
						GET_BANK_GOLD(ch) = num;
					else if (!strcmp(tag, "Brth"))
						ch->player.time.birth = num;
					break;
				case 'C':
					if (!strcmp(tag, "Cha "))
						ch->real_abils.cha = num;
					else if (!strcmp(tag, "Clas"))
						GET_CLASS(ch) = num;
					else if (!strcmp(tag, "Con "))
						ch->real_abils.con = num;
					else if(!strcmp(tag, "Copr"))
						add_money_to_char(ch, num, COPPER_COINS);
					else if (!strcmp(tag, "Cnt "))
						GET_CONTACT(ch) = strdup(line);
					break;
				case 'D':
					if (!strcmp(tag, "Desc")) 
						ch->player.description = fbgetstring(fl);
					else if (!strcmp(tag, "Dex "))
						ch->real_abils.dex = num;
					else if (!strcmp(tag, "Drnk"))
						GET_COND(ch, DRUNK) = num;
					else if (!strcmp(tag, "Drol"))
						GET_DAMROLL(ch) = num;
					break;
				case 'E':
					if(!strcmp(tag, "Exp "))
						GET_EXP(ch) = num;
					break;
				case 'F':
					if(!strcmp(tag, "Frez"))
						GET_FREEZE_LEV(ch) = num;
					break;
				case 'G':
					if(!strcmp(tag, "Gold"))
						add_money_to_char(ch, num, GOLD_COINS);
					break;
				case 'H':
					if (!strcmp(tag, "Hit ")) {
						sscanf(line, "%d/%d", &num, &num2);
						GET_HIT(ch) = num;
						GET_MAX_HIT(ch) = num2;
					} else if (!strcmp(tag, "Hite"))
						GET_HEIGHT(ch) = num;
					else if (!strcmp(tag, "Home"))
						GET_HOME(ch) = num;
					else if (!strcmp(tag, "Host"))
						ch->player_specials->host = strdup(line);
					else if (!strcmp(tag, "Hrol"))
						GET_HITROLL(ch) = num;
					else if (!strcmp(tag, "Hung"))
						GET_COND(ch, FULL) = num;
					break;
				case 'I':
					if (!strcmp(tag, "Id  "))
						GET_IDNUM(ch) = num;
					else if (!strcmp(tag, "Int "))
						ch->real_abils.intel = num;
					else if (!strcmp(tag, "Invs"))
						GET_INVIS_LEV(ch) = num;
					break;
				case 'L':
					if (!strcmp(tag, "Last"))
						ch->player.time.logon = num;
					else if (!strcmp(tag, "Lern"))
						GET_PRACTICES(ch) = num;
					else if (!strcmp(tag, "Levl"))
						GET_LEVEL(ch) = num;
					break;
				case 'M':
					if (!strcmp(tag, "Mana")) {
						sscanf(line, "%d/%d", &num, &num2);
						GET_MANA(ch) = num;
						GET_MAX_MANA(ch) = num2;
					} else if (!strcmp(tag, "Move")) {
						sscanf(line, "%d/%d", &num, &num2);
						GET_MOVE(ch) = num;
						GET_MAX_MOVE(ch) = num2;
					}
					break;
				case 'N':
					if (!strcmp(tag, "Name"))
						GET_PC_NAME(ch) = strdup(line);
					break;
				case 'P':
					if(!strcmp(tag, "Pass"))
						strcpy(GET_PASSWD(ch), line);
					else if (!strcmp(tag, "Plyd"))
						ch->player.time.played = num;
#ifdef ASCII_SAVE_POOFS
					else if (!strcmp(tag, "PfIn"))
						POOFIN(ch) = strdup(line);
					else if (!strcmp(tag, "PfOt"))
						POOFOUT(ch) = strdup(line);
#endif
					else if(!strcmp(tag, "Plat"))
						add_money_to_char(ch, num, PLAT_COINS);
					else if (!strcmp(tag, "Pref"))
						ASCIICONV(PRF_FLAGS(ch), line);
					break;
				case 'R':
					if (!strcmp(tag, "Room"))
						GET_LOADROOM(ch) = num;
					else if (!strcmp(tag, "Race"))
						GET_RACE(ch) = num;
					break;
				case 'S':
					if (!strcmp(tag, "Sex "))
						GET_SEX(ch) = num;
					else if (!strcmp(tag, "Shar"))
						GET_SHARES(ch) = num;
					else if (!strcmp(tag, "Skil")) {
						do {
							fbgetline(fl, line);
							sscanf(line, "%d %d", &num, &num2);
							if(num != 0)
								GET_SKILL(ch, num) = num2;
						} while (num != 0);
					}
					else if(!strcmp(tag, "Slvr"))
						add_money_to_char(ch, num, SILVER_COINS);
					else if (!strcmp(tag, "Str ")) {
						sscanf(line, "%d/%d", &num, &num2);
						ch->real_abils.str = num;
						ch->real_abils.str_add = num2;
					}
					break;
				case 'T':
					if(!strcmp(tag, "Thir"))
						GET_COND(ch, THIRST) = num;
					else if(!strcmp(tag, "Thr1"))
						GET_SAVE(ch, 0) = num;
					else if(!strcmp(tag, "Thr2"))
						GET_SAVE(ch, 1) = num;
					else if(!strcmp(tag, "Thr3"))
						GET_SAVE(ch, 2) = num;
					else if(!strcmp(tag, "Thr4"))
						GET_SAVE(ch, 3) = num;
					else if(!strcmp(tag, "Thr5"))
						GET_SAVE(ch, 4) = num;
					else if(!strcmp(tag, "Titl"))
						GET_TITLE(ch) = strdup(line);
					break;
				case 'W':
					if(!strcmp(tag, "Wate"))
						GET_WEIGHT(ch) = num;
					else if(!strcmp(tag, "Wimp"))
						GET_WIMP_LEV(ch) = num;
					else if(!strcmp(tag, "Wis "))
						ch->real_abils.wis = num;
					break;
				default:
					sprintf(buf, "SYSERR: Unknown tag %s in pfile %s", tag, name);
			}
		}
	}

	affect_total(ch);

	/* initialization for imms */
	if (GET_LEVEL(ch) >= LVL_IMMORT) {
		for (i = 1; i <= MAX_SKILLS; i++)
			GET_SKILL(ch, i) = 100;
		GET_COND(ch, FULL) = -1;
		GET_COND(ch, THIRST) = -1;
		GET_COND(ch, DRUNK) = -1;
	}

	fbclose(fl);
	return(id);
}

/* remove ^M's from file output */
/* There may be a similar function in Oasis (and I'm sure it's part of
 * obuild).  Remove this if you get a multiple definition error or if
 * it you want to use a substitute
*/
void kill_ems(char *str)
{
	char *ptr1, *ptr2, *tmp;
	tmp = str;
	ptr1 = str;
	ptr2 = str;
	while (*ptr1) {
		if ((*(ptr2++) = *(ptr1++)) == '\r')
			if (*ptr1 == '\r')
				ptr1++;
	}
	*ptr2 = '\0';
}
 

/*
 * write the vital data of a player to the player file
 *
 * NOTE: load_room should be an *RNUM* now.  It is converted to a vnum here.
 */
/* This is the ascii pfiles save routine */
void save_char(struct char_data * ch)
{
	FBFILE *fl;
	char outname[40], bits[127], buf[MAX_STRING_LENGTH];
	int i, j, id, save_index = FALSE;
	struct affected_type *aff, tmp_aff[MAX_AFFECT];
	struct obj_data *char_eq[NUM_WEARS];

	if (IS_NPC(ch) || GET_PFILEPOS(ch) < 0)
		return;
	/* This version of save_char allows ch->desc to be null (to allow
	 * "set file" to use it).  This causes the player's last host and
	 *	probably last login to null out.
	 */

	/*strcpy(player.pwd, GET_PASSWD(ch));*/

	for (i = 0; (*(bits + i) = LOWER(*(GET_NAME(ch) + i))); i++)
		;
	sprintf(outname, "%s%s%c%s%s%s", 
			  PLR_PREFIX, SLASH, *bits, SLASH, bits, PLR_SUFFIX);

	if (!(fl = fbopen(outname, FB_WRITE))) {
		mudlog(NRM, LVL_GOD, TRUE, "SYSERR: Couldn't open player file %s "
				 "for write", outname);
		return;
	}
	/* remove affects from eq and spells (from char_to_store) */
	/* Unaffect everything a character can be affected by */

	for (i = 0; i < NUM_WEARS; i++) {
		if (GET_EQ(ch, i))
			char_eq[i] = unequip_char(ch, i);
		else
			char_eq[i] = NULL;
	}

	for (aff = ch->affected, i = 0; i < MAX_AFFECT; i++) {
		if (aff) {
			tmp_aff[i] = *aff;
			tmp_aff[i].next = 0;
			aff = aff->next;
		} else {
			tmp_aff[i].type = 0;	/* Zero signifies not used */
			tmp_aff[i].duration = 0;
			tmp_aff[i].modifier = 0;
			tmp_aff[i].location = 0;
			EMPTY(tmp_aff[i].bitvector);
			tmp_aff[i].next = 0;
		}
	}

	/* remove the affections so that the raw values are stored;
	 * otherwise the effects are doubled when the char logs back in. */
	while (ch->affected)
		affect_remove(ch, ch->affected);

	if ((i >= MAX_AFFECT) && aff && aff->next)
		log("SYSERR: WARNING: OUT OF STORE ROOM FOR AFFECTED TYPES!!!");

	ch->aff_abils = ch->real_abils;
	/* end char_to_store code */


	if(GET_NAME(ch))
		fbprintf(fl, "Name: %s\n", GET_NAME(ch));
	if(GET_PASSWD(ch))
		fbprintf(fl, "Pass: %s\n", GET_PASSWD(ch));
	if(GET_TITLE(ch))
		fbprintf(fl, "Titl: %s\n", GET_TITLE(ch));
	if(ch->player.description && *ch->player.description) {
		strcpy(buf, ch->player.description);
		kill_ems(buf);
		fbprintf(fl, "Desc:\n%s~\n", buf);
	}
#ifdef ASCII_SAVE_POOFS
	if(POOFIN(ch))
		fbprintf(fl, "PfIn: %s\n", POOFIN(ch));
	if(POOFOUT(ch))
		fbprintf(fl, "PfOt: %s\n", POOFOUT(ch));
#endif
	if (GET_CONTACT(ch))
		fbprintf(fl, "Cnt : %s\n", GET_CONTACT(ch));
	if(GET_SEX(ch) != PFDEF_SEX)
		fbprintf(fl, "Sex : %d\n", GET_SEX(ch)); 
	if(GET_CLASS(ch) != PFDEF_CLASS)
		fbprintf(fl, "Clas: %d\n", GET_CLASS(ch)); 
	if(GET_RACE(ch) != PFDEF_RACE)
		fbprintf(fl, "Race: %d\n", GET_RACE(ch)); 
	if(GET_LEVEL(ch) != PFDEF_LEVEL)
		fbprintf(fl, "Levl: %d\n", GET_LEVEL(ch));
	if(GET_HOME(ch) != PFDEF_HOMETOWN)
		fbprintf(fl, "Home: %d\n", GET_HOME(ch));
	fbprintf(fl, "Brth: %d\n", ch->player.time.birth);
	fbprintf(fl, "Plyd: %d\n", ch->player.time.played);
	fbprintf(fl, "Last: %d\n", ch->player.time.logon);
	if(ch->player_specials->host)
		fbprintf(fl, "Host: %s\n", ch->player_specials->host);
	if(GET_HEIGHT(ch) != PFDEF_HEIGHT)
		fbprintf(fl, "Hite: %d\n", GET_HEIGHT(ch));
	if(GET_WEIGHT(ch) != PFDEF_HEIGHT)
		fbprintf(fl, "Wate: %d\n", GET_WEIGHT(ch));

	if(GET_ALIGNMENT(ch) != PFDEF_ALIGNMENT)
		fbprintf(fl, "Alin: %d\n", GET_ALIGNMENT(ch));
	fbprintf(fl, "Id  : %d\n", GET_IDNUM(ch));
	if(!ISEMPTY(PLR_FLAGS(ch))) {
		strlcpy(bits, struct2ascii((char *)&PLR_FLAGS(ch), sizeof(PLR_FLAGS(ch))), sizeof(bits));
		fbprintf(fl, "Act : %s\n", bits);
	}
	if(!ISEMPTY(AFF_FLAGS(ch))) {
		strlcpy(bits, struct2ascii((char *)&AFF_FLAGS(ch), sizeof(AFF_FLAGS(ch))), sizeof(bits));
		fbprintf(fl, "Aff : %s\n", bits);
	}
	if(GET_SAVE(ch, 0) != PFDEF_SAVETHROW)
		fbprintf(fl, "Thr1: %d\n", GET_SAVE(ch, 0));
	if(GET_SAVE(ch, 1) != PFDEF_SAVETHROW)
		fbprintf(fl, "Thr2: %d\n", GET_SAVE(ch, 1));
	if(GET_SAVE(ch, 2) != PFDEF_SAVETHROW)
		fbprintf(fl, "Thr3: %d\n", GET_SAVE(ch, 2));
	if(GET_SAVE(ch, 3) != PFDEF_SAVETHROW)
		fbprintf(fl, "Thr4: %d\n", GET_SAVE(ch, 3));
	if(GET_SAVE(ch, 4) != PFDEF_SAVETHROW)
		fbprintf(fl, "Thr5: %d\n", GET_SAVE(ch, 4));

	if(GET_LEVEL(ch) < LVL_IMMORT) {
		fbprintf(fl, "Skil:\n");
		for(i = 1; i <= MAX_SKILLS; i++) {
			if(GET_SKILL(ch, i))
				fbprintf(fl, "%d %d\n", i, GET_SKILL(ch, i));
		}
		fbprintf(fl, "0 0\n");
	}
	if(GET_WIMP_LEV(ch) != PFDEF_WIMPLEV)
		fbprintf(fl, "Wimp: %d\n", GET_WIMP_LEV(ch));
	if(GET_FREEZE_LEV(ch) != PFDEF_FREEZELEV)
		fbprintf(fl, "Frez: %d\n", GET_FREEZE_LEV(ch));
	if(GET_INVIS_LEV(ch) != PFDEF_INVISLEV)
		fbprintf(fl, "Invs: %d\n", GET_INVIS_LEV(ch));
	if(GET_LOADROOM(ch) != PFDEF_LOADROOM)
		fbprintf(fl, "Room: %d\n", GET_LOADROOM(ch));
	if(!ISEMPTY(PRF_FLAGS(ch))) {
		strlcpy(bits, struct2ascii((char *)&PRF_FLAGS(ch), sizeof(PRF_FLAGS(ch))), sizeof(bits));
		fbprintf(fl, "Pref: %s\n", bits);
	}
	if(GET_BAD_PWS(ch) != PFDEF_BADPWS)
		fbprintf(fl, "Badp: %d\n", GET_BAD_PWS(ch));
	if(GET_COND(ch, FULL) != PFDEF_HUNGER && GET_LEVEL(ch) < LVL_IMMORT)
		fbprintf(fl, "Hung: %d\n", GET_COND(ch, FULL));
	if(GET_COND(ch, THIRST) != PFDEF_THIRST && GET_LEVEL(ch) < LVL_IMMORT)
		fbprintf(fl, "Thir: %d\n", GET_COND(ch, THIRST));
	if(GET_COND(ch, DRUNK) != PFDEF_DRUNK && GET_LEVEL(ch) < LVL_IMMORT)
		fbprintf(fl, "Drnk: %d\n", GET_COND(ch, DRUNK));
	if(GET_PRACTICES(ch) != PFDEF_PRACTICES)
		fbprintf(fl, "Lern: %d\n", GET_PRACTICES(ch));

	if(GET_STR(ch) != PFDEF_STR || GET_ADD(ch) != PFDEF_STRADD)
		fbprintf(fl, "Str : %d/%d\n", GET_STR(ch), GET_ADD(ch));
	if(GET_INT(ch) != PFDEF_INT)
		fbprintf(fl, "Int : %d\n", GET_INT(ch));
	if(GET_WIS(ch) != PFDEF_WIS)
		fbprintf(fl, "Wis : %d\n", GET_WIS(ch));
	if(GET_DEX(ch) != PFDEF_DEX)
		fbprintf(fl, "Dex : %d\n", GET_DEX(ch));
	if(GET_CON(ch) != PFDEF_CON)
		fbprintf(fl, "Con : %d\n", GET_CON(ch));
	if(GET_CHA(ch) != PFDEF_CHA)
		fbprintf(fl, "Cha : %d\n", GET_CHA(ch));

	if(GET_HIT(ch) != PFDEF_HIT || GET_MAX_HIT(ch) != PFDEF_MAXHIT)
		fbprintf(fl, "Hit : %d/%d\n", GET_HIT(ch), GET_MAX_HIT(ch));
	if(GET_MANA(ch) != PFDEF_MANA || GET_MAX_MANA(ch) != PFDEF_MAXMANA)
		fbprintf(fl, "Mana: %d/%d\n", GET_MANA(ch), GET_MAX_MANA(ch));
	if(GET_MOVE(ch) != PFDEF_MOVE || GET_MAX_MOVE(ch) != PFDEF_MAXMOVE)
		fbprintf(fl, "Move: %d/%d\n", GET_MOVE(ch), GET_MAX_MOVE(ch));
	if(GET_AC(ch) != PFDEF_AC)
		fbprintf(fl, "Ac  : %d\n", GET_AC(ch));
	if(GET_MONEY(ch, COPPER_COINS) != PFDEF_COPPER)
		fbprintf(fl, "Copr: %d\n", GET_MONEY(ch, COPPER_COINS));
	if(GET_MONEY(ch, SILVER_COINS) != PFDEF_SILVER)
		fbprintf(fl, "Slvr: %d\n", GET_MONEY(ch, SILVER_COINS));
	if(GET_MONEY(ch, GOLD_COINS) != PFDEF_GOLD)
		fbprintf(fl, "Gold: %d\n", GET_MONEY(ch, GOLD_COINS));
	if(GET_MONEY(ch, PLAT_COINS) != PFDEF_PLAT)
		fbprintf(fl, "Plat: %d\n", GET_MONEY(ch, PLAT_COINS));
	if(GET_BANK_GOLD(ch) != PFDEF_BANK)
		fbprintf(fl, "Bank: %d\n", GET_BANK_GOLD(ch));
	if(GET_SHARES(ch) != PFDEF_SHARES)
		fbprintf(fl, "Shar: %d\n", GET_SHARES(ch));
	if(GET_EXP(ch) != PFDEF_EXP)
		fbprintf(fl, "Exp : %d\n", GET_EXP(ch));
	if(GET_HITROLL(ch) != PFDEF_HITROLL)
		fbprintf(fl, "Hrol: %d\n", GET_HITROLL(ch));
	if(GET_DAMROLL(ch))
		fbprintf(fl, "Drol: %d\n", GET_DAMROLL(ch));

	/* affected_type */
	if(tmp_aff[0].type > 0) {
		fbprintf(fl, "Affs:\n");
		for(i = 0; i < MAX_AFFECT; i++) {
			aff = &tmp_aff[i];
			if(aff->type) {
				strlcpy(bits, struct2ascii((char *)&aff->bitvector, sizeof(aff->bitvector)), sizeof(bits));
				fbprintf(fl, "%d %d %d %d %s\n", aff->type, aff->duration,
							aff->modifier, aff->location, bits);
			}
		}
		fbprintf(fl, "0 0 0 0 0\n");
	}
	fbclose(fl);

	/* more char_to_store code to restore affects */

	/* add spell and eq affections back in now */
	for (i = 0; i < MAX_AFFECT; i++) {
		if (tmp_aff[i].type)
			affect_to_char(ch, &tmp_aff[i]);
	}

	for (i = 0; i < NUM_WEARS; i++) {
		if (char_eq[i])
			equip_char(ch, char_eq[i], i);
	}

	if ((id = find_name(GET_NAME(ch))) < 0)
		return;
 
	/* update the player in the player index */
	if(player_table[id].level != GET_LEVEL(ch)) {
		save_index = TRUE;
		player_table[id].level = GET_LEVEL(ch);
	}
	if(player_table[id].last != ch->player.time.logon) {
		save_index = TRUE;
		player_table[id].last = ch->player.time.logon;
	}
	i = player_table[id].flags;
	if(PLR_FLAGGED(ch, PLR_DELETED))
		SET_BIT(player_table[id].flags, PINDEX_DELETED);
	else
		REMOVE_BIT(player_table[id].flags, PINDEX_DELETED);
	if(PLR_FLAGGED(ch, PLR_NODELETE) || PLR_FLAGGED(ch, PLR_CRYO))
		SET_BIT(player_table[id].flags, PINDEX_NODELETE);
	else
		REMOVE_BIT(player_table[id].flags, PINDEX_NODELETE);
	if (PLR_FLAGGED(ch, PLR_FROZEN) || PLR_FLAGGED(ch, PLR_NOWIZLIST) ||
		 PLR_FLAGGED(ch, PLR_DELETED))
		j = TRUE;
	else
		j = FALSE;
	
	if (player_table[id].autowiz != j) {
      save_index = TRUE;
      player_table[id].autowiz = j;
	}

	save_player_index();

/*   affect_total(ch); unnecessary, I think !?! */
}				/* Char to store */


void save_etext(struct char_data *ch)
{
/* this will be really cool soon */
}


/*
 * Create a new entry in the in-memory index table for the player file.
 * If the name already exists, by overwriting a deleted character, then
 * we re-use the old position.
 */
int create_entry(char *name)
{
	int i, pos;

	if (top_of_p_table == -1) {	/* no table */
		CREATE(player_table, struct player_index_element, 1);
		pos = top_of_p_table = 0;
	} else if ((pos = get_ptable_by_name(name)) == -1) {	/* new name */
		i = ++top_of_p_table + 1;

		RECREATE(player_table, struct player_index_element, i);
		pos = top_of_p_table;
	}

	CREATE(player_table[pos].name, char, strlen(name) + 1);

	/* copy lowercase equivalent of name to table field */
	for (i = 0; (player_table[pos].name[i] = LOWER(name[i])); i++)
		/* Nothing */;

	return (pos);
}


/* release memory allocated for a char struct */
void free_char(struct char_data *ch)
{
	int i;
	struct alias_data *a;

	if (ch->player_specials != NULL && ch->player_specials != &dummy_mob) {
		while ((a = GET_ALIASES(ch)) != NULL) {
			GET_ALIASES(ch) = (GET_ALIASES(ch))->next;
			free_alias(a);
		}
		if (ch->player_specials->poofin)
			free(ch->player_specials->poofin);
		if (ch->player_specials->poofout)
			free(ch->player_specials->poofout);
		if (ch->player_specials->contact)
			free(ch->player_specials->contact);
		if (ch->player_specials->host)
			free(ch->player_specials->host);

		free(ch->player_specials);
		if (IS_NPC(ch))
			log("SYSERR: Mob %s (#%d) had player_specials allocated!", GET_NAME(ch), GET_MOB_VNUM(ch));
	}
	if (!IS_NPC(ch) || (IS_NPC(ch) && GET_MOB_RNUM(ch) == NOBODY)) {
		/* if this is a player, or a non-prototyped non-player, free all */
		if (GET_NAME(ch))
			free(GET_NAME(ch));
		if (ch->player.title)
			free(ch->player.title);
		if (ch->player.short_descr)
			free(ch->player.short_descr);
		if (ch->player.long_descr)
			free(ch->player.long_descr);
		if (ch->player.description)
			free(ch->player.description);
	} else if ((i = GET_MOB_RNUM(ch)) != NOBODY) {
		/* otherwise, free strings only if the string is not pointing at proto */
		if (ch->player.name && ch->player.name != mob_proto[i].player.name)
			free(ch->player.name);
		if (ch->player.title && ch->player.title != mob_proto[i].player.title)
			free(ch->player.title);
		if (ch->player.short_descr && ch->player.short_descr != mob_proto[i].player.short_descr)
			free(ch->player.short_descr);
		if (ch->player.long_descr && ch->player.long_descr != mob_proto[i].player.long_descr)
			free(ch->player.long_descr);
		if (ch->player.description && ch->player.description != mob_proto[i].player.description)
			free(ch->player.description);
	}
	while (ch->affected)
		affect_remove(ch, ch->affected);

	if (ch->desc)
		ch->desc->character = NULL;

	free(ch);
}


/* clear some of the the working variables of a char */
void reset_char(struct char_data *ch)
{
	int i;

	for (i = 0; i < NUM_WEARS; i++)
		GET_EQ(ch, i) = NULL;

	ch->followers = NULL;
	ch->master = NULL;
	IN_ROOM(ch) = NOWHERE;
	ch->carrying = NULL;
	ch->next = NULL;
	ch->next_fighting = NULL;
	ch->next_in_room = NULL;
	char_from_chair(ch);
	FIGHTING(ch) = NULL;
	ch->char_specials.position = POS_STANDING;
	ch->mob_specials.default_pos = POS_STANDING;
	ch->char_specials.carry_weight = 0;
	ch->char_specials.carry_items = 0;

	if (GET_HIT(ch) <= 0)
		GET_HIT(ch) = 1;
	if (GET_MOVE(ch) <= 0)
		GET_MOVE(ch) = 1;
	if (GET_MANA(ch) <= 0)
		GET_MANA(ch) = 1;

	ch->char_specials.hitgain = 0;
	ch->char_specials.managain = 0;
	ch->char_specials.movegain = 0;

	GET_LAST_TELL(ch) = NOBODY;
}


/* clear ALL the working variables of a char; do NOT free any space alloc'ed */
void clear_char(struct char_data *ch)
{
	memset((char *) ch, 0, sizeof(struct char_data));

	IN_ROOM(ch) = NOWHERE;
	GET_PFILEPOS(ch) = -1;
	GET_MOB_RNUM(ch) = NOBODY;
	GET_WAS_IN(ch) = NOWHERE;
	GET_POS(ch) = POS_STANDING;
	ch->mob_specials.default_pos = POS_STANDING;

	ch->char_specials.hitgain = 0;
	ch->char_specials.managain = 0;
	ch->char_specials.movegain = 0;

	GET_AC(ch) = 100;		/* Basic Armor */
	if (ch->points.max_mana < 100)
		ch->points.max_mana = 100;
}


/*
 * Called during character creation after picking character class
 * (and then never again for that character).
 */
void init_char(struct char_data *ch)
{
	int i;

	/* create a player_special structure */
	if (ch->player_specials == NULL)
		CREATE(ch->player_specials, struct player_special_data, 1);

	/* *** if this is our first player --- he be God *** */
	if (top_of_p_table == 0) {
		GET_LEVEL(ch) = LVL_IMPL;
		GET_EXP(ch) = 7000000;

		/* The implementor never goes through do_start(). */
		GET_MAX_HIT(ch) = 500;
		GET_MAX_MANA(ch) = 100;
		GET_MAX_MOVE(ch) = 82;
		GET_HIT(ch) = GET_MAX_HIT(ch);
		GET_MANA(ch) = GET_MAX_MANA(ch);
		GET_MOVE(ch) = GET_MAX_MOVE(ch);
	}

	set_title(ch, NULL);
	ch->player.short_descr = NULL;
	ch->player.long_descr = NULL;
	ch->player.description = NULL;

	ch->player.time.birth = time(0);
	ch->player.time.logon = time(0);
	ch->player.time.played = 0;

	GET_HOME(ch) = 1;
	GET_AC(ch) = 100;

	for (i = 0; i < MAX_TONGUE; i++)
		GET_TALK(ch, i) = 0;

	/*
	 * make favors for sex -- or in English, we bias the height and
	 * weight of the character depending on what gender they've chosen
	 * for themselves. While it is possible to have a tall, heavy
	 * female it's not as likely as a male.
	 *
	 * Height is in centimeters. Weight is in pounds.  The only place they're
	 * ever printed (in stock code) is SPELL_IDENTIFY.
	 */
	if (GET_SEX(ch) == SEX_MALE) {
		GET_WEIGHT(ch) = rand_number(120, 180);
		GET_HEIGHT(ch) = rand_number(160, 200); /* 5'4" - 6'8" */
	} else {
		GET_WEIGHT(ch) = rand_number(100, 160);
		GET_HEIGHT(ch) = rand_number(150, 180); /* 5'0" - 6'0" */
	}

	if ((i = get_ptable_by_name(GET_NAME(ch))) != -1)
		player_table[i].id = GET_IDNUM(ch) = ++top_idnum;
	else
		log("SYSERR: init_char: Character '%s' not found in player table.", GET_NAME(ch));

	for (i = 1; i <= MAX_SKILLS; i++) {
		if (GET_LEVEL(ch) < LVL_IMPL)
			SET_SKILL(ch, i, 0);
		else
			SET_SKILL(ch, i, 100);
	}

	EMPTY(ch->char_specials.saved.affected_by);

	for (i = 0; i < 5; i++)
		GET_SAVE(ch, i) = 0;

	ch->real_abils.intel = 25;
	ch->real_abils.wis = 25;
	ch->real_abils.dex = 25;
	ch->real_abils.str = 25;
	ch->real_abils.str_add = 100;
	ch->real_abils.con = 25;
	ch->real_abils.cha = 25;

	GET_SHARES(ch) = 0;
	ch->char_specials.hitgain = 0;
	ch->char_specials.managain = 0;
	ch->char_specials.movegain = 0;

	for (i = 0; i < 3; i++)
		GET_COND(ch, i) = (GET_LEVEL(ch) == LVL_IMPL ? -1 : 24);

	GET_LOADROOM(ch) = NOWHERE;
}


/* new functions used by ascii pfiles */

/* Separate a 4-character id tag from the data it precedes */
void tag_argument(char *argument, char *tag)
{
	char *tmp = argument, *ttag = tag, *wrt = argument;
	int i;

	for (i = 0; i < 4; i++)
		*(ttag++) = *(tmp++);
	*ttag = '\0';
  
	while (*tmp == ':' || *tmp == ' ')
		tmp++;

	while (*tmp)
		*(wrt++) = *(tmp++);

	*wrt = '\0';
}


/* This function necessary to save a seperate ascii player index */
void save_player_index(void)
{
	int i;
	char bits[64];
	FBFILE *index_file;

	if (!(index_file = fbopen(PLR_INDEX_FILE, FB_WRITE))) {
		log("SYSERR:  Could not write player index file");
		return;
	}

	for (i = 0; i <= top_of_p_table; i++) {
		if (*player_table[i].name) {
			sprintbits(player_table[i].flags, bits);
			fbprintf(index_file, "%ld %s %d %s %d %d\n", player_table[i].id,
						player_table[i].name, player_table[i].level, 
						*bits ? bits : "0", player_table[i].last,
						player_table[i].autowiz);
		}
	}
	
	fbprintf(index_file, "~\n");
	fbclose(index_file);
}


/* This is a general purpose function originally from obuild OLC,
   and there is probably a similar function in Oasis.  Feel free to
   remove this and use another
*/
/*thanks to Luis Carvalho for sending this my way..it's probably a
  lot shorter than the one I would have made :)  */
void sprintbits(long vektor,char *outstring)
{
	int i;
	char flags[53] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";

	strcpy(outstring, "");
	for (i = 0; i < 32; i++) {
		if (vektor & 1) {
			*outstring = flags[i];
			outstring++;
		}
		vektor >>= 1;
	}
	*outstring = 0;
}


/* remove_player removes all files associated with a player who is
 * self-deleted, deleted by an immortal, or deleted by the auto-wipe
 * system (if enabled).  If you add more character files, you'll want
 * to put a remover here.
 */ 
void remove_player(int pfilepos)
{
	char pfile_name[128]/*, rent_name[128]*/;

	if (!*player_table[pfilepos].name)
		return;

	remove(pfile_name);

	/* Unlink any other player-owned files here if you have them  */

	log("PCLEAN: %s Lev: %d Last: %s",
		 player_table[pfilepos].name, player_table[pfilepos].level,
		 asctime(localtime(&player_table[pfilepos].last)));

	player_table[pfilepos].name[0] = '\0';
	save_player_index();
}


void clean_pfiles(void)
{
	int i, ci, timeout;

	for (i = 0; i <= top_of_p_table; i++) {
		if (IS_SET(player_table[i].flags, PINDEX_NODELETE))
			continue;

		timeout = -1;

		for (ci = 0; 
			  ci == 0 || (pclean_criteria[ci].level > pclean_criteria[ci - 1].level);
			  ci++) {

			if ((pclean_criteria[ci].level == -1 && 
				  IS_SET(player_table[i].flags, PINDEX_DELETED)) ||
				 player_table[i].level <= pclean_criteria[ci].level) {
				timeout = pclean_criteria[ci].days;
				break;
			}
		}

		if (timeout >= 0) {
			timeout *= SECS_PER_REAL_DAY;
			if ((time(0) - player_table[i].last) > timeout)
				remove_player(i);
		}
	}  
}

/* end of ascii pfile added functions */


