#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"


/* extern vars */
extern room_vnum mortal_start_room;
extern room_vnum immort_start_room;
extern room_vnum frozen_start_room;
extern int mini_mud;

/* local functions */
void parse_room(FILE *fl, int virtual_nr);
void setup_dir(FILE *fl, int room, int dir);
void check_start_rooms(void);
void renum_world(void);
room_rnum real_room(room_vnum vnum);

/* global vars */
room_rnum r_mortal_start_room;	/* rnum of mortal start room	 */
room_rnum r_immort_start_room;	/* rnum of immort start room	 */
room_rnum r_frozen_start_room;	/* rnum of frozen start room	 */


/* load the rooms */
void parse_room(FILE *fl, int virtual_nr)
{
	static int room_nr = 0, zone = 0;
	int t[10], i;
	char line[READ_SIZE], flags[128], buf2[MAX_STRING_LENGTH], buf[128];
	struct extra_descr_data *new_descr;

	/* This really had better fit or there are other problems. */
	snprintf(buf2, sizeof(buf2), "room #%d", virtual_nr);

	if (virtual_nr < zone_table[zone].bot) {
		log("SYSERR: Room #%d is below zone %d.", virtual_nr, zone);
		exit(1);
	}
	while (virtual_nr > zone_table[zone].top)
		if (++zone > top_of_zone_table) {
			log("SYSERR: Room %d is outside of any zone.", virtual_nr);
			exit(1);
		}
	world[room_nr].zone = zone;
	world[room_nr].number = virtual_nr;
	world[room_nr].name = fread_string(fl, buf2);
	world[room_nr].description = fread_string(fl, buf2);

	if (!get_line(fl, line)) {
		log("SYSERR: Expecting roomflags/sector type of room #%d but file ended!",
			 virtual_nr);
		exit(1);
	}

	if (sscanf(line, " %d %s %d ", t, flags, t + 2) != 3) {
		log("SYSERR: Format error in roomflags/sector type of room #%d",
			 virtual_nr);
		exit(1);
	}
	/* t[0] is the zone number; ignored with the zone-file system */

	ASCIICONV(world[room_nr].room_flags, flags);
	snprintf(flags, sizeof(flags), "object #%d", virtual_nr);
/*
	check_bitvector_names(world[room_nr].room_flags, 
								 room_bits_count, flags, "room");
*/
	world[room_nr].sector_type = t[2];

	world[room_nr].func = NULL;
	world[room_nr].contents = NULL;
	world[room_nr].people = NULL;
	world[room_nr].light = 0;	/* Zero light sources */

	for (i = 0; i < NUM_OF_DIRS; i++)
		world[room_nr].dir_option[i] = NULL;

	world[room_nr].ex_description = NULL;

	snprintf(buf, sizeof(buf), "SYSERR: Format error in room #%d (expecting D/E/S)", virtual_nr);

	for (;;) {
		if (!get_line(fl, line)) {
			log("%s", buf);
			exit(1);
		}
		switch (*line) {
			case 'D':
				setup_dir(fl, room_nr, atoi(line + 1));
				break;
			case 'E':
				CREATE(new_descr, struct extra_descr_data, 1);
				new_descr->keyword = fread_string(fl, buf2);
				new_descr->description = fread_string(fl, buf2);
				new_descr->next = world[room_nr].ex_description;
				world[room_nr].ex_description = new_descr;
				break;
			case 'S':			/* end of room */
				top_of_world = room_nr++;
				return;
			default:
				log("%s", buf);
				exit(1);
		}
	}
}



/* read direction data */
void setup_dir(FILE *fl, int room, int dir)
{
	int t[5];
	char line[READ_SIZE], buf2[128];

	snprintf(buf2, sizeof(buf2), "room #%d, direction D%d", GET_ROOM_VNUM(room), dir);

	CREATE(world[room].dir_option[dir], struct room_direction_data, 1);
	world[room].dir_option[dir]->general_description = fread_string(fl, buf2);
	world[room].dir_option[dir]->keyword = fread_string(fl, buf2);

	if (!get_line(fl, line)) {
		log("SYSERR: Format error, %s", buf2);
		exit(1);
	}
	if (sscanf(line, " %d %d %d ", t, t + 1, t + 2) != 3) {
		log("SYSERR: Format error, %s", buf2);
		exit(1);
	}
	if (t[0] == 1)
		world[room].dir_option[dir]->exit_info.EX_ISDOOR = 1;
	else if (t[0] == 2) {
		world[room].dir_option[dir]->exit_info.EX_ISDOOR = 1;
		world[room].dir_option[dir]->exit_info.EX_PICKPROOF = 1;
	} else if (t[0] == 3)
		world[room].dir_option[dir]->exit_info.EX_HIDDEN = 1;
	else if (t[0] == 4) {
		world[room].dir_option[dir]->exit_info.EX_HIDDEN = 1;
		world[room].dir_option[dir]->exit_info.EX_ISDOOR = 1;
	} else if (t[0] == 5) {
		world[room].dir_option[dir]->exit_info.EX_HIDDEN = 1;
		world[room].dir_option[dir]->exit_info.EX_ISDOOR = 1;
		world[room].dir_option[dir]->exit_info.EX_PICKPROOF = 1;
	}

	world[room].dir_option[dir]->key = t[1];
	world[room].dir_option[dir]->to_room = t[2];
}


/* make sure the start rooms exist & resolve their vnums to rnums */
void check_start_rooms(void)
{
	if ((r_mortal_start_room = real_room(mortal_start_room)) == NOWHERE) {
		log("SYSERR:  Mortal start room does not exist.  Change in config.c.");
		exit(1);
	}
	if ((r_immort_start_room = real_room(immort_start_room)) == NOWHERE) {
		if (!mini_mud)
			log("SYSERR:  Warning: Immort start room does not exist.  Change in config.c.");
		r_immort_start_room = r_mortal_start_room;
	}
	if ((r_frozen_start_room = real_room(frozen_start_room)) == NOWHERE) {
		if (!mini_mud)
			log("SYSERR:  Warning: Frozen start room does not exist.  Change in config.c.");
		r_frozen_start_room = r_mortal_start_room;
	}
}


/* resolve all vnums into rnums in the world */
void renum_world(void)
{
	int room, door;

	for (room = 0; room <= top_of_world; room++)
		for (door = 0; door < NUM_OF_DIRS; door++)
			if (world[room].dir_option[door])
				if (world[room].dir_option[door]->to_room != NOWHERE)
					world[room].dir_option[door]->to_room =
						real_room(world[room].dir_option[door]->to_room);
}


/* returns the real number of the room with given virtual number */
room_rnum real_room(room_vnum vnum)
{
	room_rnum bot, top, mid;

	bot = 0;
	top = top_of_world;

	/* perform binary search on world-table */
	for (;;) {
		mid = (bot + top) / 2;

		if ((world + mid)->number == vnum)
			return (mid);
		if (bot >= top)
			return (NOWHERE);
		if ((world + mid)->number > vnum)
			top = mid - 1;
		else
			bot = mid + 1;
	}
}


int vnum_room(char *searchname, struct char_data * ch)
{
	int nr, found = 0;
	char buf[MAX_STRING_LENGTH] = "";
	size_t len = 0, nlen = 0;

	for (nr = 0; nr <= top_of_world && found <= 50; nr++) {
		if (isname(searchname, world[nr].name)) {
			nlen = snprintf(buf + len, sizeof(buf) - len, "%3d. [%5d] %s\r\n", 
								 ++found, world[nr].number, world[nr].name);

			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;
		}
	}
	page_string(ch->desc, buf, TRUE);

	return (found);
}
