/*********************************************************
 * New metaphysiscian procedure based on race and level. *
 * By Catherine Gore of DarkonMUD               May 2001 *
 * Based on meta_store by Rafiq "JurniMan" Premji        *
 * Please give credit where it's due.                    *
 *********************************************************/
 
/* 
 * Some notes: This system puts a cap on HPs, mana, and moves at 4500.
 * Our imms have 5000 across for that, so no mortal can attain that
 * amount.  It also puts a cap on the stats based on race, to give
 * some variation in the mud.  Feel free to change these cap numbers,
 * found in stat_max.
 */

/* My turn :)
 *
 * Fixed the hideous assigning of aff_abils to real_abils... BAD!
 *
 * Also, made it so only maxxed level morts (i.e. LVL_IMMORT - 1)
 * could use it.  Accordingly the procs to determine the cost were
 * much simplified.  When you let lower level players use this, you
 * create a potentially horrible situation wherein a high level player
 * gets the lower level one lots of exp and gold by grouping and the
 * lower level player, not needing as much exp or gold to use the meta
 * physician, gets really high stats while at low level.  You could do
 * all sorts of grouping restrictions to avoid this, but it really is
 * much better to just limit the meta physician to only the highest
 * level players.
 *
 *  -Edward Glamkowski   5/13/02
 *
 */
 
#include "conf.h"
#include "sysdep.h"
 
#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
 

#define HMV   0
#define STAT  1
 
/* external functions */
ACMD(do_say);
extern int level_exp(int chclass, int level);

 
/* internal functions */
int stat_max( int race, int wch );
int meta_exp( int wch, struct char_data *ch );
int meta_gold( int wch, struct char_data *ch );
 

SPECIAL(meta_phys)
{
 	int meta_choice, i, mod;
 	sh_int *hmvmod = NULL;
 	sbyte *statmod = NULL;
	char buf[MAX_STRING_LENGTH];
 	
 	struct meta_struct {
 		int gold;
 		int exp;
 		char *success;
 		int apply;
 		int min_add;
 		int max_add;
 		char *failure;
 		int max_stat;
 	} meta_stat[] = {
 		/* gold, exp, success message,
		 * apply, min to apply, max to apply, 
		 failure message, 
		 max possible */
 		{ meta_gold(HMV, ch), meta_exp(HMV, ch),
		  "Your feel healthier!\r\n", APPLY_HIT, 1, 5, 
		  "You are as healthy as possible!\r\n", 
		  stat_max(GET_RACE(ch), 1) },
 		{ meta_gold(HMV, ch), meta_exp(HMV, ch), 
		  "You feel your mana raise!\r\n", APPLY_MANA, 2, 6,
		  "You cannot summon any more magic.\r\n",
		  stat_max(GET_RACE(ch), 2) },
 		{ meta_gold(HMV, ch), meta_exp(HMV, ch), 
		  "You feel more endurance ability!\r\n", APPLY_MOVE, 3, 7,
		  "You can already move as far as possible.\r\n",
		  stat_max(GET_RACE(ch), 3) },
 		{ meta_gold(STAT, ch), meta_exp(STAT, ch), 
		  "You feel stronger!\r\n", APPLY_STR,	1, 1, 
		  "You are as naturally strong as you can be.\r\n", 
		  stat_max(GET_RACE(ch), 4) },
 		{ meta_gold(STAT, ch), meta_exp(STAT, ch),
		  "You feel faster!\r\n", APPLY_DEX, 1, 1,
		  "Sorry, you are already as agile as you can be.\r\n",
		  stat_max(GET_RACE(ch), 5) },
 		{ meta_gold(STAT, ch), meta_exp(STAT, ch), 
		  "You suddenly feel more hardy.\r\n", APPLY_CON, 1, 1, 
		  "Sorry, you are as hardy as you can be.\r\n", 
		  stat_max(GET_RACE(ch), 6) },
 		{ meta_gold(STAT, ch), meta_exp(STAT, ch),
		  "You feel smarter!\r\n", APPLY_INT, 1, 1, 
		  "Sorry, you cannot understand any more.\r\n", 
		  stat_max(GET_RACE(ch), 7) },
 		{ meta_gold(STAT, ch), meta_exp(STAT, ch), 
		  "You feel more wise!\r\n", APPLY_WIS, 1, 1, 
		  "Sorry, I cannot make you any more wise.\r\n",
		  stat_max(GET_RACE(ch), 8) },
 		{ meta_gold(STAT, ch), meta_exp(STAT, ch), 
		  "Your social skills increase!\r\n", APPLY_CHA, 1, 1,
		  "Sorry, your charisma is already godlike.\r\n", 
		  stat_max(GET_RACE(ch), 9) },

 		{ -1, -1, NULL, -1, -1, -1, NULL, -1 }
 	};
 	
 	snprintf(buf, sizeof(buf),
				"__________________________________________\r\n"
				"#  Max   Stat Raised:   Exp:      Gold:   \r\n"
				"__________________________________________\r\n"
				"1  %4d  Health         %7d  %6d\r\n"
				"2  %4d  Mana           %7d  %6d\r\n"
				"3  %4d  Movement       %7d  %6d\r\n"
				"-  ----  ------------  ---------  --------\r\n"
				"4  %2d    Strength       %7d  %6d\r\n"
				"5  %2d    Dexterity      %7d  %6d\r\n"
				"6  %2d    Constitution   %7d  %6d\r\n"
				"7  %2d    Intelligence   %7d  %6d\r\n"
				"8  %2d    Wisdom         %7d  %6d\r\n"
				"9  %2d    Charisma       %7d  %6d\r\n",
				meta_stat[0].max_stat, meta_stat[0].exp, meta_stat[0].gold, 
				meta_stat[1].max_stat, meta_stat[1].exp, meta_stat[1].gold, 
				meta_stat[2].max_stat, meta_stat[2].exp, meta_stat[2].gold, 
				meta_stat[3].max_stat, meta_stat[3].exp, meta_stat[3].gold, 
				meta_stat[4].max_stat, meta_stat[4].exp, meta_stat[4].gold, 
				meta_stat[5].max_stat, meta_stat[5].exp, meta_stat[5].gold, 
				meta_stat[6].max_stat, meta_stat[6].exp, meta_stat[6].gold, 
				meta_stat[7].max_stat, meta_stat[7].exp, meta_stat[7].gold, 
				meta_stat[8].max_stat, meta_stat[8].exp, meta_stat[8].gold);	
 	
 	if (CMD_IS("meta")) {
 		if (!*argument) {
 			send_to_char(ch, buf);
 			return (TRUE);
 		}
 		if (GET_LEVEL(ch) < LVL_IMMORT - 1) {
 			do_say(me, "You are not yet experienced enough to meta.", 0, 0);
 			return (TRUE);
 		}
 		meta_choice = atoi(argument) - 1;
		for (i = 0; meta_stat[i].gold != -1 && i != meta_choice; i++)
			; /* Loop to find out which choice */

 		if (i != meta_choice) {
 			send_to_char(ch, buf);
 			send_to_char(ch, "That is not an option.\r\n");
 			return (TRUE);
 		}
 
 		if (convert_all_to_copper(ch) < meta_stat[i].gold)
 			do_say(me, "You don't have enough gold for that.", 0, 0);
 		else if (GET_EXP(ch) < meta_stat[i].exp)
 			do_say(me, "You don't have enough experience for that.", 0, 0);
 		else {
 			switch (meta_stat[i].apply) {
				case APPLY_STR: statmod = &ch->real_abils.str; break;
				case APPLY_DEX: statmod = &ch->real_abils.dex; break;
				case APPLY_CON: statmod = &ch->real_abils.con; break;
				case APPLY_INT: statmod = &ch->real_abils.intel; break;
				case APPLY_WIS: statmod = &ch->real_abils.wis; break;
				case APPLY_CHA: statmod = &ch->real_abils.cha; break;
				case APPLY_HIT: hmvmod  = &ch->points.max_hit; break;
				case APPLY_MANA: hmvmod = &ch->points.max_mana; break;
				case APPLY_MOVE: hmvmod = &ch->points.max_move; break;
					/*
					 * Add more APPLY_xxx's here if you need more.
					 */
				default: 
					send_to_char(ch, "That is not an option.  Type 'meta' for a "
									 "list of options.\r\n");
					return (TRUE);
 			}
 			if ((hmvmod && meta_stat[i].max_stat <= *hmvmod) || 
				 (statmod && meta_stat[i].max_stat <= *statmod)) {
 				send_to_char(ch, meta_stat[i].failure);
 				return (TRUE);
 			}
 			
 			GET_EXP(ch) -= meta_stat[i].exp;
			make_change(ch, meta_stat[i].gold, BUYING);
 			mod = rand_number(meta_stat[i].min_add, meta_stat[i].max_add);

 			if (hmvmod)
 				*hmvmod += mod;
 			if (statmod)
 				*statmod += mod;

			affect_total(ch);
 			send_to_char(ch, meta_stat[i].success);

 			save_char(ch);
 		}	/* end of valid meta_stat and enough gold/exp */
 		return (TRUE);
 	}	/* end of valid meta_stat */
 
 	return (FALSE);		/* Person didn't type 'meta' */
}
 

/* Adjust races for your system.  This is based on a system with the
 * basic races + Namek, Half-Elf, and Giant */
int stat_max(int race, int wch)
{
 	switch(race) {
		case RACE_HUMAN: /* No limits - all 24's for stats, all 4000 for hmv */
         switch(wch){
				case 1:
				case 2:
				case 3:
					return 5000; /* hp, mana, move */
				default: 
					return 25; /* all stats */
			}
		case RACE_ELF:
			switch(wch){
				case 1: return 3000; /*hp*/
				case 2: return 6000; /*mana*/
				case 3: return 4000; /*move*/
				case 4: return 23; /*str*/
				case 5: return 25; /*dex*/
				case 6: return 22; /*con*/
				case 7: return 25; /*int*/
				case 8: return 24; /*wis*/
				case 9: return 24; /*cha*/
				default: 
					return 24;
			}
			break;
		case RACE_GNOME:
			switch(wch) {
				case 1: return 4000; /*hp*/
				case 2: return 5000; /*mana*/
				case 3: return 4000; /*move*/
				case 4: return 23; /*str*/
				case 5: return 25; /*dex*/
				case 6: return 24; /*con*/
				case 7: return 25; /*int*/
				case 8: return 22; /*wis*/
				case 9: return 24; /*cha*/
				default:
					return 24;
			}
			break;
		case RACE_DWARF:
			switch(wch) {
				case 1: return 6000; /*hp*/
				case 2: return 3000; /*mana*/
				case 3: return 4000; /*move*/
				case 4: return 25; /*str*/
				case 5: return 23; /*dex*/
				case 6: return 25; /*con*/
				case 7: return 24; /*int*/
				case 8: return 24; /*wis*/
				case 9: return 22; /*cha*/
				default: 
					return 24;
			}
			break;
		default: /* we shouldn't get here */
			log("SYSERR: Unknown race requested at meta_phys! Check races.c "
				 "for list of races.");
			return 0; /* someone using a weird race?  not letting them get far. */
 	}
}

 
/* How much exp it costs to raise a stat */
int meta_exp(int wch, struct char_data *ch)
{
	float mod = (wch == HMV ? 0.25 : 0.75);
	
	return (mod * level_exp(GET_CLASS(ch), GET_LEVEL(ch)));
}

 
/* How much gold it costs to raise a stat */
int meta_gold(int wch, struct char_data *ch)
{
	int mod = (wch == HMV ? 25000 : 50000);
	
	return (mod * GET_LEVEL(ch));
}
