/* ************************************************************************
*   File: structs.h                                     Part of CircleMUD *
*  Usage: header file for central structures and contstants               *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

/*
 * Intended use of this macro is to allow external packages to work with
 * a variety of CircleMUD versions without modifications.  For instance,
 * an IS_CORPSE() macro was introduced in pl13.  Any future code add-ons
 * could take into account the CircleMUD version and supply their own
 * definition for the macro if used on an older version of CircleMUD.
 * You are supposed to compare this with the macro CIRCLEMUD_VERSION()
 * in utils.h.  See there for usage.
 */
#define _CIRCLEMUD	0x030015 /* Major/Minor/Patchlevel - MMmmPP */

/*
 * If you want equipment to be automatically equipped to the same place
 * it was when players rented, set the define below to 1.  Please note
 * that this will require erasing or converting all of your rent files.
 * And of course, you have to recompile everything.  We need this feature
 * for CircleMUD 3.0 to be complete but we refuse to break binary file
 * compatibility.
 */
#define USE_AUTOEQ	1	/* TRUE/FALSE aren't defined yet. */


/* preamble *************************************************************/

/*
 * As of bpl20, it should be safe to use unsigned data types for the
 * various virtual and real number data types.  There really isn't a
 * reason to use signed anymore so use the unsigned types and get
 * 65,535 objects instead of 32,768.
 *
 * NOTE: This will likely be unconditionally unsigned later.
 */
#define CIRCLE_UNSIGNED_INDEX	0	/* 0 = signed, 1 = unsigned */

#if CIRCLE_UNSIGNED_INDEX
# define IDXTYPE	ush_int
# define NOWHERE	((IDXTYPE)~0)
# define NOTHING	((IDXTYPE)~0)
# define NOBODY		((IDXTYPE)~0)
#else
# define IDXTYPE	sh_int
# define NOWHERE	(-1)	/* nil reference for rooms	*/
# define NOTHING	(-1)	/* nil reference for objects	*/
# define NOBODY		(-1)	/* nil reference for mobiles	*/
#endif


/* Bitfields aren't the most portable thing, so change this to
 * whatever makes sense for your system setup.  -ejg */
#define BIT(name)  unsigned int name: 1

/* A special procedure function, so mobs, objs or rooms can do cool
 * stuff.  See spec_proc.c and spec_assign.c  */
#define SPECIAL(name) \
   int (name)(struct char_data *ch, void *me, int cmd, char *argument)


/* Callback function, so you can ask a person a question and store the
 * answer for use as input into another function!  */
#define C_FUNC(name) \
	void (name) (struct descriptor_data *d, char *arg, void *info)


/* For the economy code */
extern int share_value;

/* room-related defines *************************************************/


/* The cardinal directions: used as index to room_data.dir_option[] */
#define NORTH          0
#define EAST           1
#define SOUTH          2
#define WEST           3
#define UP             4
#define DOWN           5


/* Room affections */
struct room_affs {
	BIT(RAFF_FOG);
};


/* Room flags: used in room_data.room_flags */
/* WARNING: In the world files, NEVER set the bits marked "R" ("Reserved") */
struct room_flags {
	BIT(ROOM_DARK);		/* Dark				*/
	BIT(ROOM_DEATH);		/* Death trap			*/
	BIT(ROOM_NOMOB);		/* MOBs not allowed		*/
	BIT(ROOM_INDOORS);		/* Indoors			*/
	BIT(ROOM_PEACEFUL);		/* Violence not allowed		*/
	BIT(ROOM_SOUNDPROOF);		/* Shouts, gossip blocked	*/
	BIT(ROOM_NOTRACK);		/* Track won't go through	*/
	BIT(ROOM_NOMAGIC);		/* Magic not allowed		*/
	BIT(ROOM_TUNNEL);		/* room for only 1 pers		*/
	BIT(ROOM_PRIVATE);		/* Can't teleport in		*/
	BIT(ROOM_GODROOM);		/* LVL_GOD+ only allowed	*/
	BIT(ROOM_HOUSE);		/* (R) Room is a house		*/
	BIT(ROOM_HOUSE_CRASH);	/* (R) House needs saving	*/
	BIT(ROOM_ATRIUM);		/* (R) The door to a house	*/
	BIT(ROOM_OLC);		/* (R) Modifyable/!compress	*/
	BIT(ROOM_BFS_MARK);		/* (R) breath-first srch mrk	*/
	BIT(ROOM_NOTELEPORT);  /* Protects against teleport spell */
	BIT(ROOM_FAST_REGEN); /* Faster regen */
	BIT(ROOM_SLOW_REGEN); /* Slower regen */
	BIT(ROOM_NO_MAGE); /* No mages allowed    */
	BIT(ROOM_NO_CLERIC); /* No clerics allowed  */
	BIT(ROOM_NO_THIEF); /* No thieves allowed  */
	BIT(ROOM_NO_WARRIOR); /* No warriors allowed */
	BIT(ROOM_NO_RECALL); /* Cannot recall from this room */
};


/* Zone info: Used in zone_data.zone_flags */
struct zone_flags {
	BIT(ZONE_NORECALL);
	BIT(ZONE_NOSUMMON);
};


/* Exit info: used in room_data.dir_option.exit_info */
struct exit_info_types {
	BIT(EX_ISDOOR);		/* Exit is a door	*/
	BIT(EX_CLOSED);		/* The door is closed	*/
	BIT(EX_LOCKED);		/* The door is locked	*/
	BIT(EX_PICKPROOF);		/* Lock can't be picked	*/
	BIT(EX_HIDDEN);   /* Exit is hidden		*/
};


/* Sector types: used in room_data.sector_type */
#define SECT_INSIDE          0		   /* Indoors			*/
#define SECT_CITY            1		   /* In a city			*/
#define SECT_FIELD           2		   /* In a field		*/
#define SECT_FOREST          3		   /* In a forest		*/
#define SECT_HILLS           4		   /* In the hills		*/
#define SECT_MOUNTAIN        5		   /* On a mountain		*/
#define SECT_WATER_SWIM      6		   /* Swimmable water		*/
#define SECT_WATER_NOSWIM    7		   /* Water - need a boat	*/
#define SECT_FLYING          8		   /* Wheee!			*/
#define SECT_UNDERWATER	     9		   /* Underwater		*/
#define SECT_SWAMP          10
#define SECT_JUNGLE         11
#define SECT_LAVA           12
#define SECT_TUNDRA         13
#define SECT_DESERT         14

/* If you add new sectors, be sure to check out asciimap.c and update
 * it accordingly (if you are using the do_map command, that is).
 * Also update the dig command in act.other.c -ejg */
#define SECT_MAX            15

/* char and mob-related defines *****************************************/


/* PC classes */
#define CLASS_UNDEFINED	  (-1)
#define CLASS_MAGIC_USER  0
#define CLASS_CLERIC      1
#define CLASS_THIEF       2
#define CLASS_WARRIOR     3

#define NUM_CLASSES	  4  /* This must be the number of classes!! */

/* NPC classes (currently unused - feel free to implement!) */
#define CLASS_OTHER       0
#define CLASS_UNDEAD      1
#define CLASS_HUMANOID    2
#define CLASS_ANIMAL      3
#define CLASS_DRAGON      4
#define CLASS_GIANT       5


/* PC races */
#define RACE_UNDEFINED    (-1)
#define RACE_HUMAN        0
#define RACE_ELF          1
#define RACE_GNOME        2
#define RACE_DWARF        3

#define NUM_RACES         4


/* Sex */
#define SEX_NEUTRAL   0
#define SEX_MALE      1
#define SEX_FEMALE    2


/* Positions */
#define POS_DEAD       0	/* dead			*/
#define POS_MORTALLYW  1	/* mortally wounded	*/
#define POS_INCAP      2	/* incapacitated	*/
#define POS_STUNNED    3	/* stunned		*/
#define POS_SLEEPING   4	/* sleeping		*/
#define POS_RESTING    5	/* resting		*/
#define POS_SITTING    6	/* sitting		*/
#define POS_FIGHTING   7	/* fighting		*/
#define POS_STANDING   8	/* standing		*/
#define POS_FLYING     9   /* flying      */


/* Player flags: used by char_data.char_specials.act */
struct playerflags {
	BIT(PLR_KILLER);		/* Player is a player-killer		*/
	BIT(PLR_THIEF);		/* Player is a player-thief		*/
	BIT(PLR_FROZEN);		/* Player is frozen			*/
	BIT(PLR_DONTSET);		/* Don't EVER set (ISNPC bit)		*/
	BIT(PLR_CRASH);		/* Player needs to be crash-saved	*/
	BIT(PLR_SITEOK);		/* Player has been site-cleared		*/
	BIT(PLR_NOSHOUT);		/* Player not allowed to shout/goss	*/
	BIT(PLR_NOTITLE);		/* Player not allowed to set title	*/
	BIT(PLR_DELETED);		/* Player deleted - space reusable	*/
	BIT(PLR_LOADROOM);		/* Player uses nonstandard loadroom	*/
	BIT(PLR_NOWIZLIST);		/* Player shouldn't be on wizlist	*/
	BIT(PLR_NODELETE);		/* Player shouldn't be deleted		*/
	BIT(PLR_INVSTART);		/* Player should enter game wizinvis	*/
	BIT(PLR_CRYO);		/* Player is cryo-saved (purge prog)	*/
	BIT(PLR_NOTDEADYET);  /* (R) Player being extracted.	*/
	BIT(PLR_FISHING);  /* Player has a line in the water   */
	BIT(PLR_FISH_ON);  /* Player has a fish on their line  */
};


/* Mobile flags: used by char_data.char_specials.act */
struct mobflags {
	BIT(MOB_SPEC);		/* Mob has a callable spec-proc		*/
	BIT(MOB_SENTINEL);		/* Mob should not move			*/
	BIT(MOB_SCAVENGER);		/* Mob picks up stuff on the ground	*/
	BIT(MOB_ISNPC);		/* (R) Automatically set on all Mobs	*/
	BIT(MOB_AWARE);		/* Mob can't be backstabbed		*/
	BIT(MOB_AGGRESSIVE);		/* Mob hits players in the room		*/
	BIT(MOB_STAY_ZONE);		/* Mob shouldn't wander out of zone	*/
	BIT(MOB_WIMPY);		/* Mob flees if severely injured	*/
	BIT(MOB_AGGR_EVIL);		/* auto attack evil PC's		*/
	BIT(MOB_AGGR_GOOD);		/* auto attack good PC's		*/
	BIT(MOB_AGGR_NEUTRAL);	/* auto attack neutral PC's		*/
	BIT(MOB_MEMORY);		/* remember attackers if attacked	*/
	BIT(MOB_HELPER);		/* attack PCs fighting other NPCs	*/
	BIT(MOB_NOCHARM);		/* Mob can't be charmed			*/
	BIT(MOB_NOSUMMON);		/* Mob can't be summoned		*/
	BIT(MOB_NOSLEEP);		/* Mob can't be slept			*/
	BIT(MOB_NOBASH);		/* Mob can't be bashed (e.g. trees)	*/
	BIT(MOB_NOBLIND);		/* Mob can't be blinded			*/
	BIT(MOB_NOTDEADYET); /* (R) Mob being extracted.		*/
	BIT(MOB_SMART); /* Attacks char with lowest hit points */
	BIT(MOB_HYPER_AGGR); /* Attacks as soon as char enters room */
	BIT(MOB_MERCY); /* Mob will try not to kill chars */
	BIT(MOB_MOUNTABLE); /* Mob can be mounted and ridden */
};


/* Preference flags: used by char_data.player_specials.pref */
struct preferances {
	BIT(PRF_BRIEF);		/* Room descs won't normally be shown	*/
	BIT(PRF_COMPACT);		/* No extra CRLF pair before prompts	*/
	BIT(PRF_DEAF);		/* Can't hear shouts			*/
	BIT(PRF_NOTELL);		/* Can't receive tells			*/
	BIT(PRF_DISPHP);		/* Display hit points in prompt		*/
	BIT(PRF_DISPMANA);		/* Display mana points in prompt	*/
	BIT(PRF_DISPMOVE);		/* Display move points in prompt	*/
	BIT(PRF_AUTOEXIT);		/* Display exits in a room		*/
	BIT(PRF_NOHASSLE);		/* Aggr mobs won't attack		*/
	BIT(PRF_QUEST);		/* On quest				*/
	BIT(PRF_SUMMONABLE);		/* Can be summoned			*/
	BIT(PRF_NOREPEAT);		/* No repetition of comm commands	*/
	BIT(PRF_HOLYLIGHT);		/* Can see in dark			*/
	BIT(PRF_COLOR_1);		/* Color (low bit)			*/
	BIT(PRF_COLOR_2);		/* Color (high bit)			*/
	BIT(PRF_NOWIZ);		/* Can't hear wizline			*/
	BIT(PRF_LOG1);		/* On-line System Log (low bit)		*/
	BIT(PRF_LOG2);		/* On-line System Log (high bit)	*/
	BIT(PRF_NOAUCT);		/* Can't hear auction channel		*/
	BIT(PRF_NOGOSS);		/* Can't hear gossip channel		*/
	BIT(PRF_NOGRATZ);		/* Can't hear grats channel		*/
	BIT(PRF_ROOMFLAGS);		/* Can see room flags (ROOM_x)		*/
	BIT(PRF_AUTOSPLIT);		/* Split all gold picked up */
	BIT(PRF_AUTOLOOT);		/* Take loot from mob you kill */
	BIT(PRF_DISPEXP);		/* Display exp to level in prompt */
	BIT(PRF_AFK);		/* Away From Keyboard */
	BIT(PRF_AUTOASSIST);		/* Auto assist group members */
	BIT(PRF_AUTOSAC);       /* Auto sacrifice corpses */
	BIT(PRF_DISPGOLD);      /* Display gold in prompt */
};


/* Affect bits: used in char_data.char_specials.saved.affected_by */
/* WARNING: In the world files, NEVER set the bits marked "R" ("Reserved") */
struct affections {
	BIT(AFF_BLIND);		/* (R) Char is blind		*/
	BIT(AFF_INVISIBLE);		/* Char is invisible		*/
	BIT(AFF_DETECT_ALIGN);	/* Char is sensitive to align	*/
	BIT(AFF_DETECT_INVIS);	/* Char can see invis chars  	*/
	BIT(AFF_DETECT_MAGIC);	/* Char is sensitive to magic	*/
	BIT(AFF_SENSE_LIFE);		/* Char can sense hidden life	*/
	BIT(AFF_WATERWALK);		/* Char can walk on water	*/
	BIT(AFF_SANCTUARY);		/* Char protected by sanct.	*/
	BIT(AFF_GROUP);		/* (R) Char is grouped		*/
	BIT(AFF_CURSE);		/* Char is cursed		*/
	BIT(AFF_INFRAVISION);		/* Char can see in dark		*/
	BIT(AFF_POISON);		/* (R) Char is poisoned		*/
	BIT(AFF_PROTECT_EVIL);	/* Char protected from evil  	*/
	BIT(AFF_PROTECT_GOOD);	/* Char protected from good  	*/
	BIT(AFF_SLEEP);		/* (R) Char magically asleep	*/
	BIT(AFF_NOTRACK);		/* Char can't be tracked	*/
	BIT(AFF_MIRROR_IMAGE);	/* Char affected by mirror image */
	BIT(AFF_FLY);		/* Char is flying! */
	BIT(AFF_SNEAK);		/* Char can move quietly	*/
	BIT(AFF_HIDE);		/* Char is hidden		*/
	BIT(AFF_STUN);		/* Char is stunned */
	BIT(AFF_CHARM);		/* Char is charmed		*/
	BIT(AFF_REGEN);      /* Char regens faster */
	BIT(AFF_TAMED);      /* Char has been tamed (DAK) */
	BIT(AFF_AWARE);      /* Char has heightened awareness */
};

#define NUM_AFF_FLAGS 24


/* Modes of connectedness: used by descriptor_data.state */
#define CON_PLAYING       0	/* Playing - Nominal state		*/
#define CON_CLOSE         1	/* User disconnect, remove character.	*/
#define CON_GET_NAME      2	/* By what name ..?			*/
#define CON_NAME_CNFRM    3	/* Did I get that right, x?		*/
#define CON_PASSWORD      4	/* Password:				*/
#define CON_NEWPASSWD     5	/* Give me a password for x		*/
#define CON_CNFPASSWD     6	/* Please retype password:		*/
#define CON_QSEX          7	/* Sex?					*/
#define CON_QCLASS        8	/* Class?				*/
#define CON_RMOTD         9	/* PRESS RETURN after MOTD		*/
#define CON_MENU         10	/* Your choice: (main menu)		*/
#define CON_CHPWD_GETOLD 11   /* Changing passwd: get old      */
#define CON_CHPWD_GETNEW 12   /* Changing passwd: get new      */
#define CON_CHPWD_VRFY   13   /* Verify new password        */
#define CON_DELCNF1      14   /* Delete confirmation 1      */
#define CON_DELCNF2      15   /* Delete confirmation 2      */
#define CON_DISCONNECT   16   /* In-game link loss (leave character) */
#define CON_QRACE        17   /* Race?                      */
#define CON_IDCONING     18		/* waiting for ident connection */
#define CON_IDCONED      19		/* ident connection complete    */
#define CON_IDREADING    20		/* waiting to read ident sock   */
#define CON_IDREAD       21		/* ident results read           */
#define CON_ASKNAME      22		/* Ask user for name            */
#define CON_LINE_INPUT   23
#define CON_QSTATS       25      /* In-game disconnection        */

/* Character equipment positions: used as index for char_data.equipment[] */
/* NOTE: Don't confuse these constants with the ITEM_ bitvectors
   which control the valid places you can wear a piece of equipment */
#define WEAR_LIGHT      0
#define WEAR_FINGER_R   1
#define WEAR_FINGER_L   2
#define WEAR_NECK_1     3
#define WEAR_NECK_2     4
#define WEAR_BODY       5
#define WEAR_HEAD       6
#define WEAR_LEGS       7
#define WEAR_FEET       8
#define WEAR_HANDS      9
#define WEAR_ARMS      10
#define WEAR_SHIELD    11
#define WEAR_ABOUT     12
#define WEAR_WAIST     13
#define WEAR_WRIST_R   14
#define WEAR_WRIST_L   15
#define WEAR_WIELD     16
#define WEAR_HOLD      17
#define WEAR_DWIELD    18

#define NUM_WEARS      19	/* This must be the # of eq positions!! */


/* object-related defines ********************************************/


/* Item types: used by obj_data.obj_flags.type_flag */
#define ITEM_LIGHT      1		/* Item is a light source	*/
#define ITEM_SCROLL     2		/* Item is a scroll		*/
#define ITEM_WAND       3		/* Item is a wand		*/
#define ITEM_STAFF      4		/* Item is a staff		*/
#define ITEM_WEAPON     5		/* Item is a weapon		*/
#define ITEM_FIREWEAPON 6		/* Unimplemented		*/
#define ITEM_MISSILE    7		/* Unimplemented		*/
#define ITEM_TREASURE   8		/* Item is a treasure, not gold	*/
#define ITEM_ARMOR      9		/* Item is armor		*/
#define ITEM_POTION    10 		/* Item is a potion		*/
#define ITEM_WORN      11		/* Unimplemented		*/
#define ITEM_OTHER     12		/* Misc object			*/
#define ITEM_TRASH     13		/* Trash - shopkeeps won't buy	*/
#define ITEM_TRAP      14		/* Unimplemented		*/
#define ITEM_CONTAINER 15		/* Item is a container		*/
#define ITEM_NOTE      16		/* Item is note 		*/
#define ITEM_DRINKCON  17		/* Item is a drink container	*/
#define ITEM_KEY       18		/* Item is a key		*/
#define ITEM_FOOD      19		/* Item is food			*/
#define ITEM_MONEY     20		/* Item is money (gold)		*/
#define ITEM_PEN       21		/* Item is a pen		*/
#define ITEM_BOAT      22		/* Item is a boat		*/
#define ITEM_FOUNTAIN  23		/* Item is a fountain		*/
#define ITEM_CHAIR     24     /* Item is a chair   */
#define ITEM_POLE      25     /* Fishing Pole         */
#define ITEM_THROW     26     /* Item can be thrown as weapon */
#define ITEM_GRENADE   27     /* Item is a grenade      */
#define ITEM_BOW       28     /* shoots arrows */
#define ITEM_SLING     29     /* shoots rocks */
#define ITEM_CROSSBOW  30     /* shoots bolts */
#define ITEM_BOLT      31
#define ITEM_ARROW     32
#define ITEM_ROCK      33

#define NUM_ITEM_TYPES 33
#define MAX_MISSILE_RANGE  3 /* Max number of rooms you can fire a
										* missile weapon. */

#define MAX_PEOPLE_IN_CHAIR 10   /* The maximum number of people you want *
											 * to sit in a chair at the same time.   */


/* Take/Wear flags: used by obj_data.obj_flags.wear_flags */
struct can_be_worn_on {
	BIT(ITEM_WEAR_TAKE);		/* Item can be takes		*/
	BIT(ITEM_WEAR_FINGER);	/* Can be worn on finger	*/
	BIT(ITEM_WEAR_NECK);		/* Can be worn around neck 	*/
	BIT(ITEM_WEAR_BODY);		/* Can be worn on body 		*/
	BIT(ITEM_WEAR_HEAD);		/* Can be worn on head 		*/
	BIT(ITEM_WEAR_LEGS);		/* Can be worn on legs		*/
	BIT(ITEM_WEAR_FEET);		/* Can be worn on feet		*/
	BIT(ITEM_WEAR_HANDS);		/* Can be worn on hands		*/
	BIT(ITEM_WEAR_ARMS);		/* Can be worn on arms		*/
	BIT(ITEM_WEAR_SHIELD);	/* Can be used as a shield	*/
	BIT(ITEM_WEAR_ABOUT);		/* Can be worn about body 	*/
	BIT(ITEM_WEAR_WAIST);		/* Can be worn around waist 	*/
	BIT(ITEM_WEAR_WRIST);		/* Can be worn on wrist 	*/
	BIT(ITEM_WEAR_WIELD);		/* Can be wielded		*/
	BIT(ITEM_WEAR_HOLD);		/* Can be held			*/
};


/* Extra object flags: used by obj_data.obj_flags.extra_flags */
struct item_is {
	BIT(ITEM_GLOW);		/* Item is glowing		*/
	BIT(ITEM_HUM);		/* Item is humming		*/
	BIT(ITEM_NORENT);		/* Item cannot be rented	*/
	BIT(ITEM_NODONATE);		/* Item cannot be donated	*/
	BIT(ITEM_NOINVIS);		/* Item cannot be made invis	*/
	BIT(ITEM_INVISIBLE);		/* Item is invisible		*/
	BIT(ITEM_MAGIC);		/* Item is magical		*/
	BIT(ITEM_NODROP);		/* Item is cursed: can't drop	*/
	BIT(ITEM_BLESS);		/* Item is blessed		*/
	BIT(ITEM_ANTI_GOOD);		/* Not usable by good people	*/
	BIT(ITEM_ANTI_EVIL);		/* Not usable by evil people	*/
	BIT(ITEM_ANTI_NEUTRAL);	/* Not usable by neutral people	*/
	BIT(ITEM_ANTI_MAGIC_USER);	/* Not usable by mages		*/
	BIT(ITEM_ANTI_CLERIC);	/* Not usable by clerics	*/
	BIT(ITEM_ANTI_THIEF);	/* Not usable by thieves	*/
	BIT(ITEM_ANTI_WARRIOR);	/* Not usable by warriors	*/
	BIT(ITEM_NOSELL);		   /* Shopkeepers won't touch it	*/
	BIT(ITEM_ANTI_HUMAN);   /* Not usable by humans */
	BIT(ITEM_ANTI_ELF);     /* Not usable by elves */
	BIT(ITEM_ANTI_DWARF);   /* Not usable by dwarves */
	BIT(ITEM_ANTI_GNOME);   /* Not usable by gnomes */
	BIT(ITEM_NO_RECHARGE);  /* Item cannot be recharged */
	BIT(ITEM_BURIED);       /* Item has been buried in the ground */
	BIT(ITEM_TWO_HANDED);   /* Item requires two hands to use */
	BIT(ITEM_UNIQUE_SAVE);  /* (XAP) Item has unique text that needs saving */
	BIT(ITEM_LIVE_GRENADE); /* grenade's pin has been pulled */
};


/* Modifier constants used with obj affects ('A' fields) */
#define APPLY_NONE              0	/* No effect			*/
#define APPLY_STR               1	/* Apply to strength		*/
#define APPLY_DEX               2	/* Apply to dexterity		*/
#define APPLY_INT               3	/* Apply to intelligence	*/
#define APPLY_WIS               4	/* Apply to wisdom		*/
#define APPLY_CON               5	/* Apply to constitution	*/
#define APPLY_CHA		6	/* Apply to charisma		*/
#define APPLY_CLASS             7	/* Reserved			*/
#define APPLY_LEVEL             8	/* Reserved			*/
#define APPLY_AGE               9	/* Apply to age			*/
#define APPLY_CHAR_WEIGHT      10	/* Apply to weight		*/
#define APPLY_CHAR_HEIGHT      11	/* Apply to height		*/
#define APPLY_MANA             12	/* Apply to max mana		*/
#define APPLY_HIT              13	/* Apply to max hit points	*/
#define APPLY_MOVE             14	/* Apply to max move points	*/
#define APPLY_GOLD             15	/* Reserved			*/
#define APPLY_EXP              16	/* Reserved			*/
#define APPLY_AC               17	/* Apply to Armor Class		*/
#define APPLY_HITROLL          18	/* Apply to hitroll		*/
#define APPLY_DAMROLL          19	/* Apply to damage roll		*/
#define APPLY_SAVING_PARA      20	/* Apply to save throw: paralz	*/
#define APPLY_SAVING_ROD       21	/* Apply to save throw: rods	*/
#define APPLY_SAVING_PETRI     22	/* Apply to save throw: petrif	*/
#define APPLY_SAVING_BREATH    23	/* Apply to save throw: breath	*/
#define APPLY_SAVING_SPELL     24	/* Apply to save throw: spells	*/
#define APPLY_RACE             25   /* Reserved */
#define APPLY_HITNDAM          26   /* Apply to both hit and dam rolls */
#define APPLY_SAVING_ALL       27   /* Apply to ALL saving throws */
#define APPLY_HIT_REGEN        28
#define APPLY_MANA_REGEN       29
#define APPLY_MOVE_REGEN       30
#define APPLY_PLATINUM         31
#define APPLY_SILVER           32
#define APPLY_COPPER           33

#define NUM_APPLY              34


/* Container flags - value[1] */
#define CONT_CLOSEABLE      (1 << 0)	/* Container can be closed	*/
#define CONT_PICKPROOF      (1 << 1)	/* Container is pickproof	*/
#define CONT_CLOSED         (1 << 2)	/* Container is closed		*/
#define CONT_LOCKED         (1 << 3)	/* Container is locked		*/


/* Some different kind of liquids for use in values of drink containers */
#define LIQ_WATER      0
#define LIQ_BEER       1
#define LIQ_WINE       2
#define LIQ_ALE        3
#define LIQ_DARKALE    4
#define LIQ_WHISKY     5
#define LIQ_LEMONADE   6
#define LIQ_FIREBRT    7
#define LIQ_LOCALSPC   8
#define LIQ_SLIME      9
#define LIQ_MILK       10
#define LIQ_TEA        11
#define LIQ_COFFE      12
#define LIQ_BLOOD      13
#define LIQ_SALTWATER  14
#define LIQ_CLEARWATER 15


/* other miscellaneous defines *******************************************/

/* Edit Modes (Replaces flags and connection states and crap) */
#define EDIT_EXDESC        0
#define EDIT_MAIL          1
#define EDIT_BOARD         2
#define EDIT_NOTE          3

/* Player conditions */
#define DRUNK        0
#define FULL         1
#define THIRST       2


/* Sun state for sunlight */
#define SUN_DARK	0
#define SUN_RISE	1
#define SUN_LIGHT	2
#define SUN_SET		3


/* Sky conditions for weather */
#define SKY_CLOUDLESS	0
#define SKY_CLOUDY	1
#define SKY_RAINING	2
#define SKY_LIGHTNING	3


/* Rent codes */
#define RENT_UNDEF      0
#define RENT_CRASH      1
#define RENT_RENTED     2
#define RENT_CRYO       3
#define RENT_FORCED     4
#define RENT_TIMEDOUT   5


/* other #defined constants **********************************************/

/*
 * **DO**NOT** blindly change the number of levels in your MUD merely by
 * changing these numbers and without changing the rest of the code to match.
 * Other changes throughout the code are required.  See coding.doc for
 * details.
 *
 * LVL_IMPL should always be the HIGHEST possible immortal level, and
 * LVL_IMMORT should always be the LOWEST immortal level.  The number of
 * mortal levels will always be LVL_IMMORT - 1.
 */
#define LVL_IMPL	34
#define LVL_GRGOD	33
#define LVL_GOD		32
#define LVL_IMMORT	31

/* Level of the 'freeze' command */
#define LVL_FREEZE	LVL_GRGOD

#define NUM_OF_DIRS	6	/* number of directions in a room (nsewud) */
#define MAGIC_NUMBER	(0x06)	/* Arbitrary number that won't be in a string */

/*
 * OPT_USEC determines how many commands will be processed by the MUD per
 * second and how frequently it does socket I/O.  A low setting will cause
 * actions to be executed more frequently but will increase overhead due to
 * more cycling to check.  A high setting (e.g. 1 Hz) may upset your players
 * as actions (such as large speedwalking chains) take longer to be executed.
 * You shouldn't need to adjust this.
 */
#define OPT_USEC	100000		/* 10 passes per second */
#define PASSES_PER_SEC	(1000000 / OPT_USEC)
#define RL_SEC		* PASSES_PER_SEC


#define PULSE_ZONE      (10 RL_SEC)
#define PULSE_MOBILE    (10 RL_SEC)
#define PULSE_VIOLENCE  ( 2 RL_SEC)
#define PULSE_EVENT     ( 1 RL_SEC)
#define PULSE_AUTOSAVE	(60 RL_SEC)
#define PULSE_IDLEPWD	(15 RL_SEC)
#define PULSE_SANITY	(30 RL_SEC)
#define PULSE_USAGE	(5 * 60 RL_SEC)	/* 5 mins */
#define PULSE_TIMESAVE	(30 * 60 RL_SEC) /* should be >= SECS_PER_MUD_HOUR */

/* Variables for the output buffering system */
#define MAX_SOCK_BUF            (12 * 1024) /* Size of kernel's sock buf   */
#define MAX_PROMPT_LENGTH       128          /* Max length of prompt        */
#define GARBAGE_SPACE		32          /* Space for **OVERFLOW** etc  */
#define SMALL_BUFSIZE		1024        /* Static output buffer size   */
/* Max amount of output that can be buffered */
#define LARGE_BUFSIZE	   (MAX_SOCK_BUF - GARBAGE_SPACE - MAX_PROMPT_LENGTH)

#define HISTORY_SIZE		5	/* Keep last 5 commands. */
#define MAX_STRING_LENGTH	8192
#define MAX_INPUT_LENGTH	256	/* Max length per *line* of input */
#define MAX_RAW_INPUT_LENGTH	512	/* Max size of *raw* input */
#define MAX_MESSAGES		60
#define MAX_NAME_LENGTH		20  /* Used in char_file_u *DO*NOT*CHANGE* */
/* ** MAX_PWD_LENGTH changed from 10 to 30 for ascii test - Sam ** */
#define MAX_PWD_LENGTH		30  /* Used in char_file_u *DO*NOT*CHANGE* */
#define MAX_TITLE_LENGTH	80  /* Used in char_file_u *DO*NOT*CHANGE* */
#define MAX_POOF_LENGTH    80  /* Used in char_file_u *DO*NOT*CHANGE* */
#define HOST_LENGTH		30  /* Used in char_file_u *DO*NOT*CHANGE* */
#define EXDSCR_LENGTH		240 /* Used in char_file_u *DO*NOT*CHANGE* */
#define MAX_TONGUE		3   /* Used in char_file_u *DO*NOT*CHANGE* */
#define MAX_SKILLS		200 /* Used in char_file_u *DO*NOT*CHANGE* */
#define MAX_AFFECT		32  /* Used in char_file_u *DO*NOT*CHANGE* */
#define MAX_OBJ_AFFECT		6 /* Used in obj_file_elem *DO*NOT*CHANGE* */
#define MAX_NOTE_LENGTH		1000	/* arbitrary */

/*
 * A MAX_PWD_LENGTH of 10 will cause BSD-derived systems with MD5 passwords
 * and GNU libc 2 passwords to be truncated.  On BSD this will enable anyone
 * with a name longer than 5 character to log in with any password.  If you
 * have such a system, it is suggested you change the limit to 20.
 *
 * Please note that this will erase your player files.  If you are not
 * prepared to do so, simply erase these lines but heed the above warning.
 */
#if defined(HAVE_UNSAFE_CRYPT) && MAX_PWD_LENGTH == 10
#error You need to increase MAX_PWD_LENGTH to at least 20.
#error See the comment near these errors for more explanation.
#endif

/**********************************************************************
* Structures                                                          *
**********************************************************************/


typedef signed char		sbyte;
typedef unsigned char		ubyte;
typedef signed short int	sh_int;
typedef unsigned short int	ush_int;
#if !defined(__cplusplus)	/* Anyone know a portable method? */
typedef char			bool;
#endif

#if !defined(CIRCLE_WINDOWS) || defined(LCC_WIN32)	/* Hm, sysdep.h? */
typedef char			byte;
#endif

/* Various virtual (human-reference) number types. */
typedef IDXTYPE room_vnum;
typedef IDXTYPE obj_vnum;
typedef IDXTYPE mob_vnum;
typedef IDXTYPE zone_vnum;
typedef IDXTYPE shop_vnum;

/* Various real (array-reference) number types. */
typedef IDXTYPE room_rnum;
typedef IDXTYPE obj_rnum;
typedef IDXTYPE mob_rnum;
typedef IDXTYPE zone_rnum;
typedef IDXTYPE shop_rnum;


/*
 * Bitvector type for 32 bit unsigned long bitvectors.
 * 'unsigned long long' will give you at least 64 bits if you have GCC.
 * For MSVC you'll have to define it as unsigned __int64 instead. 
 *
 * Since we don't want to break the pfiles, you'll have to search throughout
 * the code for "bitvector_t" and change them yourself if you'd like this
 * extra flexibility.
 */
typedef unsigned long int	bitvector_t;


/* Extra description: used in objects, mobiles, and rooms */
struct extra_descr_data {
   char	*keyword;                 /* Keyword in look/examine          */
   char	*description;             /* What to see                      */
   struct extra_descr_data *next; /* Next in list                     */
};


/* object-related structures ******************************************/


/* object flags; used in obj_data */
struct obj_flag_data {
   int	value[4];	/* Values of the item (see list)    */
   byte type_flag;	/* Type of item			    */
	struct can_be_worn_on wear_flags;	/* Where you can wear it	    */
	struct item_is        extra_flags;	/* If it hums, glows, etc.	    */
   int	weight;		/* Weigt what else                  */
   int	cost;		/* Value when sold (gp.)            */
   int	cost_per_day;	/* Cost to keep pr. real day        */
   int	timer;		/* Timer for object                 */
	struct affections	bitvector;	/* To set chars bits                */
};


/* Used in obj_file_elem *DO*NOT*CHANGE* */
struct obj_affected_type {
   byte location;      /* Which ability to change (APPLY_XXX) */
   sbyte modifier;     /* How much it changes by              */
};


/* Weapon spells - spell gets cast on a successful hit! */
struct weapon_spells {
	int spellnum; /* Number of the spell */
	int level;    /* Level the spell is cast at */
	int percent;  /* Chance the spell goes off on a hit */
};
#define MAX_WEAPON_SPELLS 3  /* Change if desired. */

/* ================== Memory Structure for Objects ================== */
struct obj_data {
   obj_vnum item_number;	/* Where in data-base			*/
   room_rnum in_room;		/* In what room -1 when conta/carr	*/

   struct obj_flag_data obj_flags;/* Object information               */
   struct obj_affected_type affected[MAX_OBJ_AFFECT];  /* affects */

   char	*name;                    /* Title of object :get etc.        */
   char	*description;		  /* When in room                     */
   char	*short_description;       /* when worn/carry/in cont.         */
   char	*action_description;      /* What to write when used          */
   struct extra_descr_data *ex_description; /* extra descriptions     */
   struct char_data *carried_by;  /* Carried by :NULL in room/conta   */
   struct char_data *worn_by;	  /* Worn by?			      */
	struct char_data *sitting_here;   /* Who is sitting in it (null if none) */
   sh_int worn_on;		  /* Worn where?		      */

	int   has_spells;
	struct weapon_spells wpn_spells[MAX_WEAPON_SPELLS];

   struct obj_data *in_obj;       /* In what object; NULL when none   */
   struct obj_data *contains;     /* Contains objects                 */

   struct obj_data *next_content; /* For 'contains' lists             */
   struct obj_data *next;         /* For the object list              */
};
/* ======================================================================= */


/* ====================== File Element for Objects ======================= */
/*                 BEWARE: Changing it will ruin rent files		   */
struct obj_file_elem {
   obj_vnum item_number;

#if USE_AUTOEQ
   sh_int location;
#endif
   int	value[4];
	struct item_is extra_flags;
   int	weight;
   int	timer;
	struct affections	bitvector;
   struct obj_affected_type affected[MAX_OBJ_AFFECT];
};


/* header block for rent files.  BEWARE: Changing it will ruin rent files  */
struct rent_info {
   int	time;
   int	rentcode;
   int	net_cost_per_diem;
   int	gold;
   int	account;
   int	nitems;
   int	plat;
   int	silver;
   int	copper;
   int	spare3;
   int	spare4;
   int	spare5;
   int	spare6;
   int	spare7;
};
/* ======================================================================= */


/* room-related structures ************************************************/


struct room_direction_data {
   char	*general_description;       /* When look DIR.			*/

   char	*keyword;		/* for open/close			*/

	struct exit_info_types exit_info;/* Exit info			        */
   obj_vnum key;		/* Key's number (-1 for no key)		*/
   room_rnum to_room;		/* Where direction leads (NOWHERE)	*/
};


struct raff_node {
	room_rnum room;        /* location in the world[] array of the room */
	int      timer;       /* how many ticks this affection lasts */
	struct room_affs affection;   /* which affection does this room have */
	int      spell;       /* the spell number */

	struct raff_node *next; /* link to the next node */
};

/* ================== Memory Structure for room ======================= */
struct room_data {
   room_vnum number;		/* Rooms number	(vnum)		      */
   zone_rnum zone;              /* Room zone (for resetting)          */
   int	sector_type;            /* sector type (move/hide)            */
   char	*name;                  /* Rooms name 'You are ...'           */
   char	*description;           /* Shown when entered                 */
   struct extra_descr_data *ex_description; /* for examine/look       */
   struct room_direction_data *dir_option[NUM_OF_DIRS]; /* Directions */
	struct room_flags room_flags;	/* DEATH,DARK ... etc */
	struct room_affs room_affections;    /* bitvector for spells/skills */

	byte blood;      /*  How many mobs have been killed here recently? */
   byte light;                  /* Number of lightsources in room     */
   SPECIAL(*func);
	int specproc;

   struct obj_data *contents;   /* List of items in room              */
   struct char_data *people;    /* List of NPC / PC in room           */
};
/* ====================================================================== */


/* char-related structures ************************************************/


/* MOBProgram foo */
struct mob_prog_act_list {
   struct mob_prog_act_list *next;
   char *buf;
   struct char_data *ch;
   struct obj_data *obj;
   void *vo;
};
typedef struct mob_prog_act_list MPROG_ACT_LIST;
 

struct mob_prog_data {
   struct mob_prog_data *next;
   int type;
   char *arglist;
   char *comlist;
};
typedef struct mob_prog_data MPROG_DATA;
 
extern int MOBTrigger;
 
#define ERROR_PROG            -1
#define IN_FILE_PROG           0
#define ACT_PROG         (1 << 0)
#define SPEECH_PROG      (1 << 1)
#define RAND_PROG        (1 << 2)
#define FIGHT_PROG       (1 << 3)
#define DEATH_PROG       (1 << 4)
#define HITPRCNT_PROG    (1 << 5)
#define ENTRY_PROG       (1 << 6)
#define GREET_PROG       (1 << 7)
#define ALL_GREET_PROG   (1 << 8)
#define GIVE_PROG        (1 << 9)
#define BRIBE_PROG      (1 << 10)
#define TALK_PROG       (1 << 11)
 
/* end of MOBProg foo */
 


/* memory structure for characters */
struct memory_rec_struct {
   long	id;
   struct memory_rec_struct *next;
};

typedef struct memory_rec_struct memory_rec;


/* This structure is purely intended to be an easy way to transfer */
/* and return information about time (real or mudwise).            */
struct time_info_data {
   int hours, day, month;
   sh_int year;
};


/* These data contain information about a players time data */
struct time_data {
   time_t birth;    /* This represents the characters age                */
   time_t logon;    /* Time of the last logon (used to calculate played) */
   int	played;     /* This is the total accumulated time played in secs */
};


/* The pclean_criteria_data is set up in config.c and used in db.c to
 * determine the conditions which will cause a player character to be
 * deleted from disk if the automagic pwipe system is enabled (see
 * config.c).
*/
struct pclean_criteria_data {
  int level;		/* max level for this time limit	*/
  int days;		/* time limit in days			*/
}; 


/* general player-related info, usually PC's and NPC's */
struct char_player_data {
   char	passwd[MAX_PWD_LENGTH+1]; /* character's password      */
   char	*name;	       /* PC / NPC s name (kill ...  )         */
   char	*short_descr;  /* for NPC 'actions'                    */
   char	*long_descr;   /* for 'look'			       */
   char	*description;  /* Extra descriptions                   */
   char	*title;        /* PC / NPC's title                     */
   byte sex;           /* PC / NPC's sex                       */
   byte chclass;       /* PC / NPC's class		       */
	byte race;          /* PC / NPC's race                      */
   byte level;         /* PC / NPC's level                     */
   sh_int hometown;    /* PC s Hometown (zone)                 */
   struct time_data time;  /* PC's AGE in days                 */
   ubyte weight;       /* PC / NPC's weight                    */
   ubyte height;       /* PC / NPC's height                    */
};


/* Char's abilities.  Used in char_file_u *DO*NOT*CHANGE* */
struct char_ability_data {
   sbyte str;
   sbyte str_add;      /* 000 - 100 if strength 18             */
   sbyte intel;
   sbyte wis;
   sbyte dex;
   sbyte con;
   sbyte cha;
};


/* Char's points.  Used in char_file_u *DO*NOT*CHANGE* */
struct char_point_data {
   sh_int mana;
   sh_int max_mana;     /* Max mana for PC/NPC			   */
   sh_int hit;
   sh_int max_hit;      /* Max hit for PC/NPC                      */
   sh_int move;
   sh_int max_move;     /* Max move for PC/NPC                     */

   sh_int armor;         /* Internal -100..100, external -10..10 AC */
   int   money[4];    /* Four coin types: Platinum, Gold, Silver, Copper */
   int	bank_gold;	/* Gold the char has in a bank account	   */
   int   bank_shares;    /* Shares char has in the bank             */
   int	exp;            /* The experience of the player            */

   sbyte hitroll;       /* Any bonus or penalty to the hit roll    */
   sbyte damroll;       /* Any bonus or penalty to the damage roll */
};


/* 
 * char_special_data_saved: specials which both a PC and an NPC have in
 * common, but which must be saved to the playerfile for PC's.
 *
 * WARNING:  Do not change this structure.  Doing so will ruin the
 * playerfile.  If you want to add to the playerfile, use the spares
 * in player_special_data.
 */
struct char_special_data_saved {
   int	alignment;		/* +-1000 for alignments                */
   long	idnum;			/* player's idnum; -1 for mobiles	*/
	union characterflags {
		struct playerflags	pc;
		struct mobflags	npc;
	} act;			/* act flag for NPC's; player flag for PC's */

	struct affections	affected_by;

   sh_int apply_saving_throw[5]; /* Saving throw (Bonuses)		*/
};


/* Event queues */

/* There are two event queues available for your use - one of them is
 * much simpler then the other.  If you don't understand how either
 * one works, don't use either....  If you do, pick and choose based
 * on what you need, or just get rid of one or the other if you don't
 * like having two of them around.  -ejg */

struct simple_event_data {
   struct char_data *ch;            /* char this event happens to */
   int cmd;                         /* command number             */
   char *args;                      /* typed in args              */
   int count;                       /* pulses to event completion */
   struct simple_event_data *next;  /* next command event in list */
};


#define EVENT(name) void (name)(void *causer, void *victim, long info)

struct complex_event_data {
  int ticks_to_go;
  EVENT(*func);
  void *causer,*victim;
  void *info;
  struct complex_event_data *next;
};


/* Special playing constants shared by PCs and NPCs which aren't in pfile */
struct char_special_data {
   struct char_data *fighting;  /* Opponent                      */
   struct char_data *hunting;   /* Char hunted by this char      */
	struct simple_event_data *my_event; /* What event do I have pending? */
	struct char_data *riding;    /* Who are they riding? (DAK)    */
	struct char_data *ridden_by; /* Who is riding them? (DAK)     */

   byte position;		/* Standing, fighting, sleeping, etc.	*/

   int	carry_weight;		/* Carried weight			*/
   byte carry_items;		/* Number of items carried		*/
   int	timer;			/* Timer for update			*/

	sbyte hitgain;
	sbyte managain;
	sbyte movegain;

	struct obj_data *chair;          /* Object the char is sitting in */
	struct char_data *next_in_chair; /* The next person in the chair */

   struct char_special_data_saved saved; /* constants saved in plrfile	*/
};


/*
 *  If you want to add new values to the playerfile, do it here.  DO NOT
 * ADD, DELETE OR MOVE ANY OF THE VARIABLES - doing so will change the
 * size of the structure and ruin the playerfile.  However, you can change
 * the names of the spares to something more meaningful, and then use them
 * in your new code.  They will automatically be transferred from the
 * playerfile into memory when players log in.
 */
struct player_special_data_saved {
   byte skills[MAX_SKILLS+1];   /* array of skills plus skill 0         */
   byte PADDING0;               /* used to be spells_to_learn           */
   bool talks[MAX_TONGUE];      /* PC s Tongues 0 for NPC               */
   int	wimp_level;            /* Below this # of hit points, flee!    */
   byte freeze_level;           /* Level of god who froze char, if any  */
   sh_int invis_level;          /* level of invisibility                */
   room_vnum load_room;         /* Which room to place char in          */
	struct preferances pref;	/* preference flags for PC's.		*/
   ubyte bad_pws;               /* number of bad password attemps       */
   sbyte conditions[3];         /* Drunk, full, thirsty                 */

   /* spares below for future expansion.  You can change the names from
      'sparen' to something meaningful, but don't change the order.  */

   ubyte spare0;
   ubyte spare1;
   ubyte spare2;
   ubyte spare3;
   ubyte spare4;
   ubyte spare5;
   int spells_to_learn;		/* How many can you learn yet this level*/
   int last_command;
   int num_sacs;
   int plat;
   int silver;
   int copper;
   int gold;
   int spare13;
   int spare14;
   int spare15;
   int spare16;
   long	spare17;
   long	spare18;
   long	spare19;
   long	spare20;
   long	spare21;
};

/*
 * Specials needed only by PCs, not NPCs.  Space for this structure is
 * not allocated in memory for NPCs, but it is for PCs and the portion
 * of it labelled 'saved' is saved in the playerfile.  This structure can
 * be changed freely; beware, though, that changing the contents of
 * player_special_data_saved will corrupt the playerfile.
 */
struct player_special_data {
   struct player_special_data_saved saved;

	char *contact;               /* name of immort contact               */
   char *poofin;		/* Description on arrival of a god.     */
   char *poofout;		/* Description upon a god's exit.       */
   struct alias_data *aliases;	/* Character's aliases			*/
   long last_tell;		/* idnum of last tell from		*/
   void *last_olc_targ;		/* olc control				*/
   int last_olc_mode;		/* olc control				*/
	int private;            /* private channel      */
   char *host;			/* player host				*/
};


/* Specials used by NPCs, not PCs */
struct mob_special_data {
   memory_rec *memory;	    /* List of attackers to remember	       */
   byte	attack_type;        /* The Attack Type Bitvector for NPC's     */
   byte default_pos;        /* Default position for NPC                */
   byte damnodice;          /* The number of damage dice's	       */
   byte damsizedice;        /* The size of the damage dice's           */
	int newitem;             /* Check if mob has new inv item       */
	byte num_attacks;        /* How many attacks can this mob make per round? */
};


/* An affect structure.  Used in char_file_u *DO*NOT*CHANGE* */
struct affected_type {
   sh_int type;          /* The type of spell that caused this      */
   sh_int duration;      /* For how long its effects will last      */
   sbyte modifier;       /* This is added to apropriate ability     */
   byte location;        /* Tells which ability to change(APPLY_XXX)*/
	struct affections bitvector; /* Tells which bits to set (AFF_XXX) */

   struct affected_type *next;
};


/* Structure used for chars following other chars */
struct follow_type {
   struct char_data *follower;
   struct follow_type *next;
};


/* ================== Structure for player/non-player ===================== */
struct char_data {
   int pfilepos;			 /* playerfile pos		  */
   mob_rnum nr;                          /* Mob's rnum			  */
   room_rnum in_room;                    /* Location (real room number)	  */
   room_rnum was_in_room;		 /* location for linkdead people  */
   int wait;				 /* wait for how many loops	  */

   struct char_player_data player;       /* Normal data                   */
   struct char_ability_data real_abils;	 /* Abilities without modifiers   */
   struct char_ability_data aff_abils;	 /* Abils with spells/stones/etc  */
   struct char_point_data points;        /* Points                        */
   struct char_special_data char_specials;	/* PC/NPC specials	  */
   struct player_special_data *player_specials; /* PC specials		  */
   struct mob_special_data mob_specials;	/* NPC specials		  */

   struct affected_type *affected;       /* affected by what spells       */
   struct obj_data *equipment[NUM_WEARS];/* Equipment array               */

   struct obj_data *carrying;            /* Head of list                  */
   struct descriptor_data *desc;         /* NULL for mobiles              */

   struct char_data *next_in_room;     /* For room->people - list         */
   struct char_data *next;             /* For either monster or ppl-list  */
   struct char_data *next_fighting;    /* For fighting list               */

   struct follow_type *followers;        /* List of chars followers       */
   struct char_data *master;             /* Who is char following?        */

	/* MOBprog foo */
	MPROG_ACT_LIST *mpact;
	int mpactnum;
};
/* ====================================================================== */


/* ==================== File Structure for Player ======================= */
/*             BEWARE: Changing it will ruin the playerfile		  */
struct char_file_u {
   /* char_player_data */
   char	name[MAX_NAME_LENGTH+1];
   char	description[EXDSCR_LENGTH];
   char	title[MAX_TITLE_LENGTH+1];
	char  poofin[MAX_POOF_LENGTH+1];
	char  poofout[MAX_POOF_LENGTH+1];
	char  contact[MAX_NAME_LENGTH+1];
   byte sex;
   byte chclass;
	byte race;
   byte level;
   sh_int hometown;
   time_t birth;   /* Time of birth of character     */
   int	played;    /* Number of secs played in total */
   ubyte weight;
   ubyte height;

   char	pwd[MAX_PWD_LENGTH+1];    /* character's password */

   struct char_special_data_saved char_specials_saved;
   struct player_special_data_saved player_specials_saved;
   struct char_ability_data abilities;
   struct char_point_data points;
   struct affected_type affected[MAX_AFFECT];

   time_t last_logon;		/* Time (in secs) of last logon */
   char host[HOST_LENGTH+1];	/* host of last logon */
};
/* ====================================================================== */


/* descriptor-related structures ******************************************/


/* Data allocated only when editing text */
struct text_edit_data {
   char **str;              /* Final destination of string being modified */
   char *smod;              /* string being modified */
   size_t max_str;          /* Maximum length of string */
   long  mail_to;           /* name for mail system       */
	obj_vnum mail_vnum;		 /* virtual number of obj you're mailing */

   int mode;                /* Editor Mode */
};

struct txt_block {
   char	*text;
   int aliased;
   struct txt_block *next;
};


struct txt_q {
   struct txt_block *head;
   struct txt_block *tail;
};


struct descriptor_data {
   socket_t	descriptor;	/* file descriptor for socket		*/
   socket_t ident_sock;		/* socket used for ident process        */
   u_short  peer_port;		/* port of peer               		*/
   char	host[HOST_LENGTH+1];	/* hostname				*/
   byte	bad_pws;		/* number of bad pw attemps this login	*/
   byte idle_tics;		/* tics idle at password prompt		*/
   int	connected;		/* mode of 'connectedness'		*/
   int	desc_num;		/* unique num assigned to desc		*/
   time_t login_time;		/* when the person connected		*/
   char *showstr_head;		/* for keeping track of an internal str	*/
   char **showstr_vector;	/* for paging through texts		*/
   int  showstr_count;		/* number of pages to page through	*/
   int  showstr_page;		/* which page are we currently showing?	*/
   struct text_edit_data *textedit; /* data used by text editor */
   int	has_prompt;		/* is the user at a prompt?             */
   char	inbuf[MAX_RAW_INPUT_LENGTH];  /* buffer for raw input		*/
   char	last_input[MAX_INPUT_LENGTH]; /* the last input			*/
   char small_outbuf[SMALL_BUFSIZE];  /* standard output buffer		*/
   char *output;		/* ptr to the current output buffer	*/
   char **history;		/* History of commands, for ! mostly.	*/
   int	history_pos;		/* Circular array position.		*/
   int  bufptr;			/* ptr to end of current output		*/
   int	bufspace;		/* space left in the output buffer	*/
   struct txt_block *large_outbuf; /* ptr to large buffer, if we need it */
   struct txt_q input;		/* q of unprocessed input		*/
   struct char_data *character;	/* linked to char			*/
   struct char_data *original;	/* original char if switched		*/
   struct descriptor_data *snooping; /* Who is this char snooping	*/
   struct descriptor_data *snoop_by; /* And who is snooping this char	*/
   struct descriptor_data *next; /* link to next descriptor		*/

   void *olc;

	C_FUNC(*callback);   /* Call back function for anything I want to do */
	int callback_depth;  /* Call back "psuedo-recursiveness" depth       */
	void *c_data;        /* Storage for the Callback function            */
};


/* other miscellaneous structures ***************************************/

struct msg_type {
   char	*attacker_msg;  /* message to attacker */
   char	*victim_msg;    /* message to victim   */
   char	*room_msg;      /* message to room     */
};


struct message_type {
   struct msg_type die_msg;	/* messages when death			*/
   struct msg_type miss_msg;	/* messages when miss			*/
   struct msg_type hit_msg;	/* messages when hit			*/
   struct msg_type god_msg;	/* messages when hit on god		*/
   struct message_type *next;	/* to next messages of this kind.	*/
};


struct message_list {
   int	a_type;			/* Attack type				*/
   int	number_of_attacks;	/* How many attack messages to chose from. */
   struct message_type *msg;	/* List of messages.			*/
};


struct dex_skill_type {
   sh_int p_pocket;
   sh_int p_locks;
   sh_int traps;
   sh_int sneak;
   sh_int hide;
	sh_int listen;
};


struct dex_app_type {
   sh_int reaction;
   sh_int miss_att;
   sh_int defensive;
};


struct str_app_type {
   sh_int tohit;    /* To Hit (THAC0) Bonus/Penalty        */
   sh_int todam;    /* Damage Bonus/Penalty                */
   sh_int carry_w;  /* Maximum weight that can be carrried */
   sh_int wield_w;  /* Maximum weight that can be wielded  */
};


struct wis_app_type {
   byte bonus;       /* how many practices player gains per lev */
};


struct int_app_type {
   byte learn;       /* how many % a player learns a spell/skill */
};


struct con_app_type {
   sh_int hitp;
   sh_int shock;
	sh_int poison;
};


struct cha_app_type {
	sh_int num_followers;
	sh_int loyalty;
	sh_int reaction;
};


/*
 * Element in monster and object index-tables.
 *
 * NOTE: Assumes sizeof(mob_vnum) >= sizeof(obj_vnum)
 */
struct index_data {
   mob_vnum	vnum;	/* virtual number of this mob/obj		*/
   int		number;	/* number of existing units of this mob/obj	*/
   SPECIAL(*func);
	int specproc;

	int  progtypes;  /* program types for MOBProg                */
	MPROG_DATA *mobprogs;  /* programs for MOBProg                */
};

struct guild_info_type {
  int pc_class;
  room_vnum guild_room;
  int direction;
};
